@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2010 Luca Deri <deri@ntop.org>
  *
  *  			    http://www.ntop.org/
  *
@@ -40,6 +40,24 @@ static void updateHostNameInfo(HostAddr addr, char* symbolic, int type);
 /* #define DNS_DEBUG */
 /* #define MDNS_DEBUG  */
 
+typedef struct hostAddrList {
+  HostAddr addr;
+  struct hostAddrList *next;
+} HostAddrList;
+
+static HostAddrList *hostAddrList_head = NULL;
+static HostAddr lastResolvedAddr[MAX_NUM_DEQUEUE_ADDRESS_THREADS];
+static u_short lastResolvedAddrIdx;
+
+
+/* **************************************** */
+
+void initAddressResolution(void) {
+  memset(&lastResolvedAddr, 0, sizeof(lastResolvedAddr));
+  lastResolvedAddrIdx = 0;
+  createCondvar(&myGlobals.queueAddressCondvar);
+}
+
 /* **************************************** */
 
 static void updateDeviceHostNameInfo(HostAddr addr, char* symbolic, int actualDeviceId, int type) {
@@ -58,11 +76,11 @@ static void updateDeviceHostNameInfo(HostAddr addr, char* symbolic, int actualDe
 
 	if(strlen(symbolic) >= (MAX_LEN_SYM_HOST_NAME-1))
 	  symbolic[MAX_LEN_SYM_HOST_NAME-2] = '\0';
-	
+
 	/* Really needed ? */
 	for(i=0; i<strlen(symbolic); i++) symbolic[i] = tolower(symbolic[i]);
 
-	/* traceEvent(CONST_TRACE_INFO, "--> %s", symbolic); */
+	/* traceEvent(CONST_TRACE_INFO, "[%s] --> %s", el->hostResolvedName, symbolic); */
 
 	setResolvedName(el, symbolic, type);
       }
@@ -77,6 +95,10 @@ static void updateDeviceHostNameInfo(HostAddr addr, char* symbolic, int actualDe
 static void updateHostNameInfo(HostAddr addr, char* symbolic, int type) {
   int i;
 
+#ifdef DEBUG
+  traceEvent(CONST_TRACE_INFO, "updateDeviceHostNameInfo(%s <--> %s)", symbolic, addrtostr(&addr));
+#endif
+
   for(i=0; i<myGlobals.numDevices; i++) {
     if(!myGlobals.device[i].virtualDevice)
       updateDeviceHostNameInfo(addr, symbolic, i, type);
@@ -85,175 +107,61 @@ static void updateHostNameInfo(HostAddr addr, char* symbolic, int type) {
 
 /* ************************************ */
 
-static int validDNSChar(char c) {
-  if((c == '-') || (c == '_') || (c == '.')) return(1);
-  if((c >= '0') && (c <= '9')) return(1);
-  if((c >= 'a') && (c <= 'z')) return(1);
-  if((c >= 'A') && (c <= 'Z')) return(1);
+static void queueAddress(HostAddr elem) {
+  int i;
+  HostAddrList *cloned = NULL;
 
-  return(0);
-}
-
-/* ************************************ */
-
-static int validDNSName(char *name) {
-  int i, len;
-
-  if(name == NULL)
-    return(0);
-  else
-    len = strlen(name);
-
-  for(i=0; i<len; i++)
-    if(!validDNSChar(name[i]))
-      return(0);
-
-  return(1);
-}
-
-/* ************************************ */
-
-static void dns_response_callback(int result, char type, int count, int ttl, void *addresses, void *arg) {
-  HostAddr *addr = (HostAddr*)arg;
-  char *symAddr = NULL;
-  char buf[255];
-
-  //traceEvent(CONST_TRACE_WARNING, "dns_response_callback(): result=%d", result);
-
-  if((addr == NULL) || (result != DNS_ERR_NONE))
-    goto out;
-
-  // traceEvent(CONST_TRACE_INFO, "type: %d, count: %d, ttl: %d: ", type, count, ttl);
-
-  switch (type) {
-  case DNS_IPv6_AAAA: {
-#ifdef INET6
-    struct in6_addr *in6_addrs = addresses;
-    int i;
-
-    /* a resolution that's not valid does not help */
-    if (ttl < 0)
-      goto out;
-    for (i = 0; i < count; ++i) {
-      const char *b = inet_ntop(AF_INET6, &in6_addrs[i], buf,sizeof(buf));
-
-      if(b) {
-	/* traceEvent(CONST_TRACE_INFO, "%s ", b); */
-	symAddr = b;
-      } else
-	traceEvent(CONST_TRACE_INFO, "%s ", strerror(errno));
-    }
-#endif
-    // FIX - ADD IPv6 storage
-    break;
+  if(myGlobals.runningPref.numericFlag == noDnsResolution) return;
+  if(_pseudoLocalAddress(&elem, NULL, NULL)) {
+    /* Local Host */
+    if(myGlobals.runningPref.trackOnlyLocalHosts) return;
+    else if(myGlobals.runningPref.numericFlag == dnsResolutionForLocalRemoteOnly) return;
+  } else {
+    /* Remote Host */
+    if(myGlobals.runningPref.numericFlag == dnsResolutionForLocalHostsOnly) return;
   }
-  case DNS_IPv4_A: {
-    struct in_addr *in_addrs = addresses;
-    int i;
-
-    /* a resolution that's not valid does not help */
-    if (ttl < 0)
-      goto out;
-
-    for (i = 0; i < count; ++i) {
-      symAddr = inet_ntoa(in_addrs[i]);
-      /* traceEvent(CONST_TRACE_INFO, "%s ", rsp); */
-    }
-    break;
-  }
-  case DNS_PTR:
-    /* may get at most one PTR */
-    if (count != 1)
-      goto out;
-
-    symAddr = *(char **)addresses;
-    break;
-  default:
-    goto out;
-  }
-
-  if(symAddr) {
-    StoredAddress storedAddreslones;
-    int len;
-    char keyBuf[LEN_ADDRESS_BUFFER];
-    datum key_data;
-    datum data_data;
-    StoredAddress storedAddress;
-
-    memset(storedAddress.symAddress, 0, sizeof(storedAddress.symAddress));
-    len = min(sizeof(storedAddress.symAddress)-1, strlen(symAddr));
-    memcpy(storedAddress.symAddress, symAddr, len);
-    storedAddress.symAddress[len] = '\0';
-    storedAddress.recordCreationTime = myGlobals.actTime;
-    storedAddress.symAddressType = FLAG_HOST_SYM_ADDR_TYPE_NAME;
-
-    key_data.dptr = _addrtonum(addr, keyBuf, sizeof(keyBuf));
-    key_data.dsize = strlen(keyBuf)+1;
-       
-    data_data.dptr = (void*)&storedAddress;
-    data_data.dsize = sizeof(storedAddress) /* Remember, StoredAddress has 1 byte padding so don't add more here */;
-
-    if(gdbm_store(myGlobals.dnsCacheFile, key_data, data_data, GDBM_REPLACE) != 0)
-      traceEvent(CONST_TRACE_ERROR, "dnsCache error adding '%s', %s", symAddr,
-#if defined(WIN32) && defined(__GNUC__)
-		 "no additional information available"
-#else
-		 gdbm_strerror(gdbm_errno)
-#endif
-		 );
-      
-#ifdef DNS_DEBUG
-    traceEvent(CONST_TRACE_INFO, "DNS_DEBUG: Added data: '%s'='%s'(%d)",
-	       key_data.dptr,
-	       ((StoredAddress*)data_data.dptr)->symAddress,
-	       ((StoredAddress*)data_data.dptr)->symAddressType);
-#endif
-
-    updateHostNameInfo(*addr, storedAddress.symAddress, storedAddress.symAddressType);
-  }
-
- out:
-  if(addr) free(addr);
-  event_loopexit(NULL);
-}
-
-/* *************************** */
-
-static void queueAddress(HostAddr elem, int forceResolution) {
-  datum key_data, data_data;
-  char dataBuf[sizeof(StoredAddress)+4];
-  int rc;
-  HostAddr *cloned = (HostAddr*)malloc(sizeof(HostAddr));
-
-  if(myGlobals.runningPref.numericFlag
-     || (cloned == NULL)
-     || ((!forceResolution)
-	 && myGlobals.runningPref.trackOnlyLocalHosts
-	 && (!_pseudoLocalAddress(&elem, NULL, NULL))))
-    return;
-
-  memcpy(cloned, &elem, sizeof(HostAddr));
 
   accessAddrResMutex("queueAddress");
 
-  if (elem.hostFamily == AF_INET) {
-    struct in_addr in;
-
-    in.s_addr = htonl(elem.Ip4Address.s_addr);
-    if((rc = evdns_resolve_reverse(&in, 0, dns_response_callback, cloned)) != DNS_ERR_NONE) {
-      traceEvent(CONST_TRACE_ERROR, "evdns_resolve_reverse() returned %d", rc);
-    }
-  }
-#ifdef INET6
-  else if (elem.hostFamily == AF_INET6) {
-    if((rc = evdns_resolve_reverse_ipv6(&elem.Ip6Address, 0, dns_response_callback, cloned)) != DNS_ERR_NONE) {
-      traceEvent(CONST_TRACE_ERROR, "evdns_resolve_reverse_ipv6() returned %d", rc);
-    }
-  }
+  for(i=0; i<myGlobals.numDequeueAddressThreads; i++) {
+    if(memcmp(&lastResolvedAddr[i], &elem, sizeof(HostAddr)) == 0) {
+#ifdef DEBUG
+      traceEvent(CONST_TRACE_ERROR, "queueAddress(%s): already been resolved",
+		 addrtostr(&elem));
 #endif
-  else {
-    traceEvent(CONST_TRACE_WARNING, "Invalid address family (%d) found!",
-	       elem.hostFamily);
+      releaseAddrResMutex();
+      return;
+    }
+  }
+
+  if(myGlobals.addressQueuedCurrent > 16384) {
+    myGlobals.addressUnresolvedDrops++;
+  } else {
+    /* First check if the address we want to resolve is already in queue */
+    HostAddrList *head = hostAddrList_head;
+
+    while(head != NULL) {
+      if(memcmp(&head->addr, &elem, sizeof(elem)) == 0) {
+	releaseAddrResMutex();
+	return;
+      }
+
+      head = head->next;
+    }
+
+#ifdef DEBUG
+    traceEvent(CONST_TRACE_ERROR, "queueAddress(%s)", addrtostr(&elem));
+#endif
+
+    if((cloned = (HostAddrList*)malloc(sizeof(HostAddrList))) != NULL) {
+      memcpy(&cloned->addr, &elem, sizeof(HostAddr));
+      cloned->next = hostAddrList_head;
+      hostAddrList_head = cloned;
+      signalCondvar(&myGlobals.queueAddressCondvar);
+      myGlobals.addressQueuedCurrent++;
+      if(myGlobals.addressQueuedCurrent > myGlobals.addressQueuedMax)
+	myGlobals.addressQueuedMax = myGlobals.addressQueuedCurrent;
+    }
   }
 
   releaseAddrResMutex();
@@ -261,6 +169,115 @@ static void queueAddress(HostAddr elem, int forceResolution) {
 
 /* ************************************ */
 
+void* dequeueAddress(void *_i) {
+  int dqaIndex = (int)((long)_i);
+
+  traceEvent(CONST_TRACE_INFO,
+	     "THREADMGMT[t%lu]: DNSAR(%d): Address resolution thread running",
+             (long unsigned int)pthread_self(), dqaIndex+1);
+
+  while(myGlobals.ntopRunState <= FLAG_NTOPSTATE_RUN) {
+#ifdef DEBUG
+    traceEvent(CONST_TRACE_INFO, "DEBUG: Waiting for address to resolve...");
+#endif
+
+    while(hostAddrList_head == NULL) {
+      if(myGlobals.ntopRunState > FLAG_NTOPSTATE_RUN) break;
+      waitCondvar(&myGlobals.queueAddressCondvar);
+    }
+
+#ifdef DEBUG
+    traceEvent(CONST_TRACE_INFO, "DEBUG: Address resolution started...");
+#endif
+
+    while(hostAddrList_head != NULL) {
+      HostAddrList *elem;
+
+      if(myGlobals.ntopRunState > FLAG_NTOPSTATE_RUN) break;
+
+      accessAddrResMutex("dequeueAddress");
+
+      if(hostAddrList_head != NULL) {
+	elem = hostAddrList_head;
+	hostAddrList_head = hostAddrList_head->next;
+	if(myGlobals.addressQueuedCurrent > 0) myGlobals.addressQueuedCurrent--;
+	memcpy(&lastResolvedAddr[lastResolvedAddrIdx], &elem->addr, sizeof(HostAddr));
+	lastResolvedAddrIdx = (lastResolvedAddrIdx + 1) % myGlobals.numDequeueAddressThreads;
+      } else
+	elem = NULL;
+
+      releaseAddrResMutex();
+
+      if(elem) {
+	struct hostent *he = NULL;
+	int family, size;
+	char theAddr[64];
+#if defined(HAVE_GETHOSTBYADDR_R)
+	struct hostent _hp, *__hp;
+	char buffer[4096]; /* It MUST be 4096 as otherwise on Linux will fail */
+#endif
+
+	memset(theAddr, 0, sizeof(theAddr));
+	addrget(&elem->addr, theAddr, &family, &size);
+
+#if defined(HAVE_GETHOSTBYADDR_R)
+#ifdef SOLARIS
+	he = gethostbyaddr_r((const char*)theAddr, size,
+			     family, &_hp,
+			     buffer, sizeof(buffer),
+			     &h_errno);
+#else
+#if 0
+	traceEvent(CONST_TRACE_INFO, "About to resolve %s [family=%d][size=%d]", addrtostr(&elem->addr), family, size);
+#endif
+	if(gethostbyaddr_r((const char*)theAddr, size,
+			   family, &_hp,
+			   buffer, sizeof(buffer),
+			   &__hp, &h_errno) == 0) {
+
+	  if(h_errno == 0)
+	    he = &_hp;
+	  else
+	    he = NULL;
+#endif
+	} else
+	  he = NULL;
+#else
+	he = gethostbyaddr(theAddr, size, family);
+#endif
+
+	if((he != NULL) && (he->h_name != NULL)) {
+	  updateHostNameInfo(elem->addr, he->h_name, FLAG_HOST_SYM_ADDR_TYPE_NAME);
+	  accessAddrResMutex("dequeueAddress"); myGlobals.resolvedAddresses++; releaseAddrResMutex();
+	} else {
+#if 0
+	  traceEvent(CONST_TRACE_ERROR, "Address resolution failure [%d][%s]", h_errno, hstrerror(h_errno));
+#endif
+	  accessAddrResMutex("dequeueAddress"); myGlobals.failedResolvedAddresses++; releaseAddrResMutex();
+	}
+
+	memset(elem, 0, sizeof(HostAddr));
+	free(elem);
+      }
+    } /* while */
+  } /* endless loop */
+
+  myGlobals.dequeueAddressThreadId[dqaIndex] = 0;
+
+  traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: DNSAR(%d): Address resolution thread terminated [p%d]",
+             (long unsigned int)pthread_self(), dqaIndex+1,
+#ifndef WIN32
+			 getpid()
+#else
+			 0
+#endif
+			 );
+
+  return(NULL);
+}
+
+/* ************************************ */
+
 void cleanupAddressQueue(void) {
   /* Nothing to do */
 }
@@ -363,7 +380,6 @@ char * _addrtostr(HostAddr *addr, char* buf, u_short bufLen) {
 /* ************************************ */
 
 char * _addrtonum(HostAddr *addr, char* buf, u_short bufLen) {
-
   if((addr == NULL) || (buf == NULL))
     return NULL;
 
@@ -386,113 +402,38 @@ char * _addrtonum(HostAddr *addr, char* buf, u_short bufLen) {
 
 /* ******************************* */
 
-int fetchAddressFromCache(HostAddr hostIpAddress, char *buffer, int *type) {
-  char keyBuf[LEN_ADDRESS_BUFFER];
-  datum key_data;
-  datum data_data;
-
-  if(buffer == NULL) return(0);
-
-  memset(&keyBuf, 0, sizeof(keyBuf));
-
-    if(addrfull(&hostIpAddress) || addrnull(&hostIpAddress)) {
-    strcpy(buffer, "0.0.0.0");
-    *type = FLAG_HOST_SYM_ADDR_TYPE_IP;
-    return(0);
-  }
-
-  key_data.dptr = _addrtonum(&hostIpAddress,keyBuf, sizeof(keyBuf));
-  key_data.dsize = strlen(key_data.dptr)+1;
-
-  if(myGlobals.dnsCacheFile == NULL) return(0); /* ntop is quitting... */
-
-  data_data = gdbm_fetch(myGlobals.dnsCacheFile, key_data);
-
-  if((data_data.dptr != NULL) && (data_data.dsize == (sizeof(StoredAddress))) ) {
-    StoredAddress *retrievedAddress;
-
-    retrievedAddress = (StoredAddress*)data_data.dptr;
-    *type = retrievedAddress->symAddressType;
-
-#ifdef GDBM_DEBUG
-    traceEvent(CONST_TRACE_INFO, "GDBM_DEBUG: gdbm_fetch(..., {%s, %d}) = %s, age %d",
-               key_data.dptr, key_data.dsize,
-               retrievedAddress->symAddress,
-               myGlobals.actTime - retrievedAddress->recordCreationTime);
-#endif
-
-    if (myGlobals.actTime - retrievedAddress->recordCreationTime < CONST_DNSCACHE_LIFETIME) {
-      safe_snprintf(__FILE__, __LINE__, buffer, MAX_LEN_SYM_HOST_NAME, "%s", retrievedAddress->symAddress);
-    } else {
-        buffer[0] = '\0';
-    }
-
-    free(data_data.dptr);
-  } else {
-    #ifdef GDBM_DEBUG
-    if(data_data.dptr != NULL)
-      traceEvent(CONST_TRACE_WARNING, "GDBM_DEBUG: Dropped data for %s [wrong data size]", keyBuf);
-    else
-      traceEvent(CONST_TRACE_WARNING, "GDBM_DEBUG: Unable to retrieve %s", keyBuf);
-#endif
-
-    buffer[0] = '\0';
-    *type = FLAG_HOST_SYM_ADDR_TYPE_IP;
-    /* It might be that the size of the retrieved data is wrong */
-    if(data_data.dptr != NULL) free(data_data.dptr);
-  }
-
-#ifdef DEBUG
-  {
-    char buf[LEN_ADDRESS_BUFFER];
-    traceEvent(CONST_TRACE_INFO, "fetchAddressFromCache(%s) returned '%s'",
-               _addrtostr(&hostIpAddress, buf, sizeof(buf)), buffer);
-  }
-#endif
-
-  return(1);
-}
-
-/* ******************************* */
-
 /* This function automatically updates the instance name */
+void ipaddr2str(HostTraffic *el, HostAddr hostIpAddress,
+		short vlanId, u_int actualDeviceId) {
+  HostTraffic *h;
 
-void ipaddr2str(HostAddr hostIpAddress, int updateHost) {
-  char buf[MAX_LEN_SYM_HOST_NAME+1] = { '\0' };
-  int type;
+  if(((hostIpAddress.hostFamily == AF_INET) && (hostIpAddress.addr._hostIp4Address.s_addr == 0))
+     || (el->hostResolvedNameType == FLAG_HOST_SYM_ADDR_TYPE_NAME))
+    return;
 
-  if(fetchAddressFromCache(hostIpAddress, buf, &type)  && (buf[0] != '\0')) {
-    if(updateHost) updateHostNameInfo(hostIpAddress, buf, type);
-  } else {
-    queueAddress(hostIpAddress, !updateHost);
-  }
+  h = findHostByNumIP(hostIpAddress, vlanId, actualDeviceId);
+
+  if((el != NULL) && (h != NULL)
+     /* && (el != h) */
+     && (h->hostResolvedNameType == FLAG_HOST_SYM_ADDR_TYPE_NAME)
+     && (h->hostNumIpAddress[0] != '\0')
+     && strcmp(h->hostNumIpAddress, h->hostResolvedName)
+     && strcmp(h->hostResolvedName, "0.0.0.0")) {
+#ifdef DEBUG
+    traceEvent(CONST_TRACE_ERROR, "Recycling %s = %s ", addrtostr(&hostIpAddress), el->hostResolvedName);
+#endif
+    strcpy(el->hostResolvedName, h->hostResolvedName), el->hostResolvedNameType = h->hostResolvedNameType;
+  } else
+    queueAddress(hostIpAddress);
 }
 
 /* ************************************ */
 
 char* etheraddr_string(const u_char *ep, char *buf) {
-  u_int i, j;
-  char *cp;
-
-  cp = buf;
-  if ((j = *ep >> 4) != 0)
-    *cp++ = hex[j];
-  else
-    *cp++ = '0';
-
-  *cp++ = hex[*ep++ & 0xf];
-
-  for(i = 5; (int)--i >= 0;) {
-    *cp++ = ':';
-    if ((j = *ep >> 4) != 0)
-      *cp++ = hex[j];
-    else
-      *cp++ = '0';
-
-    *cp++ = hex[*ep++ & 0xf];
-  }
-
-  *cp = '\0';
+  sprintf(buf, "%02X:%02X:%02X:%02X:%02X:%02X",
+	  ep[0] & 0xFF, ep[1] & 0xFF,
+	  ep[2] & 0xFF, ep[3] & 0xFF,
+	  ep[4] & 0xFF, ep[5] & 0xFF);
   return (buf);
 }
 
@@ -996,17 +937,17 @@ static void handleMdnsName(HostTraffic *srcHost, u_short sport, u_char *mdns_nam
 
       if((!strcmp(appl, "ipp")) || (!strcmp(appl, "printer"))) {
 	/* Printer */
-	FD_SET(FLAG_HOST_TYPE_PRINTER, &srcHost->flags);
+	setHostFlag(FLAG_HOST_TYPE_PRINTER, srcHost);
 	setHostName(srcHost, name);
       } else if(!strcmp(appl, "afpovertcp")) {
 	/* Sharing name under MacOS */
 	setHostName(srcHost, name);
       } else if(!strcmp(appl, "workstation")) {
-	/* Host name under MacOS */	
+	/* Host name under MacOS */
 	setHostName(srcHost, strtok(name, "["));
       } else if(!strcmp(appl, "http")) {
 	/* HTTP server */
-	FD_SET(FLAG_HOST_TYPE_SVC_HTTP, &srcHost->flags);
+	setHostFlag(FLAG_HOST_TYPE_SVC_HTTP, srcHost);
       } else if(!strcmp(appl, "daap")) {
 	/* Digital Audio Access Protocol (daap.sourceforge.net) */
 	updateHostUsers(name, BITFLAG_DAAP_USER, srcHost);
@@ -1365,8 +1306,8 @@ void checkSpoofing(HostTraffic *hostToCheck, int actualDeviceId) {
       /* Spoofing detected */
       if((!hasDuplicatedMac(el))
 	 && (!hasDuplicatedMac(hostToCheck))) {
-	FD_SET(FLAG_HOST_DUPLICATED_MAC, &hostToCheck->flags);
-	FD_SET(FLAG_HOST_DUPLICATED_MAC, &el->flags);
+	setHostFlag(FLAG_HOST_DUPLICATED_MAC, hostToCheck);
+	setHostFlag(FLAG_HOST_DUPLICATED_MAC, el);
 
 	if(myGlobals.runningPref.enableSuspiciousPacketDump) {
 	  traceEvent(CONST_TRACE_WARNING,
@@ -1385,16 +1326,16 @@ void checkSpoofing(HostTraffic *hostToCheck, int actualDeviceId) {
 char* subnetId2networkName(int8_t known_subnet_id, char *buf, u_short buf_len) {
   struct in_addr addr;
   char buf1[64];
-      
+
   if((known_subnet_id == UNKNOWN_SUBNET_ID)
      || (known_subnet_id < 0)
      || (known_subnet_id >= myGlobals.numKnownSubnets))
     safe_snprintf(__FILE__, __LINE__, buf, buf_len, "0.0.0.0/0");
   else {
     addr.s_addr = myGlobals.subnetStats[known_subnet_id].address[CONST_NETWORK_ENTRY];
-    
+
     safe_snprintf(__FILE__, __LINE__, buf, buf_len, "%s/%d",
-		  _intoa(addr, buf1, sizeof(buf1)), 
+		  _intoa(addr, buf1, sizeof(buf1)),
 		  myGlobals.subnetStats[known_subnet_id].address[CONST_NETMASK_V6_ENTRY]);
   }
 
@@ -1420,7 +1361,7 @@ char* host2networkName(HostTraffic *el, char *buf, u_short buf_len) {
 		    el->network_mask);
     }
   }
-  
+
   return(buf);
 }
 
@@ -1457,14 +1398,16 @@ void addDeviceNetworkToKnownSubnetList(NtopInterface *device) {
 void updateHostKnownSubnet(HostTraffic *el) {
   int i;
 
-  if(el->hostIpAddress.hostFamily != AF_INET /* v4 */) return;
+  if((myGlobals.numKnownSubnets == 0)
+     || (el->hostIpAddress.hostFamily != AF_INET /* v4 */))
+    return;
 
   for(i=0; i<myGlobals.numKnownSubnets; i++) {
     if((el->hostIpAddress.addr._hostIp4Address.s_addr & myGlobals.subnetStats[i].address[CONST_NETMASK_ENTRY])
        == myGlobals.subnetStats[i].address[CONST_NETWORK_ENTRY]) {
       el->known_subnet_id = i;
-      // FD_SET(FLAG_SUBNET_LOCALHOST, &el->flags);
-      FD_SET(FLAG_SUBNET_PSEUDO_LOCALHOST, &el->flags);
+      // setHostFlag(FLAG_SUBNET_LOCALHOST, el);
+      setHostFlag(FLAG_SUBNET_PSEUDO_LOCALHOST, el);
       return;
     }
   }
