@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2010 Luca Deri <deri@ntop.org>
  *
  *			    http://www.ntop.org/
  *
@@ -27,7 +27,7 @@ static void encodeWebFormURL(char *in, char *buf, int buflen);
 static void decodeWebFormURL(char *buf);
 static int processNtopConfigData (char *buf, int savePref);
 
-/* *******************************/
+/* ****************************** */
 
 void showUsers(void) {
   u_int numUsers=0;
@@ -88,7 +88,7 @@ void showUsers(void) {
   sendMenuFooter(1, 2);
 }
 
-/* *******************************/
+/* ****************************** */
 
 void clearUserUrlList(void) {
   int i;
@@ -113,7 +113,7 @@ void clearUserUrlList(void) {
     releaseMutex(&myGlobals.securityItemsMutex);
 }
 
-/* *******************************/
+/* ****************************** */
 
 void addUser(char* user) {
   char tmpStr[128];
@@ -231,7 +231,7 @@ void addUser(char* user) {
   sendMenuFooter(0, 2);
 }
 
-/* *******************************/
+/* ****************************** */
 
 void deleteUser(char* user) {
   if(user == NULL) {
@@ -269,7 +269,7 @@ void deleteUser(char* user) {
   sendMenuFooter(1, 2);
   printHTMLtrailer();
 }
-/* *******************************/
+/* ****************************** */
 
 #define MAX_NUM_COMMUNITIES     32
 
@@ -456,7 +456,7 @@ void showURLs(void) {
   sendMenuFooter(3, 0);
 }
 
-/* *******************************/
+/* ****************************** */
 
 void addURL(char* url) {
   int i, numUsers=0;
@@ -570,7 +570,7 @@ void addURL(char* url) {
   sendMenuFooter(0, 2);
 }
 
-/* *******************************/
+/* ****************************** */
 
 void deleteURL(char* url) {
 
@@ -610,7 +610,7 @@ void deleteURL(char* url) {
   printHTMLtrailer();
 }
 
-/* *******************************/
+/* ****************************** */
 
 void doAddURL(int len) {
   char *err=NULL;
@@ -790,13 +790,12 @@ void setPcapFilter(char* filters, int device_id) {
   }
 }
 
-/* *******************************/
+/* ****************************** */
 
 /* Courtesy of Michael Weidel <michael.weidel@gmx.de> */
 
 int doChangeFilter(int len) {
   int i,idx,badChar=0;
-  struct bpf_program fcode;
   char *currentFilterExpressionSav;
   char buf[LEN_GENERAL_WORK_BUFFER],postData[256],*key,*err=NULL;
 
@@ -913,7 +912,7 @@ void changeFilter(void) {
   sendString("<BR><B></FONT>\n");
 }
 
-/* *******************************/
+/* ****************************** */
 
 struct _menuData {
   char	*text, *anchor;
@@ -926,7 +925,7 @@ static struct _menuData menuItem[] = {
   { "Add URL",    CONST_ADD_URLS_HTML }
 };
 
-/* *******************************/
+/* ****************************** */
 
 static void sendMenuFooter(int itm1Idx, int itm2Idx) {
   char	buf[128];
@@ -941,7 +940,7 @@ static void sendMenuFooter(int itm1Idx, int itm2Idx) {
   sendString("</FONT>\n</CENTER>\n");
 }
 
-/* *******************************/
+/* ****************************** */
 
 static void encodeWebFormURL(char *in, char *buf, int buflen) {
   int i, j, c, d;
@@ -963,7 +962,7 @@ static void encodeWebFormURL(char *in, char *buf, int buflen) {
   buf[j] = '\0';
 }
 
-/* *******************************/
+/* ****************************** */
 
 static void decodeWebFormURL(char *buf) {
   int i, j;
@@ -988,8 +987,7 @@ static void decodeWebFormURL(char *buf) {
   C C Magnus Gustavsson <magnus@gustavsson.se>
 */
 static void addKeyIfMissing(char* key, char* value,
-			    int encryptValue, int existingOK,
-			    char *userQuestion) {
+			    int encryptValue, char *userQuestion) {
   datum key_data, return_data, data_data;
 #ifndef WIN32
  /* 14 is ok for traditional crypt, but the enhanced crypt() in FreeBSD
@@ -1005,7 +1003,7 @@ static void addKeyIfMissing(char* key, char* value,
 
   return_data = gdbm_fetch(myGlobals.pwFile, key_data);
 
-  if((return_data.dptr == NULL) || (existingOK != 0)) {
+  if(return_data.dptr == NULL) {
     char *thePw, pw1[16], pw2[16];
     /* If not existing, then add user 'admin' and ask for password  */
 
@@ -1096,32 +1094,36 @@ static void addKeyIfMissing(char* key, char* value,
     free(return_data.dptr);
 }
 
-/* *******************************/
+/* ****************************** */
 
 void setAdminPassword(char* pass) {
+  traceEvent(CONST_TRACE_INFO, "Setting administrator password...");
+  
   if (pass == NULL)
-    addKeyIfMissing("1admin", NULL, 1, 1, CONST_ADMINPW_QUESTION);
+    addKeyIfMissing("1admin", NULL, 1, CONST_ADMINPW_QUESTION);
   else
-    addKeyIfMissing("1admin", pass, 1, 1, NULL);
+    addKeyIfMissing("1admin", pass, 1, NULL);
+
+  traceEvent(CONST_TRACE_INFO, "Admin password set...");
 }
 
-/* *******************************/
+/* ****************************** */
 
 void addDefaultAdminUser(void) {
   /* Add user 'admin' and ask for password if not existing */
-  addKeyIfMissing("1admin", NULL, 1, 0, CONST_ADMINPW_QUESTION);
+  addKeyIfMissing("1admin", NULL, 1, CONST_ADMINPW_QUESTION);
 
   /* Add user 'admin' for URL 'show...' if not existing */
-  addKeyIfMissing("2showU",      "users=1admin", 0, 0, NULL);
-  addKeyIfMissing("2modifyU",    "users=1admin", 0, 0, NULL);
-  addKeyIfMissing("2deleteU",    "users=1admin", 0, 0, NULL);
-  addKeyIfMissing("2shut",       "users=1admin", 0, 0, NULL);
-  addKeyIfMissing("2resetStats", "users=1admin", 0, 0, NULL);
-  addKeyIfMissing("2chang",      "users=1admin", 0, 0, NULL);
-  addKeyIfMissing("2configNtop", "users=1admin", 0, 0, NULL);
-  addKeyIfMissing("2privacyFlag","users=1admin", 0, 0, NULL);
-  addKeyIfMissing("2"CONST_EDIT_PREFS,"users=1admin", 0, 0, NULL);
-  addKeyIfMissing("2"CONST_PURGE_HOST,"users=1admin", 0, 0, NULL);
+  addKeyIfMissing("2showU",      "users=1admin", 0, NULL);
+  addKeyIfMissing("2modifyU",    "users=1admin", 0, NULL);
+  addKeyIfMissing("2deleteU",    "users=1admin", 0, NULL);
+  addKeyIfMissing("2shut",       "users=1admin", 0, NULL);
+  addKeyIfMissing("2resetStats", "users=1admin", 0, NULL);
+  addKeyIfMissing("2chang",      "users=1admin", 0, NULL);
+  addKeyIfMissing("2configNtop", "users=1admin", 0, NULL);
+  addKeyIfMissing("2privacyFlag","users=1admin", 0, NULL);
+  addKeyIfMissing("2"CONST_EDIT_PREFS,"users=1admin", 0, NULL);
+  addKeyIfMissing("2"CONST_PURGE_HOST,"users=1admin", 0, NULL);
 }
 
 /* ************************************ */
@@ -1129,7 +1131,7 @@ void addDefaultAdminUser(void) {
 #define NTOP_SAVE_PREFS     "SP"
 #define NTOP_RESTORE_DEF    "RD"
 #define CONFIG_STR_ENTRY(bg,title,name,size,configvalue,descr) \
-        safe_snprintf (__FILE__, __LINE__, buf, sizeof (buf), "<tr><td align=left %s>%s</td><td align=left><INPUT NAME=%s SIZE=%d VALUE=%s><BR>%s</td></TR>\n", bg, title, name, size, (configvalue != NULL) ? configvalue : "", descr); \
+        safe_snprintf (__FILE__, __LINE__, buf, sizeof (buf), "<tr><td align=left %s>%s</td><td align=left><INPUT NAME=\"%s\" SIZE=%d VALUE=\"%s\"><BR>%s</td></TR>\n", bg, title, name, size, (configvalue != NULL) ? configvalue : "", descr); \
         sendString (buf);
 
 #define CONFIG_FILE_ENTRY(bg,title,name,size,value,descr) \
@@ -1171,17 +1173,11 @@ void addDefaultAdminUser(void) {
    tmpPrefs.disablePromiscuousMode = tmpPrefs.disableMutexExtraInfo = 0;
    tmpPrefs.disableInstantSessionPurge = tmpPrefs.disableStopcap = 0;
    tmpPrefs.debugMode = tmpPrefs.daemonMode = tmpPrefs.w3c = 0;
-   tmpPrefs.numericFlag = tmpPrefs.mergeInterfaces = tmpPrefs.enableL7 = 0;
+   tmpPrefs.numericFlag = dnsResolutionForAll;
+   tmpPrefs.mergeInterfaces = tmpPrefs.enableL7 = 0;
    tmpPrefs.dontTrustMACaddr = 0;
    tmpPrefs.enableOtherPacketDump = tmpPrefs.enableSuspiciousPacketDump = 0;
    tmpPrefs.enableSessionHandling = 0;
-#ifdef MAKE_WITH_SSLWATCHDOG_RUNTIME
-   tmpPrefs.useSSLwatchdog = 0;
-#endif
-
-#ifdef MAKE_WITH_SCHED_YIELD
-   tmpPrefs.disableSchedYield = 0;
-#endif
 
    devices = tmpPrefs.devices;
    tmpPrefs.devices = NULL;
@@ -1266,10 +1262,12 @@ void addDefaultAdminUser(void) {
        processNtopPref(NTOP_PREF_DAEMON, FALSE, savePref, &tmpPrefs);
      }
 
+#ifdef ENABLE_FC
      if (display_prefs && myGlobals.savedPref.noInvalidLunDisplay &&
 	 !tmpPrefs.noInvalidLunDisplay) {
        processNtopPref(NTOP_PREF_NO_INVLUN, FALSE, savePref, &tmpPrefs);
      }
+#endif
 
    if (display_prefs && myGlobals.savedPref.w3c && !tmpPrefs.w3c) {
      processNtopPref(NTOP_PREF_W3C, FALSE, savePref, &tmpPrefs);
@@ -1296,18 +1294,6 @@ void addDefaultAdminUser(void) {
      processNtopPref(NTOP_PREF_NO_TRUST_MAC, FALSE, savePref, &tmpPrefs);
    }
 
-#ifdef MAKE_WITH_SSLWATCHDOG_RUNTIME
-   if (advanced_prefs && myGlobals.savedPref.useSSLwatchdog && !tmpPrefs.useSSLwatchdog) {
-     processNtopPref(NTOP_PREF_USE_SSLWATCH, FALSE, savePref, &tmpPrefs);
-   }
-#endif
-
-#ifdef MAKE_WITH_SCHED_YIELD
-   if (advanced_prefs && myGlobals.savedPref.disableSchedYield && !tmpPrefs.disableSchedYield) {
-     processNtopPref(NTOP_PREF_NO_SCHEDYLD, FALSE, savePref, &tmpPrefs);
-   }
-#endif
-
    if (debug_prefs && myGlobals.savedPref.debugMode && !tmpPrefs.debugMode) {
      processNtopPref(NTOP_PREF_DBG_MODE, FALSE, savePref, &tmpPrefs);
    }
@@ -1469,8 +1455,8 @@ void printNtopConfigHeader (char *url, UserPrefDisplayPage configScr)
 		  );
     break;
 
-#ifdef HAVE_MYSQL_H
   case showPrefDBPref:
+#ifdef HAVE_MYSQL_H
     safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "<P ALIGN=CENTER>"
 		  "[ <A HREF=%s1>Basic Prefs</A> ]&nbsp;"
 		  "[ <A HREF=%s2>Display Prefs</A> ]&nbsp;"
@@ -1480,8 +1466,11 @@ void printNtopConfigHeader (char *url, UserPrefDisplayPage configScr)
 		  "[ <A HREF=%s6>Debugging Prefs</A> ]&nbsp;"
 		  "[ <B>DB Prefs</B> ]&nbsp;</p>",
 		  theLink, theLink, theLink, theLink, theLink, theLink, theLink);
+#else
+	  /* Nothing to do */
 #endif
     break;
+
   }
 
   sendString (buf);
@@ -1713,6 +1702,7 @@ void handleNtopConfig(char* url, UserPrefDisplayPage configScr,
 		     "Max number of lines that ntop will display on each "
 		     " generated HTML page");
 
+#ifdef ENABLE_FC
     sendString("<TR><TD ALIGN=LEFT "DARK_BG">Show Menus For</TD><TD ALIGN=LEFT>");
     safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf),
 		  "<INPUT TYPE=radio NAME=%s  VALUE=%d %s>IP<br>\n",
@@ -1737,6 +1727,7 @@ void handleNtopConfig(char* url, UserPrefDisplayPage configScr,
 		       NTOP_PREF_NO_INVLUN,
 		       pref->noInvalidLunDisplay,
 		       "Don't display info about non-existent LUNs");
+#endif
 
     CONFIG_RADIO_ENTRY(DARK_BG, "Use W3C", NTOP_PREF_W3C,
 		       pref->w3c,
@@ -1772,8 +1763,7 @@ void handleNtopConfig(char* url, UserPrefDisplayPage configScr,
 		     "from the interface or in case of capture files");
 
     CONFIG_RADIO_ENTRY(DARK_BG, "No DNS (-n)", NTOP_PREF_NUMERIC_IP,
-		       pref->numericFlag, "Skip DNS resolution, showing "
-		       "only numeric IP addresses");
+		       pref->numericFlag, "DNS resolution mode");
 
     CONFIG_STR_ENTRY(DARK_BG, "TCP/UDP Protocols To Monitor (-p)",
 		     NTOP_PREF_PROTOSPECS, 50, pref->protoSpecs,
@@ -1790,6 +1780,7 @@ void handleNtopConfig(char* url, UserPrefDisplayPage configScr,
 
     break;
 
+#ifdef ENABLE_FC
   case showPrefFCPref:
     sendString("<INPUT TYPE=HIDDEN NAME=FC_PREFS VALUE=1>");
 
@@ -1797,6 +1788,7 @@ void handleNtopConfig(char* url, UserPrefDisplayPage configScr,
 		     50, pref->fcNSCacheFile,
 		     "Location of file mapping VSAN/FC_ID to WWN/Alias");
     break;
+#endif
 
   case showPrefAdvPref:
     sendString("<INPUT TYPE=HIDDEN NAME=ADVANCED_PREFS VALUE=1>");
@@ -1815,13 +1807,6 @@ void handleNtopConfig(char* url, UserPrefDisplayPage configScr,
 		       NTOP_PREF_MERGEIF, pref->mergeInterfaces,
 		       "Yes = merge data from all interfaces (if possible), No = do not merge data from all interfaces");
 
-#ifdef HAVE_LIBPCRE
-    CONFIG_RADIO_ENTRY(DARK_BG, "Enable Protocol Guessing",
-		       NTOP_PREF_ENABLE_L7PROTO, pref->enableL7,
-		       "Enabling patterm matching, ntop will be able to guess the protocol being used. "
-		       "Neverthelss this practice has a little performance penalty.");
-#endif
-
     CONFIG_RADIO_ENTRY(DARK_BG, "No Instant Session Purge",
 		       NTOP_PREF_NO_ISESS_PURGE,
 		       pref->disableInstantSessionPurge,
@@ -1836,18 +1821,6 @@ void handleNtopConfig(char* url, UserPrefDisplayPage configScr,
     CONFIG_STR_ENTRY(DARK_BG, "Pcap Log Base Path (-O)",
 		     NTOP_PREF_PCAP_LOGBASE, 50, pref->pcapLogBasePath,
 		     "Directory where packet dump files are created");
-
-#ifdef MAKE_WITH_SSLWATCHDOG_RUNTIME
-    CONFIG_RADIO_ENTRY(DARK_BG, "Use SSL Watchdog",
-		       NTOP_PREF_USE_SSLWATCH, pref->useSSLwatchdog,
-		       "");
-#endif
-
-#ifdef MAKE_WITH_SCHED_YIELD
-    CONFIG_RADIO_ENTRY(DARK_BG, "Disable SchedYield",
-		       NTOP_PREF_NO_SCHEDYLD, pref->disableSchedYield,
-		       "");
-#endif
     break;
 
   case showPrefDbgPref:
