@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2001-09 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 2001-10 Luca Deri <deri@ntop.org>
  *
  *     		            http://www.ntop.org/
  *
@@ -408,9 +408,11 @@ void dumpNtopFlows(FILE *fDescr, char* options, int actualDeviceId) {
 
 	if(strcasecmp(tmpStr, "language") == 0) {
 	  lang = DEFAULT_FLAG_LANGUAGE;
-	  for(j=1;j <= MAX_FLAG_LANGUGE;j++) {
-	    if(strcasecmp(&tmpStr[i+1], languages[j]) == 0)
+	  for(j=1;j < MAX_FLAG_LANGUGE;j++) {
+	    if(strcasecmp(&tmpStr[i+1], languages[j]) == 0) {
 	      lang = j;
+	      break;
+	    }
 	  }
 	}
       }
@@ -472,9 +474,11 @@ void dumpNtopTrafficMatrix(FILE *fDescr, char* options, int actualDeviceId) {
 
 	if(strcasecmp(tmpStr, "language") == 0) {
 	  lang = DEFAULT_FLAG_LANGUAGE;
-	  for(j=1;j <= MAX_FLAG_LANGUGE;j++) {
-	    if(strcasecmp(&tmpStr[i+1], languages[j]) == 0)
+	  for(j=1;j < MAX_FLAG_LANGUGE;j++) {
+	    if(strcasecmp(&tmpStr[i+1], languages[j]) == 0) {
 	      lang = j;
+	      break;
+	    }
 	  }
 	}
       }
@@ -568,9 +572,11 @@ void dumpNtopHashes(FILE *fDescr, char* options, int actualDeviceId) {
 
 	if(strcasecmp(tmpStr, "language") == 0) {
 	  lang = DEFAULT_FLAG_LANGUAGE;
-	  for(j=1;j <= MAX_FLAG_LANGUGE;j++) {
-	    if(strcasecmp(&tmpStr[i+1], languages[j]) == 0)
+	  for(j=1;j < MAX_FLAG_LANGUGE;j++) {
+	    if(strcasecmp(&tmpStr[i+1], languages[j]) == 0) {
 	      lang = j;
+	      break;
+	    }
 	  }
 	} else if(strcmp(tmpStr, "key") == 0) {
 	  strncpy(key, &tmpStr[i+1], sizeof(key));
@@ -776,11 +782,6 @@ void dumpNtopHashes(FILE *fDescr, char* options, int actualDeviceId) {
 	wrtFloatItm(fDescr, lang, "\t", "peakTPktThpt", el->peakTPktThpt, ',', numEntries);
     }
 
-    if(checkFilter(filter, "ipv4BytesSent"))
-      wrtLlongItm(fDescr, lang, "\t", "ipv4BytesSent", el->ipv4BytesSent, ',', numEntries);
-    if(checkFilter(filter, "ipv4BytesRcvd"))
-      wrtLlongItm(fDescr, lang, "\t", "ipv4BytesRcvd", el->ipv4BytesRcvd, ',', numEntries);
-
     if(checkFilter(filter, "ipv6BytesSent"))
       wrtLlongItm(fDescr, lang, "\t", "ipv6BytesSent", el->ipv6BytesSent, ',', numEntries);
     if(checkFilter(filter, "ipv6BytesRcvd"))
@@ -1146,9 +1147,11 @@ void dumpNtopHashIndexes(FILE *fDescr, char* options, int actualDeviceId) {
 	if(strcmp(tmpStr, "language") == 0) {
 
 	  lang = DEFAULT_FLAG_LANGUAGE;
-	  for(j=1;j <= MAX_FLAG_LANGUGE;j++) {
-	    if(strcmp(&tmpStr[i+1], languages[j]) == 0)
+	  for(j=1;j < MAX_FLAG_LANGUGE;j++) {
+	    if(strcmp(&tmpStr[i+1], languages[j]) == 0) {
 	      lang = j;
+	      break;
+	    }
 	  }
 	}
       }
@@ -1224,9 +1227,11 @@ void dumpNtopTrafficInfo(FILE *fDescr, char* options) {
 
 	if(strcmp(tmpStr, "language") == 0) {
 	  lang = DEFAULT_FLAG_LANGUAGE;
-	  for(j=1;j <= MAX_FLAG_LANGUGE;j++) {
-	    if(strcmp(&tmpStr[i+1], languages[j]) == 0)
+	  for(j=1;j < MAX_FLAG_LANGUGE;j++) {
+	    if(strcmp(&tmpStr[i+1], languages[j]) == 0) {
 	      lang = j;
+	      break;
+	    }
 	  }
 	} else if(strcmp(tmpStr, "key") == 0) {
 	  strncpy(key, &tmpStr[i+1], sizeof(key));
