@@ -1,10 +1,10 @@
-/**
+/*
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
  *                          http://www.ntop.org
  *
  * Copyright (C) 2003 Dinesh G. Dutt <ddutt@cisco.com>
- * Copyright (C) 2003-09 Luca Deri <deri@ntop.org>
+ * Copyright (C) 2003-10 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -23,15 +23,16 @@
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-
 #include "ntop.h"
+
+#ifdef ENABLE_FC
 #include <stdarg.h>
 #include <assert.h>
 #include "fcUtils.h"
 
 /* **************************************** */
 
-int isFlogiAcc (FcAddress *fcAddress, u_int8_t r_ctl, u_int8_t type, u_int8_t cmd)
+int isFlogiAcc (FcAddress *fcAddress, uint8_t r_ctl, uint8_t type, uint8_t cmd)
 {
   if (((fcAddress->domain == 0xFF) && (fcAddress->area == 0xFF) && (fcAddress->port == 0xFE)) && 
       ((r_ctl & 0xF0) == FC_RCTL_ELS) && (type == FC_TYPE_ELS) &&
@@ -63,7 +64,7 @@ int fillFcHostInfo (const u_char *bp, HostTraffic *srcHost)
 /* **************************************** */
 
 /* Routine to extract WWN from PLOGI frame */
-int isPlogi (u_int8_t r_ctl, u_int8_t type, u_int8_t cmd)
+int isPlogi (uint8_t r_ctl, uint8_t type, uint8_t cmd)
 {
   if (((r_ctl & 0xF0) == FC_RCTL_ELS) && (type == FC_TYPE_ELS) &&
       (cmd == FC_ELS_CMD_PLOGI)) {
@@ -76,7 +77,7 @@ int isPlogi (u_int8_t r_ctl, u_int8_t type, u_int8_t cmd)
 
 /* **************************************** */
 
-int isLogout (u_int8_t r_ctl, u_int8_t type, u_int8_t cmd)
+int isLogout (uint8_t r_ctl, uint8_t type, uint8_t cmd)
 {
   if (((r_ctl & 0xF0) == FC_RCTL_ELS) && (type == FC_TYPE_ELS) &&
       (cmd == FC_ELS_CMD_LOGO)) {
@@ -89,7 +90,7 @@ int isLogout (u_int8_t r_ctl, u_int8_t type, u_int8_t cmd)
 
 /* **************************************** */
 
-int isRscn (u_int8_t r_ctl, u_int8_t type, u_int8_t cmd)
+int isRscn (uint8_t r_ctl, uint8_t type, uint8_t cmd)
 {
   if (((r_ctl & 0xF0) == FC_RCTL_ELS) && (type == FC_TYPE_ELS) &&
       (cmd == FC_ELS_CMD_RSCN)) {
@@ -120,7 +121,7 @@ HostTraffic *allocFcScsiCounters(HostTraffic *host) {
 int fillFcpInfo (const u_char *bp, HostTraffic *srcHost, HostTraffic *dstHost)
 {
   int offset = 0;
-  u_int32_t fcpDl;
+  uint32_t fcpDl;
     
   assert (bp != NULL);
   assert (srcHost != NULL);
@@ -130,7 +131,7 @@ int fillFcpInfo (const u_char *bp, HostTraffic *srcHost, HostTraffic *dstHost)
     /* This is a single-level LUN */
   }
 
-  fcpDl = ntohl (*(u_int32_t *)&bp[offset+28]);
+  fcpDl = ntohl (*(uint32_t *)&bp[offset+28]);
 
   if(allocFcScsiCounters(srcHost) == NULL) return(0);
   if(allocFcScsiCounters(dstHost) == NULL) return(0);
@@ -152,7 +153,7 @@ int fillFcpInfo (const u_char *bp, HostTraffic *srcHost, HostTraffic *dstHost)
 FcFabricElementHash *getFcFabricElementHash (u_short vsanId, int actualDeviceId)
 {
   FcFabricElementHash **theHash;
-  u_int myIdx = 0, idx;
+  uint myIdx = 0, idx;
 
   idx = vsanId % MAX_ELEMENT_HASH;
   theHash = myGlobals.device[actualDeviceId].vsanHash;    
@@ -192,13 +193,13 @@ int isValidFcNxPort (FcAddress *fcAddress)
 
 int updateFcFabricElementHash (FcFabricElementHash **theHash, u_short vsanId,
                                const u_char *bp, FcAddress *srcAddr, FcAddress *dstAddr,
-                               u_short protocol, u_char r_ctl, u_int32_t pktlen)
+                               u_short protocol, u_char r_ctl, uint32_t pktlen)
 {
-  u_int myIdx = 0, idx;
+  uint myIdx = 0, idx;
   FcFabricElementHash *hash;
-  u_int8_t cmd, srcDomain, dstDomain;
+  uint8_t cmd, srcDomain, dstDomain;
   u_short payload_len;
-  u_int8_t gs_type, gs_stype;
+  uint8_t gs_type, gs_stype;
 
   idx = vsanId % MAX_ELEMENT_HASH;
     
@@ -499,7 +500,7 @@ void processFcNSCacheFile(char *filename) {
   FcNameServerCacheEntry *entry;
   HostTraffic *el;
   FcAddress fcid;
-  u_int32_t vsanId, domain, area, port, tgtType, i, j;
+  uint32_t vsanId, domain, area, port, tgtType, i, j;
   wwn_t pWWN, nWWN;
   char alias[MAX_LEN_SYM_HOST_NAME];
   FILE *fd;
@@ -631,7 +632,7 @@ void processFcNSCacheFile(char *filename) {
 
     entryFound = 0;
     while (entry != NULL) {
-      if (memcmp ((u_int8_t *)&(entry->fcAddress), (u_int8_t *)&fcid,
+      if (memcmp ((uint8_t *)&(entry->fcAddress), (uint8_t *)&fcid,
 		  LEN_FC_ADDRESS) == 0) {
 	entryFound = 1;
 	break;
@@ -657,6 +658,8 @@ void processFcNSCacheFile(char *filename) {
     memcpy (&entry->pWWN.str[0], &pWWN.str[0], LEN_WWN_ADDRESS);
     memcpy (&entry->nWWN.str[0], &nWWN.str[0], LEN_WWN_ADDRESS);
     strncpy (&entry->alias[0], alias, MAX_LEN_SYM_HOST_NAME);
-    entry->alias[MAX_LEN_SYM_HOST_NAME] = '\0';
+    entry->alias[MAX_LEN_SYM_HOST_NAME-1] = '\0';
   }
 }
+
+#endif /* ENABLE_FC */
