@@ -1,8 +1,9 @@
 /*
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
+ *
  *                          http://www.ntop.org
  *
- *          Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
+ *          Copyright (C) 1998-2010 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -70,17 +71,6 @@ static char *_configFileDirs[] = { ".", CFG_CONFIGFILE_DIR, DEFAULT_NTOP_CFG_CON
 #endif
 				   NULL };
 
-/*
- *  ** TCP Wrappers
- *
- *      Because of limits in the way libwrap.a does things, these MUST
- *      be open global values.
- *
- */
-#ifdef HAVE_LIBWRAP
-int allow_severity, deny_severity;
-#endif /* HAVE_LIBWRAP */
-
 #if defined(INET6)
 struct in6_addr _in6addr_linklocal_allnodes;
 #endif
@@ -100,18 +90,18 @@ void initGdbm(char *prefDirectory,  /* Directory with persistent files */
     return;
   }
 
-  initSingleGdbm(&myGlobals.dnsCacheFile,     "dnsCache.db",     spoolDirectory, -1,    NULL);
-
   if(!myGlobals.runningPref.liveMode) {
     initSingleGdbm(&myGlobals.macPrefixFile,    "macPrefix.db",    spoolDirectory, FALSE,  &statbuf);
     initSingleGdbm(&myGlobals.fingerprintFile,  "fingerprint.db",  spoolDirectory, FALSE,  &statbuf);
     createVendorTable(&statbuf);
   }
+
+  checkCommunities(); /* Check if communities are defined */
 }
 
 /* ******************************* */
 
-static void allocateOtherHosts() {
+static void allocateOtherHosts(void) {
   if(myGlobals.otherHostEntry != NULL) {
     traceEvent(CONST_TRACE_WARNING, "Attempting to call twice allocateOtherHosts()");
     return;
@@ -177,11 +167,6 @@ void initNtopGlobals(int argc, char * argv[], int argc_started, char *argv_start
   myGlobals.checkVersionStatus = FLAG_CHECKVERSION_NOTCHECKED;
   myGlobals.checkVersionStatusAgain = 1;
 
-#if !defined(WIN32) && defined(HAVE_LIBWRAP)
-  allow_severity = DEFAULT_TCPWRAP_ALLOW;
-  deny_severity = DEFAULT_TCPWRAP_DENY;
-#endif
-
   /* Other flags (to be set via command line options one day) */
   myGlobals.enableFragmentHandling = 1;
 
@@ -212,7 +197,6 @@ void initNtopGlobals(int argc, char * argv[], int argc_started, char *argv_start
   }
 
   /* Databases */
-  myGlobals.dnsCacheFile = NULL;
   myGlobals.pwFile = NULL;
 
   /* the table of broadcast entries */
@@ -225,7 +209,7 @@ void initNtopGlobals(int argc, char * argv[], int argc_started, char *argv_start
   myGlobals.shortDomainName = NULL;
 
   myGlobals.numThreads = 0;            /* # of running threads */
-
+  
 #ifdef HAVE_OPENSSL
   myGlobals.sslInitialized = 0;
   myGlobals.runningPref.sslPort = 0; /* Disabled by default: enabled via -W */
@@ -280,6 +264,8 @@ void initNtopGlobals(int argc, char * argv[], int argc_started, char *argv_start
   myGlobals.numBadSSIRequests = 0;
   myGlobals.numHandledSSIRequests = 0;
 
+  createMutex(&myGlobals.geolocalizationMutex); /* GeoIP mutex */
+
   /* create the logView stuff Mutex first... must be before the 1st traceEvent() call */
   createMutex(&myGlobals.logViewMutex);     /* synchronize logView buffer */
 #ifdef FORPRENPTL
@@ -440,9 +426,9 @@ void initNtopGlobals(int argc, char * argv[], int argc_started, char *argv_start
 	  sizeof(myGlobals.broadcastEntry->hostNumIpAddress));
   myGlobals.broadcastEntry->hostResolvedNameType = FLAG_HOST_SYM_ADDR_TYPE_FAKE;
   strcpy(myGlobals.broadcastEntry->ethAddressString, "FF:FF:FF:FF:FF:FF");
-  FD_SET(FLAG_SUBNET_LOCALHOST, &myGlobals.broadcastEntry->flags);
-  FD_SET(FLAG_BROADCAST_HOST, &myGlobals.broadcastEntry->flags);
-  FD_SET(FLAG_SUBNET_PSEUDO_LOCALHOST, &myGlobals.broadcastEntry->flags);
+  setHostFlag(FLAG_SUBNET_LOCALHOST, myGlobals.broadcastEntry);
+  setHostFlag(FLAG_BROADCAST_HOST, myGlobals.broadcastEntry);
+  setHostFlag(FLAG_SUBNET_PSEUDO_LOCALHOST, myGlobals.broadcastEntry);
   memset(&myGlobals.broadcastEntry->hostSerial, 0, sizeof(HostSerial));
 
   allocateOtherHosts();
@@ -466,6 +452,7 @@ void initNtopGlobals(int argc, char * argv[], int argc_started, char *argv_start
 
   myGlobals.startedAs = startedAs;
 
+#ifdef ENABLE_FC
   /* FC & SCSI-specific stuff */
   myGlobals.scsiDefaultDevType = SCSI_DEV_UNINIT;
 
@@ -488,6 +475,7 @@ void initNtopGlobals(int argc, char * argv[], int argc_started, char *argv_start
 
   myGlobals.fcMatrixHashCollisions = 0;
   myGlobals.fcMatrixHashUnresCollisions = 0;
+#endif
 
   /* 
      Efficiency is the ability to fill-up ATM cells so that they
@@ -498,7 +486,7 @@ void initNtopGlobals(int argc, char * argv[], int argc_started, char *argv_start
 
 /* ********************************* */
 
-static void loadGeoIP() {
+static void loadGeoIP(void) {
   int i;
   struct stat statbuf;
 
@@ -560,18 +548,13 @@ void initNtop(char *devices) {
   if(myGlobals.numIpProtosToMonitor == 0)
     addDefaultProtocols();
 
-#if defined(HAVE_LIBPCRE) && defined(HAVE_PCRE_H)
-  if(myGlobals.runningPref.enableL7)
-    initl7();
-  else
-    traceEvent(CONST_TRACE_INFO, "No patterns to load: protocol guessing disabled.");
-#endif
-
   /*
    * initialize memory and data.
    */
   initDevices(devices);
 
+  init_events();
+
   if(myGlobals.runningPref.enableSessionHandling)
     initPassiveSessions();
 
@@ -621,11 +604,12 @@ void initNtop(char *devices) {
   if((myGlobals.pcap_file_list != NULL) &&
      ((myGlobals.runningPref.localAddresses == NULL) &&
       !myGlobals.runningPref.printFcOnly)) {
-    setRunState(FLAG_NTOPSTATE_SHUTDOWN);
-    traceEvent(CONST_TRACE_FATALERROR,
-	       "-m | local-subnets must be specified when the -f | --traffic-dump-file option is used"
-	       "Capture not started");
-    exit(2); /* Just in case */
+    char *any_net = "0.0.0.0/0";
+
+    traceEvent(CONST_TRACE_WARNING,
+	       "-m | local-subnets must be specified when the -f option is used"
+	       "Assuming %s", any_net);
+    myGlobals.runningPref.localAddresses = strdup(any_net);
   }
 
   if(myGlobals.runningPref.currentFilterExpression != NULL)
@@ -648,7 +632,7 @@ void initNtop(char *devices) {
   traceEvent(CONST_TRACE_NOISY, "Plugins started... continuing with initialization");
 #endif
 
-#if defined(MEMORY_DEBUG)
+#if defined(MEMORY_DEBUG) && defined(MAKE_WITH_SAFER_ROUTINES)
   resetLeaks();
 #endif
 
@@ -809,7 +793,7 @@ short _setRunState(char *file, int line, short newRunState) {
 
   myGlobals.ntopRunState = newRunState;
   traceEvent(CONST_TRACE_ALWAYSDISPLAY, "THREADMGMT[t%lu]: ntop RUNSTATE: %s(%d)",
-             pthread_self(),
+             (long unsigned int)pthread_self(),
              stateTransitionTableNames[newRunState],
              newRunState);
 
