@@ -2,7 +2,7 @@
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *                          http://www.ntop.org
  *
- *            Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
+ *            Copyright (C) 1998-2010 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -40,18 +40,8 @@ extern MYCODE myFacilityNames[];
 extern struct in6_addr _in6addr_linklocal_allnodes;
 #endif
 
-/* Fix courtesy of Tim Gardner <timg@tpi.com> */
-#ifdef ASYNC_ADDRESS_RESOLUTION
 #define accessAddrResMutex(a) if(myGlobals.runningPref.numericFlag == 0) accessMutex(&myGlobals.addressResolutionMutex,a)
 #define releaseAddrResMutex() if(myGlobals.runningPref.numericFlag == 0) releaseMutex(&myGlobals.addressResolutionMutex)
-#else
-#define accessAddrResMutex(a)
-#define releaseAddrResMutex()
-#endif
-
-#ifdef HAVE_LIBWRAP
-extern int allow_severity, deny_severity;
-#endif
 
 /* version.c */
 extern char *version, *osName, *author, *buildDate, *configureDate,
@@ -87,6 +77,7 @@ extern void tokenizeCleanupAndAppend(char *userAgent, int userAgentLen, char *ti
 extern void extractAndAppend(char *userAgent, int userAgentLen, char *title, char *input);
 extern int retrieveVersionFile(char *versionSite, char *versionFile, char *buf, int bufLen);
 extern int processVersionFile(char *buf, int bufLen);
+extern void maximize_socket_buffer(int sock_fd, int buf_type);
 extern void setEmptySerial(HostSerial *a);
 extern FILE* checkForInputFile(char* logTag, char* descr, char* fileName, struct stat *dbStat,
                                u_char* compressedFormat);
@@ -105,6 +96,13 @@ extern void revertSlashIfWIN32(char *str, int mode);
 extern void revertDoubleColumnIfWIN32(char *str);
 extern void checkUserIdentity(int userSpecified);
 
+/* RRD */
+typedef int (*updateRRDPtr)(char *hostPath, char *key, Counter value, int isCounter, char short_step);
+extern time_t rrdTime;
+extern void setUpdateRRDCallback(updateRRDPtr ptr);
+extern int updateCounter(char *hostPath, char *key, Counter value, char short_step);
+extern int updateGauge(char *hostPath, char *key, Counter value, char short_step);
+
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    Work-arounds.  Instead of cluttering ntop with a bunch of #ifdef logic,
    * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
@@ -130,8 +128,12 @@ extern int getDynamicLoadPaths(char *main, int mainLen, char *lib, int libLen, c
 extern void extend8021Qmtu(void);
 extern void initNtopGlobals(int argc, char * argv[], int argc_started, char * argv_started[]);
 extern void initNtop(char *devices);
+#if defined(MEMORY_DEBUG)
+extern void resetLeaks();
+#endif
 
 /* address.c */
+extern void initAddressResolution(void);
 extern int printable(int ch);
 extern void cleanupAddressQueue(void);
 extern void* dequeueAddress(void* notUsed);
@@ -146,7 +148,8 @@ extern char * _addrtostr(HostAddr *addr, char* buf, u_short bufLen);
 extern char * _addrtonum(HostAddr *addr, char* buf, u_short bufLen);
 extern char * addrtostr(HostAddr *addr);
 extern int fetchAddressFromCache(HostAddr hostIpAddress, char *buffer, int *type);
-extern void ipaddr2str(HostAddr hostIpAddress, int updateHost);
+extern void ipaddr2str(HostTraffic *el, HostAddr hostIpAddress, 
+		       short vlanId, u_int actualDeviceId);
 extern char* etheraddr_string(const u_char *ep, char *buf);
 extern char* llcsap_string(u_char sap);
 extern void extract_fddi_addrs(struct fddi_header *fddip, char *fsrc,
@@ -193,16 +196,27 @@ extern void clearUserUrlList(void);
 extern char* _formatTime(time_t *theTime, char *outStr, int outStrLen, char *file, int line);
 #define formatTime(t, o, l) _formatTime(t, o, l, __FILE__, __LINE__)
 
+#ifdef ENABLE_FC
 /* fcUtils.c */
 extern HostTraffic* allocFcScsiCounters(HostTraffic *host);
+#endif
+
+/* python.c */
+#ifdef HAVE_PYTHON
+extern void init_python(int argc, char *argv[]);
+extern void term_python(void);
+#endif
 
 /* hash.c */
 extern u_int hashHost(HostAddr *hostIpAddress,  u_char *ether_addr,
 		      short* useIPAddressForSearching, HostTraffic **el, int actualDeviceId);
+#ifdef ENABLE_FC
 extern u_int hashFcHost(FcAddress *fcAddress, u_short vsanId,
 			HostTraffic **el, int actualDeviceId);
+#endif
 extern void freeHostInfo(HostTraffic *host, int actualDeviceId);
 extern void freeHostInstances(int actualDeviceId);
+extern void readSessionPurgeParams(void);
 extern int is_host_ready_to_purge(int actDevice, HostTraffic *el, time_t now);
 extern int purgeIdleHosts(int devId);
 extern void setHostSerial(HostTraffic *el);
@@ -211,8 +225,9 @@ extern HostTraffic * _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr,
 				 u_char forceUsingIPaddress, int actualDeviceId,
 				 char *file, int line);
 #define lookupHost(a, b, c, d, e, f) _lookupHost(a, b, c, d, e, f, __FILE__, __LINE__)
-extern HostTraffic * lookupFcHost(FcAddress *fcAddress, u_short vsanId,
-				  int actualDeviceId);
+#ifdef ENABLE_FC
+extern HostTraffic * lookupFcHost(FcAddress *fcAddress, u_short vsanId, int actualDeviceId);
+#endif
 extern void add_valid_ptr(void* ptr);
 extern void remove_valid_ptr(void* ptr);
 extern int is_valid_ptr(void* ptr);
@@ -466,6 +481,7 @@ extern int isInitialFtpData(char* packetData);
 extern char* findHostCommunity(u_int32_t host_ip, char *buf, u_short buf_len);
 extern void setHostCommunity(HostTraffic *el);
 extern void updateDeviceThpt(int deviceToUpdate, int quickUpdate);
+extern void checkCommunities(void);
 
 /* prefs.c */
 extern void loadPrefs(int argc, char* argv[]);
@@ -502,16 +518,19 @@ extern int ntop_conditional_sched_yield(void);
 extern HostTraffic* findHostByNumIP(HostAddr hostIpAddress, short vlanId, u_int actualDeviceId);
 extern HostTraffic* findHostBySerial(HostSerial serial, u_int actualDeviceId);
 extern HostTraffic* findHostByMAC(char* macAddr, short vlanId, u_int actualDeviceId);
+#ifdef ENABLE_FC
 extern HostTraffic* findHostByFcAddress(FcAddress *fcAddr, u_short vsanId, u_int actualDeviceId);
 extern FcNameServerCacheEntry *findFcHostNSCacheEntry(FcAddress *fcAddr, u_short vsanId);
 extern char* fc_to_str(const u_int8_t *ad);
 extern char* fcwwn_to_str(const u_int8_t *ad);
+#endif
 #ifdef INET6
 extern unsigned long in6_hash(struct in6_addr *addr);
 extern int in6_isglobal(struct in6_addr *addr);
 extern unsigned short prefixlookup(struct in6_addr *addr, NtopIfaceAddr *addrs, int size);
 extern unsigned short addrlookup(struct in6_addr *addr,  NtopIfaceAddr *addrs);
 extern NtopIfaceAddr *getLocalHostAddressv6(NtopIfaceAddr *addrs, char* device);
+extern void maximize_socket_buffer(int sock_fd, int buf_type); 
 extern unsigned short isLinkLocalAddress(struct in6_addr *addr,
 					 u_int32_t *the_local_network, 
 					 u_int32_t *the_local_network_mask);
@@ -596,7 +615,7 @@ extern unsigned short isPseudoBroadcastAddress(HostAddr *addr,
 					       u_int32_t *the_local_network_mask);
 extern void printLogTime(void);
 extern int32_t gmt2local(time_t t);
-extern char *dotToSlash(char *name);
+extern char *dotToSlash(char *name, char *buf, int buf_len);
 extern int getLocalHostAddress(struct in_addr *hostIpAddress, u_int8_t *netmask_v6, char* device);
 extern NtopIfaceAddr * getLocalHostAddressv6(NtopIfaceAddr *addrs, char* device);
 extern void fillDomainName(HostTraffic *el);
@@ -784,7 +803,6 @@ extern int snprintf(char *str, size_t n, const char *fmt, ...);
 extern int getopt_long(int ___argc, char *const *___argv,
 		       const char *__shortopts,
 		       const struct option *__longopts, int *__longind);
-extern int getopt_long_only();
 #endif /* HAVE_GETOPT_H */
 
 #ifndef HAVE_BUILDARGV
@@ -804,12 +822,15 @@ extern unsigned short isOKtoSave(u_int32_t addr,
 extern float timeval_subtract(struct timeval x, struct timeval y);
 extern void freePortsUsage(HostTraffic *el);
 extern PortUsage* getPortsUsage(HostTraffic *el, u_int portIdx, int createIfNecessary);
+extern void setHostFlag(int flag_value, HostTraffic *host);
+extern void clearHostFlag(int flag_value, HostTraffic *host);
 extern char* vlan2name(u_int16_t vlanId, char *buf, int buf_len);
 extern void mkdir_p(char *tag, char *path, int permission);
 extern char* read_file(char* path, char* buf, u_int buf_len);
 /* Formatting for %.2f ... */
 #define xvertDOT00MB(v)(((float)(v)/(float)(1024.0*1024.0))+0.005)
 #define xvertDOT00KB(v) (((float)(v)/(float)(1024.0))+0.005)
+void web_sanitize(char *value);
 
 /* vendor.c */
 extern char* getVendorInfo(u_char* ethAddress, short encodeString);
@@ -828,8 +849,10 @@ extern void l7SessionProtoDetection(IPSession *theSession,
 extern u_int _checkSessionIdx(u_int idx, int actualDeviceId, char* file, int line);
 extern void freeSession(IPSession *sessionToPurge, int actualDeviceId, 
 			u_char allocateMemoryIfNeeded, u_char lockMutex);
+#ifdef ENABLE_FC
 extern void freeFcSession(FCSession *sessionToPurge, int actualDeviceId,
 			  u_char allocateMemoryIfNeeded, u_char lockMutex);
+#endif
 extern void scanTimedoutTCPSessions(int actualDeviceId);
 extern void updateUsedPorts(HostTraffic *srcHost, HostTraffic *dstHost,
 			    u_short sport, u_short dport, u_int length);
@@ -853,6 +876,7 @@ extern void updatePeersDelayStats(HostTraffic *peer_a, HostSerial *peer_b_serial
 				  u_char is_client_delay, 
 				  int port_idx);
 extern void updateSessionDelayStats(IPSession* session);
+#ifdef ENABLE_FC
 extern FCSession* handleFcSession(const struct pcap_pkthdr *h,
 				  u_short fragmentedData,
 				  HostTraffic *srcHost, HostTraffic *dstHost,
@@ -860,7 +884,14 @@ extern FCSession* handleFcSession(const struct pcap_pkthdr *h,
 				  u_short rxid, u_short protocol, u_char rCtl,
 				  u_char isXchgOrig, const u_char *bp,
 				  int actualDeviceId);
+#endif
 
+/* event.c */
+extern void notifyEvent(EventType evt, HostTraffic *el, IPSession *session, int eventValue);
+extern void init_events(void);
+
+
+#ifdef ENABLE_FC
 /* fcUtils.c */
 extern int isFlogiAcc(FcAddress *fcAddress, u_int8_t r_ctl, u_int8_t type, u_int8_t cmd);
 extern int fillFcHostInfo(const u_char *bp, HostTraffic *srcHost);
@@ -878,6 +909,7 @@ extern int updateFcFabricElementHash(FcFabricElementHash **theHash, u_short vsan
 				     u_short protocol, u_char r_ctl,
 				     u_int32_t pktlen);
 extern void processFcNSCacheFile(char *filename);
+#endif
 
 extern u_int32_t num_db_insert, num_db_insert_failed;
 extern int is_db_enabled(void);
@@ -1018,7 +1050,8 @@ Code "inherited" from nslookup
 
 #define isDHCPClient(a)             ((a != NULL) && FD_ISSET(FLAG_HOST_TYPE_SVC_DHCP_CLIENT, &(a->flags)))
 #define isDHCPServer(a)             ((a != NULL) && FD_ISSET(FLAG_HOST_TYPE_SVC_DHCP_SERVER, &(a->flags)))
-#define isP2P(a)                    ((a != NULL) && (a->protocolInfo != NULL) && (a->protocolInfo->fileList != NULL))
+#define isP2P(a)                    ((a != NULL) && ((a->totContactedSentPeers > CONTACTED_PEERS_THRESHOLD) || (a->totContactedRcvdPeers > CONTACTED_PEERS_THRESHOLD)))
+     
 #define isNtpServer(a)              ((a != NULL) && FD_ISSET(FLAG_HOST_TYPE_SVC_NTP_SERVER, &(a->flags)))
 
 /* Host health */
@@ -1031,46 +1064,6 @@ Code "inherited" from nslookup
 /* Shorthand, used in traffic.c */
 /* #define getSerial(a) myGlobals.device[deviceToUpdate].hash_hostTraffic[a]->hostSerial */
 
-#ifdef SSLWATCHDOG_DEBUG
-#define sslwatchdogDebug(text, bpcFlag, note) {				\
-    traceEvent(CONST_TRACE_INFO, "SSLWDDEBUG: %1d %-10s %-15s %-15s %s", \
-	       myGlobals.sslwatchdogCondvar.predicate,			\
-	       ((bpcFlag == FLAG_SSLWATCHDOG_BOTH) ? text : ""),	\
-	       ((bpcFlag == FLAG_SSLWATCHDOG_PARENT) ? text : ""),	\
-	       ((bpcFlag == FLAG_SSLWATCHDOG_CHILD) ? text : ""),	\
-	       note);							\
-  }
-#define sslwatchdogDebugN(text, bpcFlag, note) {			\
-    traceEvent(CONST_TRACE_INFO, "SSLWDDEBUG: %1d %-10s %-15s %-15s %d", \
-	       myGlobals.sslwatchdogCondvar.predicate,			\
-	       ((bpcFlag == FLAG_SSLWATCHDOG_BOTH) ? text : ""),	\
-	       ((bpcFlag == FLAG_SSLWATCHDOG_PARENT) ? text : ""),	\
-	       ((bpcFlag == FLAG_SSLWATCHDOG_CHILD) ? text : ""),	\
-	       note);							\
-  }
-#define sslwatchdogError(text, bpcFlag, note) {				\
-    traceEvent(CONST_TRACE_INFO, "SSLWDERROR: %1d %-10s %-15s %-15s %s", \
-	       myGlobals.sslwatchdogCondvar.predicate,			\
-	       ((bpcFlag == FLAG_SSLWATCHDOG_BOTH) ? text : ""),	\
-	       ((bpcFlag == FLAG_SSLWATCHDOG_PARENT) ? text : ""),	\
-	       ((bpcFlag == FLAG_SSLWATCHDOG_CHILD) ? text : ""),	\
-	       note);							\
-  }
-#define sslwatchdogErrorN(text, bpcFlag, note) {			\
-    traceEvent(CONST_TRACE_INFO, "SSLWDERROR: %1d %-10s %-15s %-15s %d", \
-	       myGlobals.sslwatchdogCondvar.predicate,			\
-	       ((bpcFlag == FLAG_SSLWATCHDOG_BOTH) ? text : ""),	\
-	       ((bpcFlag == FLAG_SSLWATCHDOG_PARENT) ? text : ""),	\
-	       ((bpcFlag == FLAG_SSLWATCHDOG_CHILD) ? text : ""),	\
-	       note);							\
-  }
-#else
-#define sslwatchdogDebug(text, bpcFlag, note) {}
-#define sslwatchdogDebugN(text, bpcFlag, note) {}
-#define sslwatchdogError(text, bpsFlag, note) {}
-#define sslwatchdogErrorN(text, bpcFlag, note) {}
-#endif
-
 #define CONST_LLC_U_CMD(u)    ((u) & 0xef)
 #define CONST_LLC_S_CMD(is)   (((is) >> 10) & 0x03)
 #define CONST_LLC_IS_NR(is)   (((is) >> 1) & 0x7f)
@@ -1078,12 +1071,12 @@ Code "inherited" from nslookup
 
 
 #ifndef IN6_IS_ADDR_MULTICAST
-#define IN6_IS_ADDR_MULTICAST(a) (((uint8_t *) (a))[0] == 0xff)
+#define IN6_IS_ADDR_MULTICAST(a) (((u_int8_t *) (a))[0] == 0xff)
 #endif
 
 #ifndef IN6_IS_ADDR_LINKLOCAL
 #define IN6_IS_ADDR_LINKLOCAL(a)		\
-  ((((uint32_t *) (a))[0] & htonl (0xffc00000))	\
+  ((((u_int32_t *) (a))[0] & htonl (0xffc00000))	\
    == htonl (0xfe800000))
 #endif
 
