@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2010 Luca Deri <deri@ntop.org>
  *
  *		 	    http://www.ntop.org/
  *
@@ -113,13 +113,17 @@ extern HostTraffic* quickHostLink(HostSerial theSerial, int deviceId, HostTraffi
 extern void printHostContactedPeers(HostTraffic *el, int actualDeviceId);
 extern void printFcHostContactedPeers(HostTraffic *el, int actualDeviceId);
 extern char *getSessionState(IPSession *session);
+#ifdef ENABLE_FC
 extern char *getFcSessionState(FCSession *session);
+#endif
 extern void printHostSessions(HostTraffic *el, int actualDeviceId);
 extern u_short isHostHealthy(HostTraffic *el);
 extern void printHostDetailedInfo(HostTraffic *el, int actualDeviceId);
+#ifdef ENABLE_FC
 extern void printFcHostDetailedInfo(HostTraffic *el, int actualDeviceId);
 extern void printVsanDetailedInfo (u_int vsanId, int actualDeviceId);
 extern void printVsanProtocolStats (FcFabricElementHash *hash, int actualDeviceId);
+#endif
 extern void printServiceStats(char* svcName, ServiceStats* ss, short printSentStats);
 extern void printHostUsedServices(HostTraffic *el, int actualDeviceId);
 extern void printHostIcmpStats(HostTraffic *el);
@@ -140,11 +144,11 @@ extern void dumpNtopTrafficInfo(FILE*, char* options);
 extern void dumpNtopTrafficMatrix(FILE *fDescr, char* options, int actualDeviceId);
 extern void checkHostProvidedServices(HostTraffic *el);
 extern void printHostsStats(int fingerprintRemote);
-
+#ifdef ENABLE_FC
 extern FcFabricElementHash *getFcFabricElementHash (u_short vsanId, int actualDeviceId);
 extern void dumpFcFabricElementHash (FcFabricElementHash **theHash, char* label,
                                      u_char dumpLoopbackTraffic, u_char vsanHash);
-
+#endif
 extern void printMutexStatus(int textPrintFlag, PthreadMutex *mutexId, char *mutexName);
 
 extern void printPluginTrailer(char *left, char *middle);
@@ -176,7 +180,10 @@ extern char* printSSLError(int errorId);
 #endif /* HAVE_OPENSSL */
 extern int isAllowedCommunity(char *community_name);
 extern void sendHTTPHeader(int mimeType, int headerFlags, int useCompressionIfAvailable);
+extern void returnHTTPnotImplemented(void);
 extern int readHTTPpostData(int len, char *buf, int buflen);
+extern void returnHTTPversionServerError(void);
+extern void returnHTTPpageBadCommunity(void);
 
 /* report.c */
 extern void printBandwidthFooter(void);
@@ -192,13 +199,8 @@ extern int combineReportTypeLocality(int reportTypeReq, LocalityDisplayPolicy sh
 extern void printHostsTraffic(int reportType, int sortedColumn, int revertOrder,
 			      int pageNum, char* url, HostsDisplayPolicy showHostsMode,
                               LocalityDisplayPolicy showLocalityMode, int vlanId);
-extern void printFcHostsTraffic(int reportType,
-			      int sortedColumn, int revertOrder,
-			      int pageNum, char* url,
-                              LocalityDisplayPolicy showLocalityMode);
 extern void printMulticastStats(int sortedColumn /* ignored so far */,
                                 int revertOrder, int pageNum);
-extern void printVSANList(unsigned int deviceId);
 extern void printHostsInfo(int sortedColumn, int revertOrder, int pageNum, 
 			   int showBytes, int vlanId, int ifId, int knownSubnetId);
 extern void makeDot(void);
@@ -216,6 +218,16 @@ extern void printIpAccounting(int remoteToLocal, int sortedColumn,
 extern void printFcAccounting(int remoteToLocal, int sortedColumn,
 			      int revertOrder, int pageNum);
 extern void printActiveTCPSessions(int actualDeviceId, int pageNum, HostTraffic *el);
+extern void printIpProtocolUsage(void);
+extern void printBar(char *buf, int bufLen, unsigned short percentageS, unsigned short percentageR,
+                     unsigned short maxPercentage, unsigned short ratio);
+extern void printIpProtocolDistribution(int mode, int revertOrder, int printGraph);
+
+#ifdef ENABLE_FC
+extern void printFcProtocolDistribution(int mode, int revertOrder, int printGraph);
+extern void printFcTrafficMatrix (u_short vsanId, u_char sent);
+extern void drawVsanStatsGraph (unsigned int deviceId);
+extern void printVSANList(unsigned int deviceId);
 extern void printFCSessions(int actualDeviceId, int sortedColumn, int revertOrder,
                             int pageNum, char *url, HostTraffic *el);
 extern int printScsiSessionBytes (int actualDeviceId, int sortedColumn,
@@ -235,18 +247,19 @@ extern void printScsiLunStats (HostTraffic *el, int actualDeviceId,
                                int pageNum, char *lun);
 extern void printScsiLunGraphs (HostTraffic *el, int actualDeviceId);
 extern void printFcTrafficSummary (u_short vsanId);
-extern void printIpProtocolUsage(void);
-extern void printBar(char *buf, int bufLen, unsigned short percentageS, unsigned short percentageR,
-                     unsigned short maxPercentage, unsigned short ratio);
-extern void printIpProtocolDistribution(int mode, int revertOrder, int printGraph);
-extern void printFcProtocolDistribution(int mode, int revertOrder, int printGraph);
+extern void printFcHostsTraffic(int reportType,
+			      int sortedColumn, int revertOrder,
+			      int pageNum, char* url,
+                              LocalityDisplayPolicy showLocalityMode);
+extern void printVSANList(unsigned int deviceId);
+#endif
+
 extern void printProtoTraffic(int printGraph);
 extern void printProcessInfo(int processPid, int actualReportDeviceId);
 extern void printIpTrafficMatrix(void);
-extern void printFcTrafficMatrix (u_short vsanId, u_char sent);
 extern void printThptStatsMatrix(int sortedColumn);
 extern void printThptStats(int sortedColumn);
-extern void printDomainStats(char* domain_domain_name, int network_mode, int clusterMode, 
+extern void printDomainStats(char* domain_domain_name, int network_mode, int communityMode, 
 			     int sortedColumn, int revertOrder, int pageNum);
 extern void printNoDataYet(void);
 extern void printNotAvailable(char* flagName);
@@ -254,8 +267,6 @@ extern void listNetFlows(void);
 extern void printHostHourlyTraffic(HostTraffic *el);
 extern void printVLANList(unsigned int deviceId);
 extern void showPortTraffic(u_short portNr);
-extern void drawVsanStatsGraph (unsigned int deviceId);
-extern void printVSANList(unsigned int deviceId);
 extern void handleNtopConfig (char *url, UserPrefDisplayPage page, int postLen);
 extern void purgeHost(char *serialized_serial);
 extern void printInterfaceStats(void);
@@ -285,15 +296,6 @@ extern void initSocket(int isSSL, int ipv4or6, int *port, int *sock, char *addr)
 extern bool processNtopPref (char *key, char *value, bool savePref,
                              UserPref *pref);
 
-#ifdef MAKE_WITH_SSLWATCHDOG
-extern int sslwatchdogWaitFor(int stateValue, int parentchildFlag, int alreadyLockedFlag);
-extern int sslwatchdogClearLock(int parentchildFlag);
-extern int sslwatchdogGetLock(int parentchildFlag);
-extern int sslwatchdogSignal(int parentchildFlag);
-extern int sslwatchdogSetState(int stateNewValue, int parentchildFlag, int enterLockedFlag, int exitLockedFlag);
-extern void sslwatchdogSighandler(int signum);
-extern void* sslwatchdogChildThread(void* notUsed _UNUSED_);
-#endif
 extern int printNtopLogReport(int printAsText);
 extern char* makeHostAgeStyleSpec(HostTraffic *el, char *buf, int bufSize);
 extern void edit_prefs(char *db_key, char *db_val);
@@ -328,9 +330,11 @@ extern char *getRowColor(void);
 extern char *makeHostLink(HostTraffic *el, short mode,
                           short cutName, short addCountryFlag,
 			  char *buf, int bufLen);
+#ifdef ENABLE_FC
 extern char *makeFcHostLink (HostTraffic *el, short mode, short cutName,
                              short compactWWN, char *buf, int buflen);
 extern char *makeVsanLink (u_short vsanId, short mode, char *buf, int buflen);
+#endif
 extern void  buildMapLink(HostTraffic *el, char *buf, int buf_len);
 extern char *getHostName(HostTraffic *el, short cutName, char *buf, int bufLen);
 
@@ -353,6 +357,7 @@ extern void drawThptGraph(int sortedColumn);
 extern void drawGlobalProtoDistribution(void);
 extern int  drawHostsDistanceGraph(int);
 extern void drawGlobalIpProtoDistribution(void);
+#ifdef ENABLE_FC
 extern void drawGlobalFcProtoDistribution(void);
 extern void drawLunStatsBytesDistribution(HostTraffic *el);
 extern void drawLunStatsPktsDistribution(HostTraffic *el);
@@ -360,12 +365,12 @@ extern void drawVsanStatsBytesDistribution(int deviceId);
 extern void drawVsanStatsPktsDistribution(int deviceId);
 extern void drawVsanDomainTrafficDistribution(u_short vsanId, u_char dataSent);
 extern void drawVsanSwilsProtoDistribution(u_short vsanId);
+#endif
 
-/* perl.c */
-extern int handlePerlHTTPRequest(char *url);
-
-/* lua.c */
-extern int handleLuaHTTPRequest(char *url);
+/* python.c */
+#ifdef HAVE_PYTHON
+extern int handlePythonHTTPRequest(char *url, u_int postLen);
+#endif
 
 /* map.c */
 extern void init_maps(void);
