@@ -2,7 +2,7 @@
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *                          http://www.ntop.org
  *
- *         Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
+ *         Copyright (C) 1998-2010 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -53,14 +53,14 @@ typedef unsigned short u_short;
 typedef unsigned int   u_int;
 typedef unsigned long  u_long;
 #endif
-#if !defined(HAVE_UINT8_T)
-typedef u_char  uint8_t;
+#if !defined(HAVE_u_int8_T)
+typedef u_char  u_int8_t;
 #endif
-#if !defined(HAVE_UINT16_T)
-typedef u_short uint16_t;
+#if !defined(HAVE_u_int16_T)
+typedef u_short u_int16_t;
 #endif
-#if !defined(HAVE_UINT32_T)
-typedef u_int   uint32_t;
+#if !defined(HAVE_u_int32_T)
+typedef u_int   u_int32_t;
 #endif
 #endif /* WIN32 */
 
@@ -68,8 +68,8 @@ typedef u_int   uint32_t;
 #if defined(WIN32)
 /* typedef _int64 u_int64_t; */
 #else
-#if defined(HAVE_UINT64_T)
-#define u_int64_t uint64_t
+#if defined(HAVE_u_int64_T)
+#define u_int64_t u_int64_t
 #else
 #error "Sorry, I'm unable to define u_int64_t on your platform"
 #endif
@@ -104,8 +104,10 @@ typedef char int8_t;
 #define bool u_int8_t
 #endif
 
+#ifdef ENABLE_FC
 #include "fcUtils.h"
 #include "scsiUtils.h"
+#endif
 
 typedef struct ether80211q {
   u_int16_t vlanId;
@@ -116,6 +118,7 @@ typedef struct _mac_t {
     u_int8_t mact_octet[6];
 } mac_t;
 
+#ifdef ENABLE_FC
 typedef struct fcAddr {
     u_int8_t domain;
     u_int8_t area;
@@ -190,6 +193,7 @@ typedef union wwn_ {
 #endif
   u_int64_t num;
 } wwn_t;
+#endif
 
 typedef struct hostAddr {
   u_int    hostFamily; /* AF_INET AF_INET6 */
@@ -229,7 +233,9 @@ typedef struct hostSerial {
   union {
     EthSerial ethSerial; /* hostSerial == SERIAL_MAC */
     IpSerial  ipSerial;  /* hostSerial == SERIAL_IPV4/SERIAL_IPV6 */
+#ifdef ENABLE_FC
     FcSerial  fcSerial;
+#endif
   } value;
 } HostSerial;
 
@@ -299,6 +305,17 @@ typedef struct protocolsList {
   struct protocolsList *next;
 } ProtocolsList;
 
+#ifndef HAVE_RW_LOCK
+#ifndef WIN32
+#define pthread_rwlock_t         pthread_mutex_t
+#define pthread_rwlock_init      pthread_mutex_init
+#define pthread_rwlock_wrlock    pthread_mutex_lock
+#define pthread_rwlock_unlock    pthread_mutex_unlock
+#define pthread_rwlock_destroy   pthread_mutex_destroy
+#define pthread_rwlock_trywrlock pthread_mutex_trywrlock
+#endif
+#endif
+
 #ifndef WIN32
 
 typedef struct conditionalVariable {
@@ -312,6 +329,7 @@ typedef struct conditionalVariable {
 #define pthread_t              HANDLE
 #define pthread_mutex_t        HANDLE
 #define pthread_cond_t         HANDLE
+#define pthread_rwlock_t       HANDLE
 
 typedef struct conditionalVariable {
   HANDLE condVar;
@@ -326,7 +344,6 @@ extern int  pthread_mutex_lock(pthread_mutex_t *mutex);
 extern int  pthread_mutex_trylock(pthread_mutex_t *mutex);
 extern int  pthread_mutex_unlock(pthread_mutex_t *mutex);
 
-
 #endif /* WIN32 */
 
 typedef struct holder {
@@ -337,15 +354,19 @@ typedef struct holder {
   char   file[5];
 } Holder;
 
+
 typedef struct pthreadMutex {
+  u_int8_t isInitialized;
+
+#ifdef MUTEX_DEBUG
   pthread_mutex_t mutex, statedatamutex;
-  char   isLocked, isInitialized;
+  u_int8_t isLocked;
   u_int  numLocks, numReleases;
-  Holder attempt,
-         lock,
-         unlock,
-         max;
+  Holder attempt, lock, unlock, max;
   float  maxLockedDuration;
+#else
+  pthread_rwlock_t mutex;
+#endif
 } PthreadMutex;
 
 typedef struct packetInformation {
@@ -398,12 +419,14 @@ typedef struct packetStats {
 
 /* *********************** */
 
+#ifdef ENABLE_FC
 typedef struct fcPacketStats {
   TrafficCounter upTo36, upTo48, upTo52, upTo68, upTo104;
   TrafficCounter upTo548, upTo1060, upTo2136, above2136;
   TrafficCounter shortest, longest;
   TrafficCounter badCRC, tooLong;
 } FcPacketStats;
+#endif
 
 /* *********************** */
 
@@ -563,6 +586,7 @@ typedef struct userList {
 /* *********************** */
 
 typedef struct fileList {
+  pcap_t *pcapPtr;
   char *fileName;
   fd_set fileFlags;
   struct fileList *next;
@@ -629,8 +653,7 @@ typedef struct protocolInfo {
   VirtualHostList *httpVirtualHosts;
   /* POP3/SMTP... */
   UserList *userList;
-  /* P2P */
-  FileList *fileList;
+
   ServiceStats     *dnsStats, *httpStats;
   DHCPStats        *dhcpStats;
 } ProtocolInfo;
@@ -646,8 +669,11 @@ typedef struct shortProtoTrafficInfo {
 typedef struct protoTrafficInfo {
   TrafficCounter sentLoc, sentRem;
   TrafficCounter rcvdLoc, rcvdFromRem;
-  TrafficCounter pktSent, pktRcvd, efficiencySent, efficiencyRcvd;
+  TrafficCounter pktSent, pktRcvd;
+#ifdef ENABLE_EFFICIENCY
+  TrafficCounter efficiencySent, efficiencyRcvd;
   TrafficCounter lastEfficiencyPktSent, lastEfficiencyPktRcvd; /* Used by the RRD plugin */
+#endif
   TrafficCounter totalFlows;
 } ProtoTrafficInfo;
 
@@ -664,6 +690,7 @@ typedef struct nonIpProtoTrafficInfo {
 
 /* *********************** */
 
+#ifdef ENABLE_FC
 /* ************************************* */
 /*         SCSI-specific structures      */
 /* ************************************* */
@@ -806,6 +833,7 @@ typedef struct fcScsiCounters {
   /* SCSI Counters */
   TrafficCounter   scsiReadBytes, scsiWriteBytes, scsiOtherBytes;
 } FcScsiCounters;
+#endif
 
 /* **************************** */
 
@@ -850,18 +878,21 @@ typedef struct hostTraffic {
   struct timeval   minLatency, maxLatency;
   GeoIPRecord      *geo_ip;
 
-  TrafficCounter   greSent, greRcvd, greEfficiencySent, greEfficiencyRcvd,
-    grePktSent, grePktRcvd, lastGrePktSent, lastGrePktRcvd;
-  TrafficCounter   ipsecSent, ipsecRcvd, ipsecEfficiencySent, ipsecEfficiencyRcvd,
-    ipsecPktSent, ipsecPktRcvd, lastIpsecPktSent, lastIpsecPktRcvd;
+#ifdef ENABLE_EFFICIENCY
+  TrafficCounter greEfficiencySent, greEfficiencyRcvd, ipsecEfficiencySent, ipsecEfficiencyRcvd,
+    efficiencySent, efficiencyRcvd, lastEfficiencyPktSent, lastEfficiencyPktRcvd;
+#endif
+
+  TrafficCounter   greSent, greRcvd, grePktSent, grePktRcvd, lastGrePktSent, lastGrePktRcvd;
+  TrafficCounter   ipsecSent, ipsecRcvd, ipsecPktSent, ipsecPktRcvd, lastIpsecPktSent, lastIpsecPktRcvd;
 
   NonIPTraffic     *nonIPTraffic;
   NonIpProtoTrafficInfo *nonIpProtoTrafficInfos; /* Info about further non IP protos */
 
   fd_set           flags;
   TrafficCounter   pktSent, pktRcvd, pktSentSession, pktRcvdSession,
-    pktDuplicatedAckSent, pktDuplicatedAckRcvd, efficiencySent, efficiencyRcvd;
-  TrafficCounter   lastThptPktSent, lastThptPktRcvd, lastEfficiencyPktSent, lastEfficiencyPktRcvd;
+    pktDuplicatedAckSent, pktDuplicatedAckRcvd;
+  TrafficCounter   lastThptPktSent, lastThptPktRcvd;
   TrafficCounter   pktBroadcastSent, bytesBroadcastSent;
   TrafficCounter   pktMulticastSent, bytesMulticastSent,
     pktMulticastRcvd, bytesMulticastRcvd;
@@ -908,8 +939,10 @@ typedef struct hostTraffic {
   ShortProtoTrafficInfo **ipProtosList;        /* List of myGlobals.numIpProtosList entries */
   ProtoTrafficInfo      **protoIPTrafficInfos; /* Info about IP traffic generated/rcvd by this host */
 
+#ifdef ENABLE_FC
   /* Fiber Channel/SCSI */
   FcScsiCounters   *fcCounters;
+#endif
 
   Counter          totContactedSentPeers, totContactedRcvdPeers; /* # of different contacted peers */
   UsageCounter     contactedSentPeers;   /* peers that talked with this host */
@@ -922,7 +955,7 @@ typedef struct hostTraffic {
 
 typedef struct domainStats {
   HostTraffic *domainHost; /* ptr to a host that belongs to the domain */
-  char *clusterName;
+  char *communityName;
   int8_t known_subnet_id;
   TrafficCounter bytesSent, bytesRcvd;
   TrafficCounter tcpSent, udpSent;
@@ -1009,6 +1042,8 @@ typedef struct ipSession {
 
 /* ************************************* */
 
+#ifdef ENABLE_FC
+
 /* FC Session Information */
 typedef struct fcSession {
   u_short magic;
@@ -1051,6 +1086,7 @@ typedef struct scsiSessionSortEntry {
     u_short lun;
     ScsiLunTrafficInfo *stats;
 } ScsiSessionSortEntry;
+#endif
 
 typedef struct ntopIfaceAddrInet {
   struct in_addr ifAddr;
@@ -1073,7 +1109,7 @@ typedef struct ntopIfaceaddr{
 #ifdef INET6
     NtopIfaceAddrInet6 inet6;
 #endif
-  }af;
+  } af;
 } NtopIfaceAddr;
 
 /* ************************************* */
@@ -1087,11 +1123,31 @@ typedef enum {
   asAggregation
 } AggregationType;
 
+typedef enum {
+  noDnsResolution = 0,
+  dnsResolutionForLocalHostsOnly = 1,
+  dnsResolutionForLocalRemoteOnly = 2,
+  dnsResolutionForAll = 3
+} DnsResolutionMode;
+
 typedef struct probeInfo {
   struct in_addr probeAddr;
+  u_int16_t probePort;
   u_int32_t      pkts;
+  u_int32_t lastSequenceNumber, lowestSequenceNumber, highestSequenceNumber, totNumFlows;
+  u_int32_t lostFlows;
 } ProbeInfo;
 
+/* Flow aggregation */
+typedef enum {
+  hostCreation = 1,
+  hostDeletion = 1 << 2,
+  sessionCreation = 1 << 3,
+  sessionDeletion = 1 << 4,
+  hostFlagged = 1 << 5,
+  hostUnflagged  = 1 << 6
+} EventType;
+
 /* *************************** */
 
 #define CONST_FLOW_VERSION_1		    1
@@ -1232,8 +1288,19 @@ typedef struct single_flow_ver7_rec {
 
 /* NetFlow v9/IPFIX */
 
+typedef struct flow_set {
+  u_int16_t templateId;
+  u_int16_t fieldCount;
+} FlowSet;
+
+typedef struct flow_ipfix_template_field {
+  u_int16_t fieldType;
+  u_int16_t fieldLen;
+  u_int8_t  isPenField;
+} V9V10TemplateField;
+
 typedef struct flow_ver9_hdr {
-  u_int16_t version;         /* Current version=9*/
+  u_int16_t version;         /* Current version=9 */
   u_int16_t count;           /* The number of records in PDU. */
   u_int32_t sysUptime;       /* Current time in msecs since router booted */
   u_int32_t unix_secs;       /* Current seconds since 0000 UTC 1970 */
@@ -1241,17 +1308,36 @@ typedef struct flow_ver9_hdr {
   u_int32_t sourceId;        /* Source id */
 } V9FlowHeader;
 
+typedef struct flow_ipfix_hdr {
+  u_int16_t version;         /* Current version=10 */
+  u_int16_t length;          /* The flow length (bytes) */
+  u_int32_t sysUptime;       /* Current time in msecs since router booted */
+  u_int32_t flow_sequence;   /* Sequence number of total flows seen */
+  u_int32_t domainId;        /* Observation domain id */
+} IPFIXFlowHeader;
+
 typedef struct flow_ver9_template_field {
   u_int16_t fieldType;
   u_int16_t fieldLen;
 } V9TemplateField;
 
-typedef struct flow_ver9_template {
+typedef struct flow_ver9_template_header {
   u_int16_t templateFlowset; /* = 0 */
   u_int16_t flowsetLen;
+} V9TemplateHeader;
+
+typedef struct flow_ver9_template_def {
   u_int16_t templateId;
   u_int16_t fieldCount;
-} V9Template;
+} V9TemplateDef;
+
+typedef struct flow_ver9_template {
+  /* V9TemplateHeader */
+  u_int16_t flowsetLen;
+  /* V9TemplateDef */
+  u_int16_t templateId;
+  u_int16_t fieldCount;
+} V9SimpleTemplate;
 
 typedef struct flow_ver9_flow_set {
   u_int16_t templateId;
@@ -1269,13 +1355,15 @@ typedef struct flow_ver9_templateids {
 #define NUM_TEMPLATES 88
 
 typedef struct flowSetV9 {
-  V9Template templateInfo;
-  V9TemplateField *fields;
+  V9SimpleTemplate templateInfo;
+  u_int16_t flowLen; /* Real flow length */
+  V9V10TemplateField *fields;
   struct flowSetV9 *next;
 } FlowSetV9;
 
 typedef struct interfaceStats {
   u_int32_t netflow_device_ip;
+  u_int16_t netflow_device_port;
   u_short interface_id;
   char interface_name[32];
   TrafficCounter inBytes, outBytes, inPkts, outPkts;
@@ -1500,7 +1588,7 @@ typedef struct ntopInterface {
 
   int fd;                        /* unique identifier (Unix file descriptor) */
 
-  PthreadMutex asMutex;
+  PthreadMutex asMutex, counterMutex;
   AsStats *asStats;
 
   /*
@@ -1525,11 +1613,13 @@ typedef struct ntopInterface {
   TrafficCounter multicastPkts;   /* # of multicast pkts captured by the application */
   TrafficCounter ipPkts;          /* # of IP pkts captured by the application */
 
+#ifdef ENABLE_FC
   TrafficCounter fcPkts;
   TrafficCounter fcEofaPkts;
   TrafficCounter fcEofAbnormalPkts;
   TrafficCounter fcAbnormalPkts;
   TrafficCounter fcBroadcastPkts;
+#endif
 
   /*
    * The bytes section
@@ -1556,6 +1646,7 @@ typedef struct ntopInterface {
   TrafficCounter otherBytes;
   TrafficCounter *ipProtosList;        /* List of myGlobals.numIpProtosList entries */
 
+#ifdef ENABLE_FC
   TrafficCounter fcBytes;
   TrafficCounter fragmentedFcBytes;
   TrafficCounter fcFcpBytes;
@@ -1567,6 +1658,7 @@ typedef struct ntopInterface {
   TrafficCounter otherFcBytes;
   TrafficCounter fcBroadcastBytes;
   TrafficCounter class2Bytes, class3Bytes, classFBytes;
+#endif
 
   PortCounter    **ipPorts; /* [MAX_IP_PORT] */
 
@@ -1588,7 +1680,9 @@ typedef struct ntopInterface {
   TrafficCounter lastNonIpBytes;
 
   PacketStats rcvdPktStats; /* statistics from start of the run to time of call */
+#ifdef ENABLE_FC
   FcPacketStats rcvdFcPktStats; /* statistics from start of the run to time of call */
+#endif
   TTLstats    rcvdPktTTLStats;
 
   float peakThroughput, actualThpt, lastMinThpt, lastFiveMinsThpt;
@@ -1618,7 +1712,9 @@ typedef struct ntopInterface {
 
   u_short hashListMaxLookups;
 
+#ifdef ENABLE_FC
   FcFabricElementHash **vsanHash;
+#endif
 
   /* ************************** */
 
@@ -1631,10 +1727,12 @@ typedef struct ntopInterface {
 
   /* ************************** */
 
+#ifdef ENABLE_FC
   struct fcSession **fcSession;
   u_short numFcSessions, maxNumFcSessions;
   TrafficEntry** fcTrafficMatrix; /* Subnet traffic Matrix */
   struct hostTraffic** fcTrafficMatrixHosts; /* Subnet traffic Matrix Hosts */
+#endif
 
   NetFlowGlobals *netflowGlobals;  /* NetFlow */
   SflowGlobals   *sflowGlobals;    /* sFlow */
@@ -2073,7 +2171,7 @@ typedef struct _userPref {
   bool enableOtherPacketDump;    /* -j | --create-other-packets */
   char *pcapLog;                 /* -l | --pcap-log */
   char *localAddresses;          /* -m | --local-subnets */
-  bool numericFlag;              /* -n | --numeric-ip-addresses */
+  DnsResolutionMode numericFlag; /* -n | --numeric-ip-addresses */
   bool dontTrustMACaddr;         /* -o | --no-mac */
   char *protoSpecs;              /* -p | --protocols */
   bool enableSuspiciousPacketDump;  /* -q | --create-suspicious-packets */
@@ -2084,7 +2182,10 @@ typedef struct _userPref {
   u_int     maxNumHashEntries;   /* -x */
   u_int     maxNumSessions;      /* -X */
   u_int8_t  liveMode;            /* --live */
+
+#ifdef ENABLE_EFFICIENCY
   u_int8_t  calculateEfficiency; /* --enable-efficiency */
+#endif
   
    /* SQL Database */
   char    sqlDbConfig[64];
@@ -2110,21 +2211,16 @@ typedef struct _userPref {
   bool mergeInterfaces;          /* -M | --no-interface-merge */
   bool enableL7;                 /* Enable/disable l7 protocol pattern matching */
   char *pcapLogBasePath;         /* -O | --pcap-file-path */  /* Added by Ola Lundqvist <opal@debian.org>. */
+
+#ifdef ENABLE_FC
   char *fcNSCacheFile;           /* -N | --wwn-map */
+#endif
 
 #ifdef HAVE_OPENSSL
   char *sslAddr;                 /* -W | --https-serveraddress[:port] */
   int  sslPort;
 #endif
 
-#ifdef MAKE_WITH_SSLWATCHDOG_RUNTIME
-  bool useSSLwatchdog;           /* --ssl-watchdog '133' */
-#endif
-
-#ifdef MAKE_WITH_SCHED_YIELD
-  bool disableSchedYield;        /* --disable-schedyield '134' */
-#endif
-
   bool w3c;                      /* --w3c '136' */
 
   char *P3Pcp;                   /* --p3p-cp '137' */
@@ -2194,7 +2290,7 @@ typedef struct ntopGlobals {
   NtopInterface *device;   /* pointer to the network interfaces table */
 
   /* Database */
-  GDBM_FILE dnsCacheFile, pwFile, prefsFile, macPrefixFile, fingerprintFile;
+  GDBM_FILE pwFile, prefsFile, macPrefixFile, fingerprintFile;
 
   /* the table of broadcast entries */
   HostTraffic *broadcastEntry;
@@ -2248,6 +2344,7 @@ typedef struct ntopGlobals {
   PthreadMutex addressResolutionMutex;
   u_int numDequeueAddressThreads;
   pthread_t dequeueAddressThreadId[MAX_NUM_DEQUEUE_ADDRESS_THREADS];
+  ConditionalVariable queueAddressCondvar;
 
   /*
    * Control mutexes
@@ -2271,12 +2368,6 @@ typedef struct ntopGlobals {
   SSL_CTX* ctx;
   SSL_connection ssl[MAX_SSL_CONNECTIONS];
 
-#ifdef MAKE_WITH_SSLWATCHDOG
-  /* sslwatchdog stuff... */
-  ConditionalVariable sslwatchdogCondvar;
-  pthread_t sslwatchdogChildThreadId;
-#endif
-
 #endif /* HAVE_OPENSSL */
 
   /* ntop state - see flags in globals-defines.h */
@@ -2294,6 +2385,8 @@ typedef struct ntopGlobals {
     dnsSniffARPACount,
     dnsSniffStoredInCache;
 
+  u_int addressQueuedCurrent, addressQueuedMax, addressUnresolvedDrops, resolvedAddresses, failedResolvedAddresses;
+
 #ifdef PARM_USE_HOST
   u_long  numResolvedFromHostAddresses;
 #endif
@@ -2377,9 +2470,6 @@ typedef struct ntopGlobals {
   unsigned long numUnsuccessfulNotfound[2];
   unsigned long numUnsuccessfulDenied[2];
   unsigned long numUnsuccessfulForbidden[2];
-#ifdef MAKE_WITH_SSLWATCHDOG
-  unsigned long numHTTPSrequestTimeouts;
-#endif
   unsigned long numSSIRequests,
                 numBadSSIRequests,
                 numHandledSSIRequests;
@@ -2398,7 +2488,7 @@ typedef struct ntopGlobals {
 #endif
 
   /* Peer2Peer Protocol Indexes */
-  u_short GnutellaIdx, KazaaIdx, WinMXIdx, DirectConnectIdx, FTPIdx, EdonkeyIdx, BitTorrentIdx, VoipIdx;
+  u_short GnutellaIdx, KazaaIdx, FTPIdx, EdonkeyIdx, BitTorrentIdx, VoipIdx;
 
   /* Hash table collisions - counted during load */
   int ipxsapHashLoadCollisions;
@@ -2426,11 +2516,13 @@ typedef struct ntopGlobals {
   int logViewNext;
   PthreadMutex logViewMutex;
 
+#ifdef ENABLE_FC
   /* SCSI */
   char scsiDefaultDevType;
   char displayOption;
   FcNameServerCacheEntry **fcnsCacheHash;
   u_int32_t fcMatrixHashCollisions, fcMatrixHashUnresCollisions;
+#endif
 
   int multipleVLANedHostCount;
 
@@ -2453,6 +2545,12 @@ typedef struct ntopGlobals {
 
   /* GeoIP */
   GeoIP *geo_ip_db, *geo_ip_asn_db;
+  PthreadMutex geolocalizationMutex;
 
+  /* Event Handling */
+  u_int32_t event_mask;
+  char *event_log;
+
+  /* RRD */
+  time_t rrdTime;
 } NtopGlobals;
-
