@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2010 Luca Deri <deri@ntop.org>
  *
  *  			    http://www.ntop.org/
  *
@@ -73,7 +73,7 @@ static void send_graph_header(u_char is_pie) {
  
 /**********************************************************/
 
-static void send_graph_middle() {
+static void send_graph_middle(void) {
 sendString("]\n"
 	     "   };\n"
 	     "\n"
@@ -1078,6 +1078,7 @@ int drawHostsDistanceGraph(int checkOnly) {
 
 /* ************************ */
 
+#ifdef ENABLE_FC
 void hostFcTrafficDistrib(HostTraffic *theHost, short dataSent) {
   float p[MAX_NUM_PROTOS];
   char	*lbl[] = { "", "", "", "", "", "", "", "", "",
@@ -1508,13 +1509,11 @@ void drawVsanSwilsProtoDistribution(u_short vsanId) {
 void drawVsanDomainTrafficDistribution(u_short vsanId, u_char dataSent) {
   int i, idx=0, numEntries = 0;
   FcFabricElementHash *hash;
-  float p[MAX_VSANS_GRAPHED+1];
+  float p[MAX_VSANS_GRAPHED+1] = { 0 };
   char *lbl[MAX_VSANS_GRAPHED+1], labels[MAX_VSANS_GRAPHED+1][8];
   Counter total;
   SortedFcDomainStatsEntry *fcDomainStats;
 
-  p[MAX_FC_DOMAINS+1] = 0;
-
   hash = getFcFabricElementHash (vsanId, myGlobals.actualReportDeviceId);
 
   if (hash == NULL) {
@@ -1570,3 +1569,4 @@ void drawVsanDomainTrafficDistribution(u_short vsanId, u_char dataSent) {
 
   build_pie(idx, p, lbl);
 }
+#endif /* ENABLE_FC */
