@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2010 Luca Deri <deri@ntop.org>
  *
  * 			    http://www.ntop.org/
  *
@@ -22,7 +22,6 @@
 
 
 /* #define HASH_DEBUG */
-
 /* #define DNS_DEBUG */
 
 #ifdef HASH_DEBUG
@@ -30,6 +29,8 @@ static void hashSanityCheck();
 static void hostHashSanityCheck(HostTraffic *host);
 #endif
 
+static u_int sec_idle_with_no_sessions, sec_idle_with_sessions;
+
 /* ******************************* */
 
 u_int hashHost(HostAddr *hostIpAddress,  u_char *ether_addr,
@@ -111,6 +112,7 @@ u_int hashHost(HostAddr *hostIpAddress,  u_char *ether_addr,
 
 /* ************************************ */
 
+#ifdef ENABLE_FC
 u_int hashFcHost(FcAddress *fcaddr, u_short vsanId, HostTraffic **el,
 		 int actualDeviceId)
 {
@@ -140,6 +142,7 @@ u_int hashFcHost(FcAddress *fcaddr, u_short vsanId, HostTraffic **el,
 
   return(idx);
 }
+#endif
 
 /* ************************************ */
 /*
@@ -202,8 +205,8 @@ static void freeHostSessions(HostTraffic *host, int theDevice) {
 		 host->hostNumIpAddress, host->hostResolvedName, host->numHostSessions);
     }
 #endif
-  }
-  else {
+  } else {
+#ifdef ENABLE_FC
     for(i=0; i<MAX_TOT_NUM_SESSIONS; i++) {
       FCSession *prevSession, *nextSession, *theSession;
 
@@ -250,6 +253,7 @@ static void freeHostSessions(HostTraffic *host, int theDevice) {
       traceEvent(CONST_TRACE_ERROR, "====> Host %s/%s has %d sessions still to be purged",
 		 host->hostNumIpAddress, host->hostResolvedName, host->numHostSessions);
     }
+#endif
 #endif
   }
 }
@@ -262,7 +266,8 @@ void freeHostInfo(HostTraffic *host, int actualDeviceId) {
   if(host == NULL) {
     traceEvent(CONST_TRACE_WARNING, "Attempting to call freeHostInfo(NULL)");
     return;
-  }
+  } else
+    notifyEvent(hostDeletion, host, NULL, 0);
 
   /* If this is one of the special ones, let's clear the other pointer to it
    * to prevent a free of freed memory error later.
@@ -330,6 +335,7 @@ void freeHostInfo(HostTraffic *host, int actualDeviceId) {
     }
   }
 
+#ifdef ENABLE_FC
   if(myGlobals.device[actualDeviceId].fcTrafficMatrix != NULL) {
       int id = matrixHostHash (host, actualDeviceId, 0);
 
@@ -340,9 +346,11 @@ void freeHostInfo(HostTraffic *host, int actualDeviceId) {
           myGlobals.device[actualDeviceId].fcTrafficMatrix[i*myGlobals.device[actualDeviceId].numHosts+id] = NULL;
       }
   }
+#endif
 
   freeHostSessions(host, actualDeviceId);
 
+#ifdef ENABLE_FC
   if(host->fcCounters != NULL) {
     if(host->l2Family == FLAG_HOST_TRAFFIC_AF_FC) {
       for (i = 0; i < MAX_LUNS_SUPPORTED; i++) {
@@ -354,6 +362,7 @@ void freeHostInfo(HostTraffic *host, int actualDeviceId) {
 
     free(host->fcCounters);
   }
+#endif
 
   myGlobals.device[actualDeviceId].hostsno--;
 
@@ -435,18 +444,6 @@ void freeHostInfo(HostTraffic *host, int actualDeviceId) {
       }
     }
 
-    if(host->protocolInfo->fileList != NULL) {
-      FileList *list = host->protocolInfo->fileList;
-
-      while(list != NULL) {
-	FileList *next = list->next;
-        if(list->fileName != NULL)
-          free(list->fileName);
-	free(list);
-	list = next;
-      }
-    }
-
     if(host->protocolInfo->dnsStats  != NULL) free(host->protocolInfo->dnsStats);
     if(host->protocolInfo->httpStats != NULL) free(host->protocolInfo->httpStats);
     if(host->protocolInfo->dhcpStats != NULL) free(host->protocolInfo->dhcpStats);
@@ -540,10 +537,34 @@ void freeHostInstances(int actualDeviceId) {
 
 /* ************************************ */
 
+void readSessionPurgeParams() {
+  char buf[32], *key;
+
+  key = "purge_host.seconds_idle_with_no_sessions";
+  if(fetchPrefsValue(key, buf, sizeof(buf)) == 0) {
+    sec_idle_with_no_sessions = atoi(buf);
+  } else {
+    sec_idle_with_no_sessions = PARM_HOST_PURGE_MINIMUM_IDLE_NOACTVSES;
+    safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%u", sec_idle_with_no_sessions);
+    storePrefsValue(key, buf);
+  }
+
+  key = "purge_host.seconds_idle_with_sessions";
+  if(fetchPrefsValue(key, buf, sizeof(buf)) == 0) {
+    sec_idle_with_sessions = atoi(buf);
+  } else {
+    sec_idle_with_sessions = PARM_HOST_PURGE_MINIMUM_IDLE_ACTVSES;
+    safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%u", sec_idle_with_sessions);
+    storePrefsValue(key, buf);
+  }
+}
+
+/* ************************************ */
+		     
 int is_host_ready_to_purge(int actDevice, HostTraffic *el, time_t now) {
   /* Time used to decide whether a host need to be purged */
-  time_t noSessionPurgeTime   = now-PARM_HOST_PURGE_MINIMUM_IDLE_NOACTVSES;
-  time_t withSessionPurgeTime = now-PARM_HOST_PURGE_MINIMUM_IDLE_ACTVSES;
+  time_t noSessionPurgeTime   = now-sec_idle_with_no_sessions;
+  time_t withSessionPurgeTime = now-sec_idle_with_sessions;
   
   if(el->to_be_deleted
      || ((myGlobals.pcap_file_list == NULL)
@@ -556,8 +577,13 @@ int is_host_ready_to_purge(int actDevice, HostTraffic *el, time_t now) {
 	     || ((el->l2Family == FLAG_HOST_TRAFFIC_AF_ETH) &&
 		 ((el->hostNumIpAddress[0] == '\0') /* Purge MAC addresses too */
 		  || (!subnetPseudoLocalHost(el)))) /* Purge remote hosts only */
+#ifdef ENABLE_FC
 	     || ((el->l2Family == FLAG_HOST_TRAFFIC_AF_FC) &&
-		 (el->fcCounters->hostNumFcAddress[0] == '\0')))))
+		 (el->fcCounters->hostNumFcAddress[0] == '\0'))
+#endif
+	     )
+	 )
+     )
     return(1);
   else
     return(0);
@@ -575,7 +601,7 @@ int purgeIdleHosts(int actDevice) {
   float hiresDeltaTime;
   struct timeval hiresTimeStart, hiresTimeEnd;
   HostTraffic *el, *prev, *next;
-  
+
   /* if(myGlobals.runningPref.rFileName != NULL) return; */
 
 #ifdef IDLE_PURGE_DEBUG
@@ -720,6 +746,7 @@ void setHostSerial(HostTraffic *el) {
   if(el->hostSerial.serialType != SERIAL_NONE)
     return;
 
+#ifdef ENABLE_FC
   if(isFcHost(el)) {
     if(el->fcCounters->hostNumFcAddress[0] != '\0') {
       el->hostSerial.serialType = SERIAL_FC;
@@ -731,7 +758,9 @@ void setHostSerial(HostTraffic *el) {
     else
       traceEvent (CONST_TRACE_ERROR, "setHostSerial: Received NULL FC Address entry");
   }
-  else if(el->hostNumIpAddress[0] == '\0') {
+  else 
+#endif
+    if(el->hostNumIpAddress[0] == '\0') {
     el->hostSerial.serialType = SERIAL_MAC;
     memcpy(&el->hostSerial.value.ethSerial.ethAddress, el->ethAddress, LEN_ETHERNET_ADDRESS);
     el->hostSerial.value.ethSerial.vlanId = el->vlanId;
@@ -810,7 +839,6 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
     el = myGlobals.device[actualDeviceId].hash_hostTraffic[idx];
     if(el) {
       lockHostsHashMutex(el, "_lookupHost");
-      accessMutex(&myGlobals.hostsHashLockMutex, "lookupHost");
       el = myGlobals.device[actualDeviceId].hash_hostTraffic[idx];
       locked_mutex = 1;
     }
@@ -844,7 +872,7 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
 	      if(!(addrnull(&el->hostIpAddress)) &&
 		 (addrcmp(&el->hostIpAddress,hostIpAddress) != 0)) {
 		isMultihomed = 1;
-		FD_SET(FLAG_HOST_TYPE_MULTIHOMED, &el->flags);
+		setHostFlag(FLAG_HOST_TYPE_MULTIHOMED, el);
 	      } else {
 		updateIPinfo = 1;
 	      }
@@ -867,7 +895,7 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
 	  */
 
 	  if(!hasDuplicatedMac(el)) {
-	    FD_SET(FLAG_HOST_DUPLICATED_MAC, &el->flags);
+	    setHostFlag(FLAG_HOST_DUPLICATED_MAC, el);
 
 	    if(myGlobals.runningPref.enableSuspiciousPacketDump) {
 	      char etherbuf[LEN_ETHERNET_ADDRESS_DISPLAY];
@@ -898,11 +926,12 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
     numRuns++;
   } /* while */
 
-  if(locked_mutex) releaseMutex(&myGlobals.hostsHashLockMutex);
+  if(locked_mutex) 
+    unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]), locked_mutex = 0;
 
   if((hostFound == 1) && (vlanId != NO_VLAN) && (el->vlanId != NO_VLAN)
      && (vlanId != el->vlanId) && (!isMultivlaned(el))) {
-    FD_SET(FLAG_HOST_TYPE_MULTIVLANED, &el->flags);
+    setHostFlag(FLAG_HOST_TYPE_MULTIVLANED, el);
 
     if(myGlobals.multipleVLANedHostCount == 0) {
       traceEvent(CONST_TRACE_ERROR, "mVLAN: Host (identical IP/MAC) found on multiple VLANs [%d][%d]", vlanId, el->vlanId);
@@ -936,11 +965,11 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
               sizeof(el->hostNumIpAddress));
       setResolvedName(el, el->hostNumIpAddress, FLAG_HOST_SYM_ADDR_TYPE_IP);
 
-      if(myGlobals.runningPref.numericFlag == 0)
-        ipaddr2str(el->hostIpAddress, 1);
+      if(myGlobals.runningPref.numericFlag != noDnsResolution)
+        ipaddr2str(el, el->hostIpAddress, el->vlanId, actualDeviceId);
 
       if(isBroadcastAddress(&el->hostIpAddress, NULL, NULL))
-        FD_SET(FLAG_BROADCAST_HOST, &el->flags);
+        setHostFlag(FLAG_BROADCAST_HOST, el);
     }
   } else {
     /* New host entry */
@@ -955,12 +984,12 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
 		   myGlobals.runningPref.maxNumHashEntries);
       }
 
-      if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]);
+      if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]), locked_mutex = 0;
       return(NULL);
     }
         
     if((el = (HostTraffic*)malloc(sizeof(HostTraffic))) == NULL) {
-      if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]);
+      if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]), locked_mutex = 0;
       return(NULL);
     }
      
@@ -971,24 +1000,26 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
     el->vlanId = vlanId;
 
     if(isMultihomed)
-      FD_SET(FLAG_HOST_TYPE_MULTIHOMED, &el->flags);
+      setHostFlag(FLAG_HOST_TYPE_MULTIHOMED, el);
 
     if(el->portsUsage != NULL)
       freePortsUsage(el);
 
     len = (size_t)myGlobals.numIpProtosList*sizeof(ShortProtoTrafficInfo**);
     if((el->ipProtosList = (ShortProtoTrafficInfo**)malloc(len)) == NULL) {
-      if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]);
+      if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]), locked_mutex = 0;
       return(NULL);
     }
     memset(el->ipProtosList, 0, len);
 
+    /*
     len = (size_t)myGlobals.numIpProtosToMonitor*sizeof(ProtoTrafficInfo**);
     if((el->protoIPTrafficInfos = (ProtoTrafficInfo**)malloc(len)) == NULL) {
-      if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]);
+      if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]), locked_mutex = 0;
       return(NULL);
     }
     memset(el->protoIPTrafficInfos, 0, len);   
+    */
 
     el->magic = CONST_MAGIC_NUMBER;
     el->hostTrafficBucket = idx; /* Set the bucket index */
@@ -1021,8 +1052,8 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
 	memcpy(el->ethAddress, ether_addr, LEN_ETHERNET_ADDRESS);
 	strncpy(el->ethAddressString, ethAddr, sizeof(el->ethAddressString));
 	symEthName = getSpecialMacInfo(el, (short)(!myGlobals.separator[0]));
-	FD_SET(FLAG_SUBNET_LOCALHOST, &el->flags);
-	FD_SET(FLAG_SUBNET_PSEUDO_LOCALHOST, &el->flags);
+	setHostFlag(FLAG_SUBNET_LOCALHOST, el);
+	setHostFlag(FLAG_SUBNET_PSEUDO_LOCALHOST, el);
 	/* traceEvent(CONST_TRACE_WARNING, "-> %u/%u", the_local_network, the_local_network_mask); */
       } else if(hostIpAddress != NULL) {
 	/* This is packet that's being routed or belonging to a
@@ -1037,21 +1068,21 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
 	    memcpy(el->ethAddress, &hostIpAddress->Ip6Address.s6_addr[8], 4);
 #endif
 
-	  FD_CLR(FLAG_SUBNET_LOCALHOST, &el->flags);
+	  clearHostFlag(FLAG_SUBNET_LOCALHOST, el);
 
 	  if(isPrivateAddress(hostIpAddress, &the_local_network, &the_local_network_mask))
-	    FD_SET(FLAG_PRIVATE_IP_ADDRESS, &el->flags);
+	    setHostFlag(FLAG_PRIVATE_IP_ADDRESS, el);
 
 	  if(!isBroadcastAddress(hostIpAddress, &the_local_network, &the_local_network_mask)) {
 	    if(isPseudoLocalAddress(hostIpAddress, actualDeviceId, &the_local_network, &the_local_network_mask))
-	      FD_SET(FLAG_SUBNET_PSEUDO_LOCALHOST, &el->flags);
+	      setHostFlag(FLAG_SUBNET_PSEUDO_LOCALHOST, el);
 	    else
-	      FD_CLR(FLAG_SUBNET_PSEUDO_LOCALHOST, &el->flags);
+	      clearHostFlag(FLAG_SUBNET_PSEUDO_LOCALHOST, el);
 	  }
 	}
       } else {
-	FD_CLR(FLAG_SUBNET_LOCALHOST, &el->flags);
-	FD_CLR(FLAG_SUBNET_PSEUDO_LOCALHOST, &el->flags);
+	clearHostFlag(FLAG_SUBNET_LOCALHOST, el);
+	clearHostFlag(FLAG_SUBNET_PSEUDO_LOCALHOST, el);
       }
 
       updateHostKnownSubnet(el);
@@ -1071,9 +1102,9 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
 #endif
 	}
 
-	FD_SET(FLAG_BROADCAST_HOST, &el->flags);
+	setHostFlag(FLAG_BROADCAST_HOST, el);
 	if(isMulticastAddress(&el->hostIpAddress, &the_local_network, &the_local_network_mask))
-	  FD_SET(FLAG_MULTICAST_HOST, &el->flags);
+	  setHostFlag(FLAG_MULTICAST_HOST, el);
 	strncpy(el->hostNumIpAddress,
 		_addrtostr(&el->hostIpAddress, buf, sizeof(buf)),
 		strlen(el->hostNumIpAddress));
@@ -1123,20 +1154,20 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
       strncpy(el->hostNumIpAddress,
 	      _addrtostr(hostIpAddress, buf, sizeof(buf)),
 	      sizeof(el->hostNumIpAddress));
-      if(isBroadcastAddress(&el->hostIpAddress, &the_local_network, &the_local_network_mask)) FD_SET(FLAG_BROADCAST_HOST, &el->flags);
-      if(isMulticastAddress(&el->hostIpAddress, &the_local_network, &the_local_network_mask)) FD_SET(FLAG_MULTICAST_HOST, &el->flags);
-      if(isPrivateAddress(hostIpAddress, &the_local_network, &the_local_network_mask))        FD_SET(FLAG_PRIVATE_IP_ADDRESS,  &el->flags);
+      if(isBroadcastAddress(&el->hostIpAddress, &the_local_network, &the_local_network_mask)) setHostFlag(FLAG_BROADCAST_HOST, el);
+      if(isMulticastAddress(&el->hostIpAddress, &the_local_network, &the_local_network_mask)) setHostFlag(FLAG_MULTICAST_HOST, el);
+      if(isPrivateAddress(hostIpAddress, &the_local_network, &the_local_network_mask))        setHostFlag(FLAG_PRIVATE_IP_ADDRESS, el);
       if((ether_addr == NULL) && (isPseudoLocalAddress(hostIpAddress, actualDeviceId, &the_local_network, &the_local_network_mask)))
-	FD_SET(FLAG_SUBNET_PSEUDO_LOCALHOST, &el->flags);
+	setHostFlag(FLAG_SUBNET_PSEUDO_LOCALHOST, el);
 
       setResolvedName(el, el->hostNumIpAddress, FLAG_HOST_SYM_ADDR_TYPE_IP);
       updateHostKnownSubnet(el);
 
       /* Trick to fill up the address cache */
-      if(myGlobals.runningPref.numericFlag == 0)
-	ipaddr2str(el->hostIpAddress, 1);
+      if(myGlobals.runningPref.numericFlag != noDnsResolution)
+	ipaddr2str(el, el->hostIpAddress, el->vlanId, actualDeviceId);
 
-      getHostAS(el);
+      /* getHostAS(el); */
     } else {
       /* This is a new entry and hostIpAddress was NOT set.  Fill in MAC address, if we have it */
       if(symEthName[0] != '\0') {
@@ -1159,13 +1190,15 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
 
     setHostSerial(el);
     handlePluginHostCreationDeletion(el, (u_short)actualDeviceId, 1 /* host creation */);
+
+    notifyEvent(hostCreation, el, NULL, 0);
   }
 
   if(el != NULL) {
     el->lastSeen = myGlobals.actTime;
 
     if(setSpoofingFlag)
-      FD_SET(FLAG_HOST_DUPLICATED_MAC, &el->flags);
+      setHostFlag(FLAG_HOST_DUPLICATED_MAC, el);
 
 #ifdef DEBUG
     {
@@ -1190,13 +1223,14 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
   hashSanityCheck();
 #endif
 
-  if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]);
+  if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]), locked_mutex = 0;
   
   return(el);
 }
 
 /* ********************************************************** */
 
+#ifdef ENABLE_FC
 HostTraffic *lookupFcHost (FcAddress *hostFcAddress, u_short vsanId,
                            int actualDeviceId) {
   u_int idx;
@@ -1270,12 +1304,12 @@ HostTraffic *lookupFcHost (FcAddress *hostFcAddress, u_short vsanId,
 		   myGlobals.runningPref.maxNumHashEntries);
       }
 
-      if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]);
+      if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]), locked_mutex = 0;
       return(NULL);
     }
 
     if((el = (HostTraffic*)malloc(sizeof(HostTraffic))) == NULL) {
-      if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]);
+      if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]), locked_mutex = 0;
       return(NULL);
     }
 
@@ -1285,7 +1319,7 @@ HostTraffic *lookupFcHost (FcAddress *hostFcAddress, u_short vsanId,
     resetHostsVariables(el);
 
     if(allocFcScsiCounters(el) == NULL) {
-      if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]);
+      if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]), locked_mutex = 0;
       return(NULL);
     }
     el->l2Family = FLAG_HOST_TRAFFIC_AF_FC;
@@ -1325,7 +1359,9 @@ HostTraffic *lookupFcHost (FcAddress *hostFcAddress, u_short vsanId,
                el->ethAddressString, el->hostNumIpAddress, list->idx, actualDeviceId,
                myGlobals.device[actualDeviceId].actualHashSize, myGlobals.device[actualDeviceId].hostsno);
 #endif
+
     setHostSerial(el);
+    notifyEvent(hostCreation, el, NULL, 0);
   }
 
   if(el != NULL) {
@@ -1340,9 +1376,10 @@ HostTraffic *lookupFcHost (FcAddress *hostFcAddress, u_short vsanId,
   hashSanityCheck();
 #endif
 
-  if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]);
+  if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]), locked_mutex = 0;
   return(el);
 }
+#endif
 
 /* ************************************ */
 
