@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2003      Abdelkader Lahmadi <Abdelkader.Lahmadi@loria.fr>
  *                         Olivier Festor <Olivier.Festor@loria.fr>
- * Copyright (C) 2003-09   Luca Deri <deri@ntop.org>
+ * Copyright (C) 2003-10   Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -45,13 +45,16 @@ static struct in6_addr *in6_cpy(struct in6_addr *dst, struct in6_addr *src) {
 
 /* ************************************************* */
 
+#if defined(HAVE_IFLIST_SYSCTL) && defined(HAVE_SYSCTL)
 static struct in_addr *in4_cpy(struct in_addr *dst, struct in_addr *src) {
   memcpy(dst, src, sizeof(struct in_addr));
   return(dst);
 }
+#endif
 
 /* ************************************************* */
 
+#ifdef LINUX
 static void str2in6_addr(char *str, struct in6_addr *addr) {
   int i;
   unsigned int x;
@@ -61,9 +64,12 @@ static void str2in6_addr(char *str, struct in6_addr *addr) {
     addr->s6_addr[i]= x & 0xff;
   }
 }
+#endif
 
 /* ************************************************* */
 
+#if defined(HAVE_IFLIST_SYSCTL) && defined(HAVE_SYSCTL)
+
 static int prefixlen(void *val, int size) {
   unsigned char *name = (unsigned char *)val;
   int byte, bit, plen = 0;
@@ -88,8 +94,6 @@ static int prefixlen(void *val, int size) {
 
 /* ************************************************* */
 
-#if defined(HAVE_IFLIST_SYSCTL) && defined(HAVE_SYSCTL)
-
 struct iface_handler *iface_new(void) {
   int		 mib[6] = { CTL_NET, PF_ROUTE, 0, 0, NET_RT_IFLIST, 0 };
   char		*buf = NULL;
@@ -99,7 +103,7 @@ struct iface_handler *iface_new(void) {
   int i;
   int if_pos = 0;
   int addr_pos = 0;
-  struct iface_addr	*ia;
+  struct iface_addr	*ia = NULL;
 
   /* Allocate memory for iface handler
    */
@@ -341,6 +345,7 @@ struct iface_handler *iface_new(void) {
     errno = ENOMEM;
     goto failed;
   }
+
   fp = fopen(PATH_PROC_NET_IF_INET6,"r");
   if (fp == NULL)
     goto failed;
@@ -380,10 +385,10 @@ struct iface_handler *iface_new(void) {
       in6_cpy(&it->af.inet6.addr, &addr);
       it->af.inet6.prefixlen = plen;
       it->next = NULL;
-      if (ii == NULL){
+      if(ii == NULL) {
 	hdlr->if_list = itf;
 	hdlr->addr_list = it;
-      }else {
+      } else {
 	ii->next = itf;
       }
       ii = itf;
@@ -391,11 +396,13 @@ struct iface_handler *iface_new(void) {
 
     }
   }
+
+  fclose(fp);
   return hdlr;
+
  failed:
   iface_destroy(hdlr);
   return NULL;
-
 }
 #else
 
