@@ -2,7 +2,7 @@
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *                          http://www.ntop.org
  *
- * Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
+ * Copyright (C) 1998-2010 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -109,32 +109,10 @@ static void setDomainName(void) {
  */
 void initIPServices(void) {
   FILE* fd;
-  int idx, numSlots, len, rc;
-  int major, minor;
+  int idx, numSlots, len;
 
   traceEvent(CONST_TRACE_NOISY, "Initializing IP services");
 
-  event_init();
-
-  sscanf(event_get_version(), "%d.%d", &major, &minor);
-
-  if(minor < 4) {
-    traceEvent(CONST_TRACE_ERROR, 
-	       "You are using libevent %d whereas ntop needs at least v1.4",
-	       event_get_version());
-    traceEvent(CONST_TRACE_ERROR, 
-	       "Due to a libevent bug with IPv6 address resolution");
-traceEvent(CONST_TRACE_ERROR, 
-	   "See http://www.mail-archive.com/debian-bugs-dist@lists.debian.org/msg408382.html");
-     traceEvent(CONST_TRACE_ERROR, 
-		"Please rebuild ntop against a newer libevent version");
-   exit(0);
-  }
-
-  if((rc = evdns_init()) != DNS_ERR_NONE) {
-    traceEvent(CONST_TRACE_ERROR, "evdns_init() returned %d", rc);
-  }
-
   /* Let's count the entries first */
   numSlots = 0;
   for(idx=0; myGlobals.configFileDirs[idx] != NULL; idx++) {
@@ -286,6 +264,7 @@ void resetDevice(int devIdx, short fullReset) {
   resetTrafficCounter(&myGlobals.device[devIdx].greBytes);
   resetTrafficCounter(&myGlobals.device[devIdx].ipv6Bytes);
   resetTrafficCounter(&myGlobals.device[devIdx].otherBytes);
+#ifdef ENABLE_FC
   resetTrafficCounter(&myGlobals.device[devIdx].fcBytes);
   resetTrafficCounter(&myGlobals.device[devIdx].fragmentedFcBytes);
   resetTrafficCounter(&myGlobals.device[devIdx].fcFcpBytes);
@@ -298,6 +277,13 @@ void resetDevice(int devIdx, short fullReset) {
   resetTrafficCounter(&myGlobals.device[devIdx].class2Bytes);
   resetTrafficCounter(&myGlobals.device[devIdx].class3Bytes);
   resetTrafficCounter(&myGlobals.device[devIdx].classFBytes);
+  resetTrafficCounter(&myGlobals.device[devIdx].fcPkts);
+  resetTrafficCounter(&myGlobals.device[devIdx].fcEofaPkts);
+  resetTrafficCounter(&myGlobals.device[devIdx].fcEofAbnormalPkts);
+  resetTrafficCounter(&myGlobals.device[devIdx].fcAbnormalPkts);
+  resetTrafficCounter(&myGlobals.device[devIdx].fcBroadcastPkts);
+  memset(&myGlobals.device[devIdx].rcvdFcPktStats, 0, sizeof(FcPacketStats));
+#endif
   resetTrafficCounter(&myGlobals.device[devIdx].lastMinEthernetBytes);
   resetTrafficCounter(&myGlobals.device[devIdx].lastFiveMinsEthernetBytes);
   resetTrafficCounter(&myGlobals.device[devIdx].lastMinEthernetPkts);
@@ -310,13 +296,7 @@ void resetDevice(int devIdx, short fullReset) {
   resetTrafficCounter(&myGlobals.device[devIdx].lastEthernetBytes);
   resetTrafficCounter(&myGlobals.device[devIdx].lastIpBytes);
   resetTrafficCounter(&myGlobals.device[devIdx].lastNonIpBytes);
-  resetTrafficCounter(&myGlobals.device[devIdx].fcPkts);
-  resetTrafficCounter(&myGlobals.device[devIdx].fcEofaPkts);
-  resetTrafficCounter(&myGlobals.device[devIdx].fcEofAbnormalPkts);
-  resetTrafficCounter(&myGlobals.device[devIdx].fcAbnormalPkts);
-  resetTrafficCounter(&myGlobals.device[devIdx].fcBroadcastPkts);
   memset(&myGlobals.device[devIdx].rcvdPktStats, 0, sizeof(PacketStats));
-  memset(&myGlobals.device[devIdx].rcvdFcPktStats, 0, sizeof(FcPacketStats));
   memset(&myGlobals.device[devIdx].rcvdPktTTLStats, 0, sizeof(TTLstats));
   myGlobals.device[devIdx].peakThroughput = 0;
   myGlobals.device[devIdx].actualThpt = 0;
@@ -336,7 +316,6 @@ void resetDevice(int devIdx, short fullReset) {
   myGlobals.device[devIdx].last60MinutesThptIdx = 0;
   myGlobals.device[devIdx].last24HoursThptIdx = 0;
   myGlobals.device[devIdx].last30daysThptIdx = 0;
-  myGlobals.device[devIdx].hostsno = 0;
 
   if(myGlobals.pcap_file_list == NULL) {
     myGlobals.device[devIdx].lastThptUpdate = myGlobals.device[devIdx].lastMinThptUpdate =
@@ -383,15 +362,6 @@ void resetDevice(int devIdx, short fullReset) {
 
 void initCounters(void) {
   int len, i;
-  FILE* fd;
-  int numRead = 0;
-  u_char compressedFormat;
-  char buf[LEN_GENERAL_WORK_BUFFER];
-  struct dirent **dirList;
-  int iLang, nLang, found;
-  DIR *testDirEnt;
-  char *tmpStr;
-  char* realLocale;
 
   setDomainName();
 
@@ -426,12 +396,16 @@ void initCounters(void) {
       myGlobals.device[i].tcpSession = (IPSession**)malloc(len);
       memset(myGlobals.device[i].tcpSession, 0, len);
 
+#ifdef ENABLE_FC
       len = sizeof(FCSession *)*MAX_TOT_NUM_SESSIONS;
       myGlobals.device[i].fcSession = (FCSession **)malloc(len);
       memset(myGlobals.device[i].fcSession, 0, len);
+#endif
     } else {
       myGlobals.device[i].tcpSession     = NULL;
+#ifdef ENABLE_FC
       myGlobals.device[i].fcSession      = NULL;
+#endif
     }
 
     myGlobals.device[i].fragmentList = NULL;
@@ -440,6 +414,7 @@ void initCounters(void) {
   myGlobals.ipxsapHashLoadCollisions = 0;
   myGlobals.hashCollisionsLookup     = 0;
 
+#ifdef ENABLE_FC
   myGlobals.numVendorLookupRead = 0;
   myGlobals.numVendorLookupAdded = 0;
   myGlobals.numVendorLookupAddedSpecial = 0;
@@ -449,14 +424,13 @@ void initCounters(void) {
   myGlobals.numVendorLookupFound24bit = 0;
   myGlobals.numVendorLookupFoundMulticast = 0;
   myGlobals.numVendorLookupFoundLAA = 0;
+#endif
 
-  if (myGlobals.pcap_file_list == NULL) {
-      myGlobals.initialSniffTime = myGlobals.lastRefreshTime = time(NULL);
-  }
-  else {
-      myGlobals.initialSniffTime = 0; /* We set the start when first pkt is
-                                       * read */
-  }
+  if (myGlobals.pcap_file_list == NULL)
+    myGlobals.initialSniffTime = myGlobals.lastRefreshTime = time(NULL);  
+  else
+    myGlobals.initialSniffTime = 0; /* We set the start when first pkt is
+                                       * read */  
 
 /* TODO why here AND in globals-core.c? */
   myGlobals.numHandledSIGPIPEerrors = 0;
@@ -492,8 +466,11 @@ void initCounters(void) {
 /* ******************************* */
 
 void resetStats(int deviceId) {
-  u_int j, i;
+  u_int j;
+#ifdef ENABLE_FC
+  int i;
   FCSession *session;
+#endif
 
   traceEvent(CONST_TRACE_INFO, "Resetting traffic statistics for device %s",
 	     myGlobals.device[deviceId].humanFriendlyName);
@@ -534,6 +511,7 @@ void resetStats(int deviceId) {
       }
   }
 
+#ifdef ENABLE_FC
   if(myGlobals.device[deviceId].fcSession != NULL) {
     for(j=0; j<MAX_TOT_NUM_SESSIONS; j++)
       if((session = myGlobals.device[deviceId].fcSession[j]) != NULL) {
@@ -552,12 +530,13 @@ void resetStats(int deviceId) {
       free (myGlobals.device[deviceId].vsanHash);
       myGlobals.device[deviceId].vsanHash = NULL;
   }
+#endif
 
   myGlobals.device[deviceId].hash_hostTraffic[BROADCAST_HOSTS_ENTRY] = myGlobals.broadcastEntry;
   myGlobals.broadcastEntry->hostSerial.serialType = SERIAL_IPV4;
   myGlobals.broadcastEntry->hostSerial.value.ipSerial.ipAddress.Ip4Address.s_addr = -1;
   myGlobals.broadcastEntry->next = NULL;
-  FD_SET(FLAG_BROADCAST_HOST, &(myGlobals.broadcastEntry->flags));
+  setHostFlag(FLAG_BROADCAST_HOST, myGlobals.broadcastEntry);
 
   if(myGlobals.otherHostEntry != myGlobals.broadcastEntry) {
     myGlobals.device[deviceId].hash_hostTraffic[OTHER_HOSTS_ENTRY] = myGlobals.otherHostEntry;
@@ -577,7 +556,7 @@ void initSingleGdbm(GDBM_FILE *database,
 		    char *dbName, char *directory,
 		    int doUnlink, struct stat *statbuf) {
   char tmpBuf[200], theDate[48];
-  time_t        st_time, now;
+  time_t st_time, now;
   struct tm t;
   int d;
 
@@ -630,7 +609,7 @@ void initSingleGdbm(GDBM_FILE *database,
 		   theDate,
 		   d = difftime(now, st_time));
 
-	if(d > CONST_DNSCACHE_PERMITTED_AGE) {
+	if(d > (15 * 60)) {
 	  traceEvent(CONST_TRACE_INFO, "...older, will recreate it");
 	  doUnlink = TRUE;
 	} else {
@@ -711,10 +690,6 @@ void reinitMutexes (void) {
 
   createMutex(&myGlobals.securityItemsMutex);
   createMutex(&myGlobals.hostsHashLockMutex);
-
-  if(myGlobals.runningPref.numericFlag == 0) {
-    createMutex(&myGlobals.addressResolutionMutex);
-  }
 }
 #endif /* HAVE_PTHREAD_ATFORK */
 
@@ -729,12 +704,12 @@ void initThreads(void) {
   int i;
 
   /*
-  for(i=0; i<myGlobals.numDevices; i++) {
+    for(i=0; i<myGlobals.numDevices; i++) {
     createThread(&myGlobals.device[i].dequeuePacketThreadId, dequeuePacket, (void*)((long)i));
     traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: NPA: Started thread for network packet analyzer (%s)",
-	       (long)myGlobals.device[i].dequeuePacketThreadId,
-	       myGlobals.device[i].humanFriendlyName);
-  }
+    (long)myGlobals.device[i].dequeuePacketThreadId,
+    myGlobals.device[i].humanFriendlyName);
+    }
   */
 
   if(!myGlobals.runningPref.liveMode) {
@@ -753,16 +728,26 @@ void initThreads(void) {
   traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: SIH: Started thread for idle hosts detection",
              (long)myGlobals.scanIdleThreadId);
 
-#ifdef MAKE_WITH_SSLWATCHDOG
-#ifdef MAKE_WITH_SSLWATCHDOG_RUNTIME
-  if(myGlobals.runningPref.useSSLwatchdog == 1)
+  if(myGlobals.runningPref.numericFlag != noDnsResolution) {
+    createMutex(&myGlobals.addressResolutionMutex);
+ 
+#if defined(HAVE_GETHOSTBYADDR_R)
+    myGlobals.numDequeueAddressThreads = MAX_NUM_DEQUEUE_ADDRESS_THREADS;
+#else
+    myGlobals.numDequeueAddressThreads = 1;
 #endif
-    {
-      traceEvent(CONST_TRACE_NOISY, "Initializing Condvar for ssl watchdog");
-      createCondvar(&myGlobals.sslwatchdogCondvar);
-      myGlobals.sslwatchdogCondvar.predicate = FLAG_SSLWATCHDOG_UNINIT;
+
+    initAddressResolution();
+   
+    /*
+     * Create the thread (5) - DNSAR - DNS Address Resolution - optional
+     */  
+    for(i=0; i<myGlobals.numDequeueAddressThreads; i++) {
+      createThread(&myGlobals.dequeueAddressThreadId[i], dequeueAddress, (char*)((long)i));
+      traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: DNSAR(%d): Started thread for DNS address resolution",
+		 (long)myGlobals.dequeueAddressThreadId[i], i+1);
     }
-#endif
+  }
 }
 
 /*
@@ -779,6 +764,7 @@ void initDeviceSemaphores(int deviceId) {
   traceEvent(CONST_TRACE_INFO, "Initializing device %s (%d)",
 	     myGlobals.device[deviceId].name, deviceId);
     
+  createMutex(&myGlobals.device[deviceId].counterMutex);
   createMutex(&myGlobals.device[deviceId].asMutex);
   createMutex(&myGlobals.device[deviceId].packetProcessMutex);
   createMutex(&myGlobals.device[deviceId].packetQueueMutex);
@@ -1098,8 +1084,9 @@ void addDevice(char* deviceName, char* deviceDescr) {
         myGlobals.fcTrafficMatrixMemoryUsage += memlen;
 
         memlen = sizeof(struct hostTraffic*)*myGlobals.device[deviceId].numHosts;
-        myGlobals.device[deviceId].fcTrafficMatrixHosts = (struct hostTraffic**)calloc(sizeof(struct hostTraffic*),
-                                                                                       myGlobals.device[deviceId].numHosts);
+        myGlobals.device[deviceId].fcTrafficMatrixHosts = 
+	  (struct hostTraffic**)calloc(sizeof(struct hostTraffic*),
+				       myGlobals.device[deviceId].numHosts);
       }
 #endif
     }
@@ -1112,8 +1099,14 @@ void addDevice(char* deviceName, char* deviceDescr) {
     
     getLocalHostAddress(&myGlobals.device[deviceId].ifAddr, &netmask_v6, myGlobals.device[deviceId].name);
 #ifdef INET6
-    myGlobals.device[deviceId].v6Addrs = getLocalHostAddressv6(myGlobals.device[deviceId].v6Addrs, myGlobals.device[deviceId].name);
+    myGlobals.device[deviceId].v6Addrs = getLocalHostAddressv6(myGlobals.device[deviceId].v6Addrs, 
+							       myGlobals.device[deviceId].name);
 #endif
+
+	if(myGlobals.device[deviceId].network.s_addr == 0) {
+		myGlobals.device[deviceId].netmask.s_addr = 0xFFFFFF00;/* /24 */
+		myGlobals.device[deviceId].network.s_addr = myGlobals.device[deviceId].ifAddr.s_addr & myGlobals.device[deviceId].netmask.s_addr;
+	}
   }
 
   mallocLen = 2;
@@ -1227,17 +1220,11 @@ void initDevices(char* devices) {
   if(myGlobals.pcap_file_list != NULL) {
     createDummyInterface("none");
     myGlobals.device[0].dummyDevice = 0;
-    myGlobals.device[0].pcapPtr  = pcap_open_offline(myGlobals.pcap_file_list->fileName, ebuf);
+    myGlobals.device[0].pcapPtr  = myGlobals.pcap_file_list->pcapPtr;
 
-    if(myGlobals.device[0].pcapPtr == NULL) {
-      traceEvent(CONST_TRACE_ERROR, "pcap_open_offline(%s): '%s'",
-		 myGlobals.pcap_file_list->fileName, ebuf);
-      exit(-1);
-    } else {
-      if(myGlobals.device[0].humanFriendlyName != NULL) free(myGlobals.device[0].humanFriendlyName);
-      myGlobals.device[0].humanFriendlyName = strdup(myGlobals.pcap_file_list->fileName);
-      calculateUniqueInterfaceName(0);
-    }
+    if(myGlobals.device[0].humanFriendlyName != NULL) free(myGlobals.device[0].humanFriendlyName);
+    myGlobals.device[0].humanFriendlyName = strdup(myGlobals.pcap_file_list->fileName);
+    calculateUniqueInterfaceName(0);
 
     resetStats(0);
     initDeviceDatalink(0);
@@ -1495,7 +1482,6 @@ void parseTrafficFilter(void) {
   /* Construct, compile and set filter */
   if(myGlobals.runningPref.currentFilterExpression != NULL) {
     int i;
-    struct bpf_program fcode;
 
     for(i=0; i<myGlobals.numDevices; i++)
       setPcapFilter(myGlobals.runningPref.currentFilterExpression, i);
@@ -1624,14 +1610,7 @@ u_int createDummyInterface(char *ifName) {
   }
 #endif
 
-  if(myGlobals.runningPref.enableSessionHandling) {
-    int len = sizeof(IPSession*)*MAX_TOT_NUM_SESSIONS;
-    myGlobals.device[deviceId].tcpSession = (IPSession**)malloc(len);
-    memset(myGlobals.device[deviceId].tcpSession, 0, len);
-  }
-
   /* Allocate memory for dhcp stats */
-
   for(i=0; i<myGlobals.numKnownSubnets; i++) {
     myGlobals.device[deviceId].networkHost[i].protocolInfo = calloc(1, sizeof(ProtocolInfo));
     myGlobals.device[deviceId].networkHost[i].protocolInfo->dnsStats = calloc(1, sizeof(ServiceStats));
