@@ -2,7 +2,7 @@
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *                          http://www.ntop.org
  *
- *         Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
+ *         Copyright (C) 1998-2010 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -47,6 +47,8 @@
 
 #ifdef MAKE_WITH_SAFER_ROUTINES
 
+/* ***************************************************** */
+
 static void stopcap(void) {
   traceEvent(CONST_TRACE_WARNING, "ntop packet capture STOPPED");
   traceEvent(CONST_TRACE_INFO, "NOTE: ntop web server remains up");
@@ -58,10 +60,12 @@ static void stopcap(void) {
 
 /* #define COUNT_MALLOCS 1 */
 
+/* ***************************************************** */
+
 void* ntop_safemalloc(unsigned int sz, char* file, int line) {
   void *thePtr;
 #ifdef COUNT_MALLOCS
-  static u_int num_allocs = 0, tot_allocs = 0;
+  static uint num_allocs = 0, tot_allocs = 0;
 #endif
 
 #ifdef DEBUG
@@ -315,7 +319,6 @@ static void traceEventLeak(int eventTraceLevel, /* Ignored */
   va_end (va_ap);
 
   inTraceEventTrapped=0;
-
 }
 
 /* *************************************** */
