@@ -2,7 +2,7 @@
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *                          http://www.ntop.org
  *
- *          Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
+ *          Copyright (C) 1998-2010 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -27,10 +27,13 @@
 
 #if defined(DARWIN) && (!defined(TIGER))
 #include <mach-o/dyld.h>
-
 extern char ** environ;
 #endif
 
+#if defined(MEMORY_DEBUG) && (MEMORY_DEBUG == 1)
+#include <mcheck.h>
+#endif
+
 #if defined(MEMORY_DEBUG) && (MEMORY_DEBUG == 4) && defined(HAVE_BACKTRACE)
 #include <execinfo.h>
 #endif
@@ -41,10 +44,12 @@ char static_ntop;
  * Hello World! This is ntop speaking...
  */
 void welcome (FILE * fp) {
-  fprintf (fp, "Welcome to %s v.%s\n[Configured on %s, built on %s]\n",
-	   myGlobals.program_name, version, configureDate, buildDate);
+  fprintf (fp, "Welcome to %s v.%s (%d bit)\n"
+	   "[Configured on %s, built on %s]\n",
+	   myGlobals.program_name, version, sizeof(long) == 8 ? 64 : 32,
+	   configureDate, buildDate);
 
-  fprintf (fp, "Copyright 1998-2009 by %s.\n", author);
+  fprintf (fp, "Copyright 1998-2010 by %s.\n", author);
   fprintf (fp, "Get the freshest ntop from http://www.ntop.org/\n");
 }
 
@@ -99,7 +104,10 @@ void usage(FILE * fp) {
   fprintf(fp, "    [-j             | --create-other-packets]             %sCreate file ntop-other-pkts.XXX.pcap file\n", newLine);
   fprintf(fp, "    [-l <path>      | --pcap-log <path>]                  %sDump packets captured to a file (debug only!)\n", newLine);
   fprintf(fp, "    [-m <addresses> | --local-subnets <addresses>]        %sLocal subnetwork(s) (see man page)\n", newLine);
-  fprintf(fp, "    [-n             | --numeric-ip-addresses]             %sNumeric IP addresses - no DNS resolution\n", newLine);
+  fprintf(fp, "    [-n <mode>      | --numeric-ip-addresses <mode>]      %sNumeric IP addresses DNS resolution mode:\n", newLine);
+  fprintf(fp, "                                                          %s  %d - %s\n", newLine, 0, "No DNS resolution at all");
+  fprintf(fp, "                                                          %s  %d - %s\n", newLine, 1, "DNS resolution for local hosts only");
+  fprintf(fp, "                                                          %s  %d - %s\n", newLine, 2, "DNS resolution for remote hosts only");
   fprintf(fp, "    [-o             | --no-mac]                           %sntop will trust just IP addresses (no MACs)\n", newLine);
   fprintf(fp, "    [-p <list>      | --protocols <list>]                 %sList of IP protocols to monitor (see man page)\n", newLine);
   fprintf(fp, "    [-q             | --create-suspicious-packets]        %sCreate file ntop-suspicious-pkts.XXX.pcap file\n", newLine);
@@ -133,7 +141,9 @@ void usage(FILE * fp) {
 
   fprintf(fp, "    [-M             | --no-interface-merge]               %sDon't merge network interfaces (see man page)\n",
 	  newLine);
+#ifdef ENABLE_FC
   fprintf(fp, "    [-N             | --wwn-map]                          %sMap file providing map of WWN to FCID/VSAN\n", newLine);
+#endif
   fprintf(fp, "    [-O <path>      | --pcap-file-path <path>]            %sPath for log files in pcap format\n", newLine);
   fprintf(fp, "    [-U <URL>       | --mapper <URL>]                     %sURL (mapper.pl) for displaying host location\n",
 	  newLine);
@@ -146,28 +156,24 @@ void usage(FILE * fp) {
 
   fprintf(fp, "    [--disable-instantsessionpurge]                       %sDisable instant FIN session purge\n", newLine);
   fprintf(fp, "    [--disable-mutexextrainfo]                            %sDisable extra mutex info\n", newLine);
-#ifdef MAKE_WITH_SCHED_YIELD
-  fprintf(fp, "    [--disable-schedyield]                                %sTurn off sched_yield() calls, if ntop is deadlocking on them\n", newLine);
-#endif
   fprintf(fp, "    [--disable-stopcap]                                   %sCapture packets even if there's no memory left\n", newLine);
+
+#ifdef ENABLE_FC
   fprintf(fp, "    [--fc-only]                                           %sDisplay only Fibre Channel statistics\n", newLine);
   fprintf(fp, "    [--no-fc]                                             %sDisable processing & Display of Fibre Channel\n", newLine);
-  fprintf(fp, "    [--instance]                                          %sSet log name for this ntop instance\n", newLine);
   fprintf(fp, "    [--no-invalid-lun]                                    %sDon't display Invalid LUN information\n", newLine);
+#endif
+
+  fprintf(fp, "    [--instance <name>]                                   %sSet log name for this ntop instance\n", newLine);
   fprintf(fp, "    [--p3p-cp]                                            %sSet return value for p3p compact policy, header\n", newLine);
   fprintf(fp, "    [--p3p-uri]                                           %sSet return value for p3p policyref header\n", newLine);
   fprintf(fp, "    [--skip-version-check]                                %sSkip ntop version check\n", newLine);
-#ifdef MAKE_WITH_SSLWATCHDOG_RUNTIME
-  fprintf(fp, "    [--ssl-watchdog]                                      %sUse ssl watchdog (NS6 problem)\n", newLine);
-#endif
-  fprintf(fp, "    [--known-subnets]                                     %sList of known subnets (separated by ,)\n", newLine);
+  fprintf(fp, "    [--known-subnets <networks>]                          %sList of known subnets (separated by ,)\n", newLine);
   fprintf(fp, "                                                          %sIf the argument starts with @ it is assumed it is a file path\n", newLine);
   fprintf(fp, "                                                          %sE.g. 192.168.0.0/14=home,172.16.0.0/16=private\n", newLine);
-  fprintf(fp, "    [--pcap-file-list] <filename>                         %sSpecify a filename containing a list\n"
-	  "                                                              %sof pcap files to read.\n"
-	  "                                                              %sIf you use this flag the -i option will be\n"
-	  "                                                              %signored.\n", newLine, newLine, newLine, newLine);
-  fprintf(fp, "    [--enable-efficiency]                                 %sCompute network traffic efficiency on ATM cells-lit\n", newLine);
+#ifdef ENABLE_EFFICIENCY
+  fprintf(fp, "    [--enable-efficiency]                                 %sCompute network traffic efficiency on ATM cells\n", newLine);
+#endif
 
  fprintf(fp, "\n"
 	 "NOTE\n"
@@ -184,14 +190,14 @@ void usage(FILE * fp) {
 
 /* *********************************** */
 
-static void verifyOptions (void) {
+static void verifyOptions (void){
 
 #ifdef HAVE_OPENSSL
-    if((myGlobals.runningPref.webPort == 0) && (myGlobals.runningPref.sslPort == 0)) {
+  if((myGlobals.runningPref.webPort == 0) && (myGlobals.runningPref.sslPort == 0)) {
         printf("WARNING: both -W and -w are set to 0. The web interface will be disabled.\n");
 #else
         if(myGlobals.runningPref.webPort == 0) {
-            printf("WARNING: -w is set to 0. The web interface will be disabled.\n");
+	  printf("WARNING: -w is set to 0. The web interface will be disabled.\n");
 #endif
 
             traceEvent(CONST_TRACE_WARNING, "The web interface will be disabled");
@@ -344,7 +350,8 @@ static void abortfn(enum mcheck_status status) {
    if(is_daemon) {
      char **     args;
      char        execPath[PATH_MAX];
-     uint32_t    execPathSize;
+     u_int32_t    execPathSize;
+     static char *osx_daemon = "--osx-daemon";
 
      // ... process any pre-daemonization arguments ...
 
@@ -363,7 +370,7 @@ static void abortfn(enum mcheck_status status) {
      // path.
 
      execPathSize = sizeof(execPath);
-     (void) _NSGetExecutablePath(execPath, &execPathSize);
+     (void)_NSGetExecutablePath(execPath, &execPathSize);
 
      args = calloc(argc, sizeof(char *));
      args[0] = execPath;
@@ -371,13 +378,21 @@ static void abortfn(enum mcheck_status status) {
      for(j = 1, i=1; i<argc; i++) {
        if(strcmp(argv[i], "-d"))
 	 args[j++] = argv[i];
+       else
+	 args[j++] = osx_daemon;
+     }
+
+     if(0) {
+       printf("--------------------\n");
+       for(i=0; i<j; i++) printf("%s\n", args[i]);
+       printf("--------------------\n");
      }
 
      // Daemonize ourself.
-     (void) daemon(0, 0);
+     (void)daemon(0, 0);
 
      // exec ourself.
-     (void) execve(execPath, args, environ);
+     (void)execve(execPath, args, environ);
      exit(0);
    }
  }
@@ -385,7 +400,6 @@ static void abortfn(enum mcheck_status status) {
 
 /* ***************************************************** */
 
-
 /* That's the meat */
 #ifdef WIN32
 int ntop_main(int argc, char *argv[]) {
@@ -520,8 +534,8 @@ int main(int argc, char *argv[]) {
         free(cmdLineBuffer);
 	return(-1);
       }
-
-      printf("   Processing file %s for parameters...\n", &argv[i][1]);
+      
+      printf("Processing file %s for parameters...\n", &argv[i][1]);
 
       if(i > 1)
 	safe_strncat(cmdLineBuffer, LEN_CMDLINE_BUFFER, " '");
@@ -620,9 +634,9 @@ int main(int argc, char *argv[]) {
 
   verifyOptions();
 
-  traceEvent(CONST_TRACE_ALWAYSDISPLAY, "ntop v.%s", version);
+  traceEvent(CONST_TRACE_ALWAYSDISPLAY, "ntop v.%s (%d bit)", version, sizeof(long) == 8 ? 64 : 32);
   traceEvent(CONST_TRACE_ALWAYSDISPLAY, "Configured on %s, built on %s.", configureDate, buildDate);
-  traceEvent(CONST_TRACE_ALWAYSDISPLAY, "Copyright 1998-2009 by %s", author);
+  traceEvent(CONST_TRACE_ALWAYSDISPLAY, "Copyright 1998-2010 by %s", author);
   traceEvent(CONST_TRACE_ALWAYSDISPLAY, "Get the freshest ntop from http://www.ntop.org/");
 
 #ifndef WIN32
@@ -638,6 +652,10 @@ int main(int argc, char *argv[]) {
 
   traceEvent(CONST_TRACE_ALWAYSDISPLAY, "Initializing ntop");
 
+#ifdef HAVE_PYTHON
+  init_python(effective_argc, effective_argv);
+#endif
+
   reportValues(&lastTime);
 
   if(myGlobals.runningPref.P3Pcp != NULL)
@@ -648,9 +666,11 @@ int main(int argc, char *argv[]) {
       traceEvent(CONST_TRACE_ALWAYSDISPLAY, "P3P: Policy reference uri is '%s'",
 		 myGlobals.runningPref.P3Puri);
 
-  if (!myGlobals.runningPref.printIpOnly && (myGlobals.runningPref.fcNSCacheFile != NULL)) {
-      processFcNSCacheFile (myGlobals.runningPref.fcNSCacheFile);
+#ifdef ENABLE_FC
+  if(!myGlobals.runningPref.printIpOnly && (myGlobals.runningPref.fcNSCacheFile != NULL)) {
+    processFcNSCacheFile (myGlobals.runningPref.fcNSCacheFile);
   }
+#endif
 
   initNtop(myGlobals.runningPref.devices);
 
@@ -706,7 +726,7 @@ int main(int argc, char *argv[]) {
   /* ******************************* */
 
   addDefaultAdminUser();
-
+  readSessionPurgeParams();
   initReports();
 
   traceEvent(CONST_TRACE_NOISY, "MEMORY: Base interface structure (no hashes loaded) is %.2fMB each",
@@ -730,8 +750,6 @@ int main(int argc, char *argv[]) {
   startSniffer();
 
   while(myGlobals.ntopRunState == FLAG_NTOPSTATE_RUN) {
-    // traceEvent(CONST_TRACE_ERROR, "About to call event_loop()");
-    rc = event_loop(EVLOOP_NONBLOCK);
     // traceEvent(CONST_TRACE_ERROR, "event_loop() returned %d", rc);
     ntopSleepWhileSameState(5);
 
@@ -743,19 +761,21 @@ int main(int argc, char *argv[]) {
   }
 
   traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: Main thread shutting down",
-	     pthread_self());
+	     (long unsigned int)pthread_self());
   endTime = time(NULL) + PARM_SLEEP_LIMIT + 2;
 
   while((myGlobals.ntopRunState != FLAG_NTOPSTATE_TERM) &&
         (time(NULL) < endTime)) {
     sleep(1);
   }
-  traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: Main thread terminating", pthread_self());
+  traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: Main thread terminating", 
+	     (long unsigned int)pthread_self());
 
   memset(&buf, 0, sizeof(buf));
   runningThreads(buf, sizeof(buf), 0);
   if(buf[0] != '\0')
-    traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: Still running threads%s", pthread_self(), buf);
+    traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: Still running threads%s", 
+	       (long unsigned int)pthread_self(), buf);
 
   traceEvent(CONST_TRACE_INFO, "===================================");
   traceEvent(CONST_TRACE_INFO, "        ntop is shutdown...        ");
