@@ -2,7 +2,7 @@
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *                          http://www.ntop.org
  *
- *            Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
+ *            Copyright (C) 1998-2010 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -33,6 +33,7 @@ static int *servicesMapper = NULL; /* temporary value */
 /* *************************** */
 
 static void printMutexInfo(PthreadMutex *mutexId, char *mutexName) {
+#ifdef MUTEX_DEBUG
   traceEvent(CONST_TRACE_INFO, "%s is %s (last lock %s:%d) [max lock time %s:%d (%.6f sec)]",
 	     mutexName,
 	     mutexId->isLocked ? "*locked*" : "unlocked",
@@ -40,6 +41,7 @@ static void printMutexInfo(PthreadMutex *mutexId, char *mutexName) {
 	     mutexId->max.file,
 	     mutexId->max.line,
 	     mutexId->maxLockedDuration);
+#endif
 }
 
 /* *************************** */
@@ -61,7 +63,7 @@ void handleSigHup(int signalId _UNUSED_) {
     printMutexInfo(&myGlobals.device[i].packetQueueMutex, tmp);
   }
 
-  if(myGlobals.runningPref.numericFlag == 0)
+  if(myGlobals.runningPref.numericFlag != noDnsResolution)
     printMutexInfo(&myGlobals.addressResolutionMutex, "myGlobals.addressResolutionMutex");
 
   handleKnownAddresses(myGlobals.runningPref.knownSubnets);
@@ -79,7 +81,7 @@ void* pcapDispatch(void *_i) {
   struct pcap_stat pcapStats;
 
   traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: NPS(%s): pcapDispatch thread starting [p%d]",
-	     pthread_self(), myGlobals.device[i].humanFriendlyName, getpid());
+	     (long unsigned int)pthread_self(), myGlobals.device[i].humanFriendlyName, getpid());
 
   /* Reset stats before to start (needed by modern libpcap versions) */
   if(myGlobals.pcap_file_list == NULL) {
@@ -88,7 +90,7 @@ void* pcapDispatch(void *_i) {
   }
 
   traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: NPS(%s): pcapDispatch thread running [p%d]",
-	     pthread_self(), myGlobals.device[i].humanFriendlyName, getpid());
+	     (long unsigned int)pthread_self(), myGlobals.device[i].humanFriendlyName, getpid());
 
   for(;myGlobals.ntopRunState <= FLAG_NTOPSTATE_RUN;) {
     rc = pcap_dispatch(myGlobals.device[i].pcapPtr, -1, queuePacket, (u_char*)_i);
@@ -96,17 +98,27 @@ void* pcapDispatch(void *_i) {
     if(myGlobals.ntopRunState > FLAG_NTOPSTATE_RUN) break;
 
     if(rc == -1) {
-      if(myGlobals.device[i].name != NULL) /* This is not a shutdown */
+      if(myGlobals.device[i].name != NULL) { /* This is not a shutdown */
+	char *emsg = pcap_geterr(myGlobals.device[i].pcapPtr);
+
 	traceEvent(CONST_TRACE_ERROR, "Reading packets on device %d (%s): '%s'",
-		   i,
-		   myGlobals.device[i].humanFriendlyName,
-		   pcap_geterr(myGlobals.device[i].pcapPtr));
-      break;
+		   i, myGlobals.device[i].humanFriendlyName, emsg);
+ 
+	/* 
+	   Ugly way to check for EAGAIN since the errno itself is not avail 
+	   Courtesy of Andreas Stolcke <stolcke@speech.sri.com>	   
+	*/
+	if(strcmp(emsg, strerror(EAGAIN)) == 0) {
+	  sleep(1);
+	} else {
+	  break;
+	}
+      }
     } else if(rc == 0) {
       if(myGlobals.pcap_file_list != NULL) {
 	traceEvent(CONST_TRACE_INFO, "pcap_loop (%s) returned %d [No more packets to read]",
 		   myGlobals.device[i].humanFriendlyName, rc);
-      reopen_pcap:	
+
 	if(myGlobals.pcap_file_list->next != NULL) {
 	  struct fileList *fl = myGlobals.pcap_file_list;
 	  
@@ -114,23 +126,15 @@ void* pcapDispatch(void *_i) {
 	  free(fl->fileName); free(fl);
 
 	  if(myGlobals.pcap_file_list != NULL) {
-	    char ebuf[CONST_SIZE_PCAP_ERR_BUF];
-
 	    if(myGlobals.device[i].pcapPtr) pcap_close(myGlobals.device[i].pcapPtr);
-	    myGlobals.device[i].pcapPtr  = pcap_open_offline(myGlobals.pcap_file_list->fileName, ebuf);
+	    myGlobals.device[i].pcapPtr  = myGlobals.pcap_file_list->pcapPtr;
 	    
-	    if(myGlobals.device[i].pcapPtr == NULL) {
-	      traceEvent(CONST_TRACE_ERROR, "pcap_open_offline(%s): '%s'",
-			 myGlobals.pcap_file_list->fileName, ebuf);
-	      goto reopen_pcap;
-	    } else {
-	      if(myGlobals.device[i].humanFriendlyName != NULL) free(myGlobals.device[i].humanFriendlyName);
-	      myGlobals.device[i].humanFriendlyName = strdup(myGlobals.pcap_file_list->fileName);
-	      calculateUniqueInterfaceName(0);
-	      traceEvent(CONST_TRACE_INFO, "pcap_loop (%s) : reading packets from file %s",
-			 myGlobals.device[i].humanFriendlyName, myGlobals.pcap_file_list->fileName);
-	      myGlobals.device[i].datalink = pcap_datalink(myGlobals.device[i].pcapPtr);
-	    }
+	    if(myGlobals.device[i].humanFriendlyName != NULL) free(myGlobals.device[i].humanFriendlyName);
+	    myGlobals.device[i].humanFriendlyName = strdup(myGlobals.pcap_file_list->fileName);
+	    calculateUniqueInterfaceName(0);
+	    traceEvent(CONST_TRACE_INFO, "pcap_loop (%s) : reading packets from file %s",
+		       myGlobals.device[i].humanFriendlyName, myGlobals.pcap_file_list->fileName);
+	    myGlobals.device[i].datalink = pcap_datalink(myGlobals.device[i].pcapPtr);	    
 	  } else
 	    break; /* No more packets to read */
 	} else	
@@ -142,7 +146,7 @@ void* pcapDispatch(void *_i) {
   myGlobals.device[i].pcapDispatchThreadId = 0;
 
   traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: NPS(%s): pcapDispatch thread terminated [p%d]",
-	     pthread_self(), myGlobals.device[i].humanFriendlyName, getpid());
+	     (long unsigned int)pthread_self(), myGlobals.device[i].humanFriendlyName, getpid());
 
   return(NULL);
 }
@@ -200,14 +204,15 @@ void daemonizeUnderUnix(void) {
   }
 
   myGlobals.mainThreadId = pthread_self();
-  traceEvent(CONST_TRACE_ALWAYSDISPLAY, "THREADMGMT[t%lu]: Now running as a daemon", myGlobals.mainThreadId);
+  traceEvent(CONST_TRACE_ALWAYSDISPLAY, "THREADMGMT[t%lu]: Now running as a daemon", 
+	     (long unsigned int)myGlobals.mainThreadId);
 
 #endif
 }
 
 /* **************************************** */
 
-void detachFromTerminalUnderUnix(int doChdir) {
+void detachFromTerminalUnderUnix(int doChdir /* if doChdir=1 ntop becomes a daemon */) {
 #ifndef WIN32
 #ifdef MAKE_WITH_SYSLOG
   /* Child processes must log to syslog.
@@ -227,14 +232,17 @@ void detachFromTerminalUnderUnix(int doChdir) {
 
   setsid();  /* detach from the terminal */
 
-  fclose(stdin);
-  fclose(stdout);
-  /* fclose(stderr); */
+  if(doChdir) {
+    /* Don't close streams as python libs might get confused */
+    fclose(stdin);
+    fclose(stdout);
+    fclose(stderr);
+  }
 
   /*
    * clear any inherited file mode creation mask
    */
-  umask (0);
+  umask(0);
 
   /*
    * Use line buffered stdout
@@ -445,7 +453,7 @@ void createPortHash(void) {
 /* **************************************** */
 
 void handleProtocols(void) {
-  char *proto, *buffer=NULL, *strtokState,
+  char *proto, *buffer=NULL, *strtokState = NULL,
 	   *bufferCurrent, *bufferWork, tmpStr[512];
   FILE *fd;
 
@@ -546,6 +554,8 @@ void handleProtocols(void) {
     }
 
     if(!ignore_protocol) {
+      char protoStr[256] = { '\0' };
+
       protoName = strchr(proto, '=');
 
       if(protoName == NULL)
@@ -554,22 +564,20 @@ void handleProtocols(void) {
 		   proto);
       else {
 	protoName[0] = '\0';
-	memset(tmpStr, 0, sizeof(tmpStr));
-	strncpy(tmpStr, &protoName[1], sizeof(tmpStr));
-	len = strlen(tmpStr);
+	strncpy(protoStr, &protoName[1], sizeof(protoStr));
+	len = strlen(protoStr);
 
-	if(tmpStr[len-1] != '|') {
+	if(protoStr[len-1] != '|') {
 	  /* Make sure that the string ends with '|' */
-	  tmpStr[len] = '|';
-	  tmpStr[len+1] = '\0';
+	  protoStr[len] = '|';
+	  protoStr[len+1] = '\0';
 	}
 
 #ifdef DEBUG
-	traceEvent(CONST_TRACE_INFO, "          %30s %s", proto, tmpStr);
+	traceEvent(CONST_TRACE_INFO, "          %30s %s", proto, protoStr);
 #endif
 
-	handleProtocolList(proto, tmpStr);
-
+	handleProtocolList(proto, protoStr);
       }
     }
 
@@ -587,7 +595,7 @@ void addDefaultProtocols(void) {
   handleProtocolList("HTTP",     "http|www|https|3128|"); /* 3128 is HTTP cache */
   handleProtocolList("DNS",      "name|domain|");
   handleProtocolList("Telnet",   "telnet|login|");
-  handleProtocolList("NBios-IP", "netbios-ns|netbios-dgm|netbios-ssn|");
+  handleProtocolList("NBios/CIFS", "netbios-ns|netbios-dgm|netbios-ssn|445");
   handleProtocolList("Mail",     "pop-2|pop-3|pop3|kpop|smtp|imap|imap2|");
   handleProtocolList("DHCP-BOOTP", "67-68|");
   handleProtocolList("SNMP",     "snmp|snmp-trap|");
@@ -601,9 +609,7 @@ void addDefaultProtocols(void) {
   /* Peer-to-Peer Protocols */
   myGlobals.GnutellaIdx = handleProtocolList("Gnutella", "6346|6347|6348|");
   myGlobals.KazaaIdx = handleProtocolList("Kazaa",       "1214|");
-  myGlobals.WinMXIdx = handleProtocolList("WinMX",       "6699|7730|");
-  myGlobals.DirectConnectIdx = handleProtocolList("DC++", "-1|"); /* Dummy port as this is a pure P2P protocol */
-  myGlobals.EdonkeyIdx = handleProtocolList("eDonkey", "4661-4665|");
+    myGlobals.EdonkeyIdx = handleProtocolList("eDonkey", "4661-4665|");
   myGlobals.BitTorrentIdx = handleProtocolList("BitTorrent", "6881-6999|6969|"); /* http://www.dessent.net/btfaq/#ports */
   handleProtocolList("Messenger", "1863|5000|5001|5190-5193|");
 }
@@ -668,12 +674,12 @@ static void purgeIpPorts(int theDevice) {
 
 void* scanIdleLoop(void* notUsed _UNUSED_) {
   traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: SIH: Idle host scan thread starting [p%d]",
-             pthread_self(), getpid());
+             (long unsigned int)pthread_self(), getpid());
 
   ntopSleepUntilStateRUN();
 
   traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: SIH: Idle host scan thread running [p%d]",
-             pthread_self(), getpid());
+             (long unsigned int)pthread_self(), getpid());
 
   for(;;) {
     int i, purged=0;
@@ -702,7 +708,7 @@ void* scanIdleLoop(void* notUsed _UNUSED_) {
   myGlobals.scanIdleThreadId = 0;
 
   traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: SIH: Idle host scan thread terminated [p%d]",
-             pthread_self(), getpid());
+             (long unsigned int)pthread_self(), getpid());
 
   return(NULL);
 }
@@ -714,14 +720,14 @@ void* scanFingerprintLoop(void* notUsed _UNUSED_) {
   int deviceId, countScan, countResolved, countCycle;
 
   traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: SFP: Fingerprint scan thread starting [p%d]",
-             pthread_self(), getpid());
+             (long unsigned int)pthread_self(), getpid());
 
   countCycle=0;
 
   ntopSleepUntilStateRUN();
 
   traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: SFP: Fingerprint scan thread running [p%d]",
-             pthread_self(), getpid());
+             (long unsigned int)pthread_self(), getpid());
 
   for(;;) {
     countScan=0;
@@ -764,7 +770,7 @@ void* scanFingerprintLoop(void* notUsed _UNUSED_) {
   myGlobals.scanFingerprintsThreadId = 0;
 
   traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: SFP: Fingerprint scan thread terminated [p%d]",
-             pthread_self(), getpid());
+             (long unsigned int)pthread_self(), getpid());
 
   return(NULL);
 }
@@ -785,11 +791,6 @@ static void cleanupThreadIs(char *buf, int sizeofbuf) {
       strncpy(buf, "SIH", sizeofbuf);
     else if(pthread_self() == myGlobals.handleWebConnectionsThreadId)
       strncpy(buf, "WEB", sizeofbuf);
-
-#if defined(HAVE_OPENSSL) && defined(MAKE_WITH_SSLWATCHDOG)
-    else if(pthread_self() == myGlobals.sslwatchdogChildThreadId)
-      strncpy(buf, "SSL", sizeofbuf);
-#endif
   }
 
   if(buf[0] == '\0') {
@@ -847,19 +848,25 @@ void runningThreads(char *buf, int sizeofbuf, int do_join) {
   if(!do_join) {
     memset(&buf2, 0, sizeof(buf2));
 
-#if defined(HAVE_OPENSSL) && defined(MAKE_WITH_SSLWATCHDOG)
-    safe_snprintf(__FILE__, __LINE__, buf, sizeofbuf, "%s%s%s%s",
-		  myGlobals.scanFingerprintsThreadId != 0 ? " SFP" : "",
-		  myGlobals.scanIdleThreadId != 0 ? " SIH" : "",
-		  myGlobals.handleWebConnectionsThreadId != 0 ? " WEB" : "",
-		  myGlobals.sslwatchdogChildThreadId != 0 ? " SSL" : "");
-#else
     safe_snprintf(__FILE__, __LINE__, buf, sizeofbuf, "%s%s%s",
 		  myGlobals.scanFingerprintsThreadId != 0 ? " SFP" : "",
 		  myGlobals.scanIdleThreadId != 0 ? " SIH" : "",
 		  myGlobals.handleWebConnectionsThreadId != 0 ? " WEB" : "");
-#endif
+  }
 
+  for(i=0; i<myGlobals.numDequeueAddressThreads; i++) {
+    if(myGlobals.dequeueAddressThreadId[i] != 0) {
+      if(!do_join) {
+        safe_snprintf(__FILE__, __LINE__, buf2, sizeof(buf2), " DNSAR%d", i+1);
+        safe_strncat(buf, sizeofbuf, buf2);
+      } else {
+        /* Wake up thread */
+        signalCondvar(&myGlobals.queueAddressCondvar);
+        traceEvent(CONST_TRACE_INFO, "Joining thread DNSAR%d", i+1);
+        if(joinThread(&myGlobals.dequeueAddressThreadId[i]) != 0)
+          traceEvent(CONST_TRACE_INFO, "joinThread() returned %s", strerror(errno));
+      }
+    }
   }
 
   if(myGlobals.allDevs != NULL) {
@@ -892,7 +899,7 @@ void runningThreads(char *buf, int sizeofbuf, int do_join) {
 		     formatPkts(myGlobals.device[i].droppedPkts.value, buf2, sizeof(buf2)));
 
 	  /* signalCondvar(&myGlobals.device[i].queueCondvar); */
-	  pcap_close(myGlobals.device[i].pcapPtr);
+	  /* pcap_close(myGlobals.device[i].pcapPtr); */
 
 	  traceEvent(CONST_TRACE_INFO, "Joining thread NPS(%s) [t%lu]",
 		     myGlobals.device[i].humanFriendlyName,
@@ -910,8 +917,8 @@ void runningThreads(char *buf, int sizeofbuf, int do_join) {
 	  safe_snprintf(__FILE__, __LINE__, buf2, sizeof(buf2), " NF%d", i);
 	  safe_strncat(buf, sizeofbuf, buf2);
 	} else {
-	  traceEvent(CONST_TRACE_INFO, "Joining thread NF%d [%u]", i,
-		     (unsigned int)myGlobals.device[i].netflowGlobals->netFlowThread);
+	  traceEvent(CONST_TRACE_INFO, "Joining thread NFt%lu [%u]",
+		     (long unsigned int)myGlobals.device[i].netflowGlobals->netFlowThread, i);
 	  close(myGlobals.device[i].netflowGlobals->netFlowInSocket);
 
 	  if(joinThread(&myGlobals.device[i].netflowGlobals->netFlowThread) != 0)
@@ -971,7 +978,7 @@ RETSIGTYPE cleanup(int signo) {
   }
 
   traceEvent(CONST_TRACE_INFO, "CLEANUP[t%lu]: ntop caught signal %d [state=%d]", 
-	     pthread_self(), signo, myGlobals.ntopRunState);
+	     (long unsigned int)pthread_self(), signo, myGlobals.ntopRunState);
 
   if(cleanup_called) {
     traceEvent(CONST_TRACE_INFO, "ntop is now quitting...");
@@ -1024,35 +1031,21 @@ RETSIGTYPE cleanup(int signo) {
   setRunState(FLAG_NTOPSTATE_SHUTDOWN);
 
   if(cleanup_called) {
-    traceEvent(CONST_TRACE_INFO, "CLEANUP[t%lu]: exiting", pthread_self());
+    traceEvent(CONST_TRACE_INFO, "CLEANUP[t%lu]: exiting", 
+	       (long unsigned int)pthread_self());
     exit(0);
   }
 
   cleanup_called = 1;
 
   cleanupThreadIs(buf, sizeof(buf));
-  traceEvent(CONST_TRACE_INFO, "CLEANUP[t%lu] catching thread is %s", pthread_self(), buf);
+  traceEvent(CONST_TRACE_INFO, "CLEANUP[t%lu] catching thread is %s", 
+	     (long unsigned int)pthread_self(), buf);
 
   runningThreads(buf, sizeof(buf), 0);
   traceEvent(CONST_TRACE_INFO, "CLEANUP: Running threads%s", buf);
   runningThreads(buf, sizeof(buf), 1);
 
-#ifndef WIN32
-
-#ifdef MAKE_WITH_SSLWATCHDOG
-  if(myGlobals.sslwatchdogChildThreadId != 0) {
-    killThread(&myGlobals.sslwatchdogChildThreadId);
-  }
-#ifdef MAKE_WITH_SSLWATCHDOG_RUNTIME
-  if(myGlobals.runningPref.useSSLwatchdog == 1)
-#endif
-    {
-      deleteCondvar(&myGlobals.sslwatchdogCondvar);
-    }
-#endif
-
-#endif /* #ifndef WIN32 */
-
   killThread(&myGlobals.handleWebConnectionsThreadId);
   killThread(&myGlobals.scanIdleThreadId);
   killThread(&myGlobals.scanFingerprintsThreadId);
@@ -1097,13 +1090,15 @@ RETSIGTYPE cleanup(int signo) {
   for(i=0; i<myGlobals.numDevices; i++) {
     tryLockMutex(&myGlobals.device[i].asMutex, "cleanup");
     deleteMutex(&myGlobals.device[i].asMutex);
+    tryLockMutex(&myGlobals.device[i].counterMutex, "cleanup");
+    deleteMutex(&myGlobals.device[i].counterMutex);
     tryLockMutex(&myGlobals.device[i].packetProcessMutex, "cleanup");
     deleteMutex(&myGlobals.device[i].packetProcessMutex);
     tryLockMutex(&myGlobals.device[i].packetQueueMutex, "cleanup");
     deleteMutex(&myGlobals.device[i].packetQueueMutex);
   }
 
-  if(myGlobals.runningPref.numericFlag == 0) {
+  if(myGlobals.runningPref.numericFlag != noDnsResolution) {
     tryLockMutex(&myGlobals.addressResolutionMutex, "cleanup");
     deleteMutex(&myGlobals.addressResolutionMutex);
   }
@@ -1131,6 +1126,11 @@ RETSIGTYPE cleanup(int signo) {
   tryLockMutex(&myGlobals.purgeMutex, "cleanup");
   deleteMutex(&myGlobals.purgeMutex);
 
+#ifdef HAVE_PYTHON
+  if(myGlobals.runningPref.debugMode)
+    term_python();
+#endif
+
   for(i=0; i<myGlobals.numDevices; i++) {
     traceEvent(CONST_TRACE_INFO, "CLEANUP: Freeing device %s",
 	       myGlobals.device[i].humanFriendlyName);
@@ -1171,7 +1171,8 @@ RETSIGTYPE cleanup(int signo) {
     if(myGlobals.device[i].packetQueue)
       free(myGlobals.device[i].packetQueue);
 
-    free(myGlobals.device[i].tcpSession);
+    if(myGlobals.device[i].tcpSession) free(myGlobals.device[i].tcpSession);
+
     free(myGlobals.device[i].humanFriendlyName);
     free(myGlobals.device[i].uniqueIfName);
     free(myGlobals.device[i].name);
@@ -1285,7 +1286,8 @@ RETSIGTYPE cleanup(int signo) {
   memset(&buf, 0, sizeof(buf));
   runningThreads(buf, sizeof(buf), 0);
   if(buf[0] != '\0')
-    traceEvent(CONST_TRACE_INFO, "CLEANUP[t%lu]: Still running threads%s", pthread_self(), buf);
+    traceEvent(CONST_TRACE_INFO, "CLEANUP[t%lu]: Still running threads%s", 
+	       (long unsigned int)pthread_self(), buf);
 
   traceEvent(CONST_TRACE_INFO, "===================================");
   traceEvent(CONST_TRACE_INFO, "        ntop is shutdown...        ");
