@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2010 Luca Deri <deri@ntop.org>
  *
  *  			    http://www.ntop.org/
  *
@@ -43,7 +43,7 @@ static void loadPlugin(char* dirName, char* pluginName) {
   char key[64], value[16];
   int rc;
 #ifndef WIN32
-  PluginInfo* (*pluginJumpFunc)();
+  PluginInfo* (*pluginJumpFunc)(void);
 #endif
   FlowFilterList *newFlow,
     *work, *prev = NULL;
@@ -74,7 +74,7 @@ static void loadPlugin(char* dirName, char* pluginName) {
     return;
   }
 
-  pluginJumpFunc = (PluginInfo*(*)())pluginEntryFctnPtr;
+  pluginJumpFunc = (PluginInfo*(*)(void))pluginEntryFctnPtr;
   pluginInfo = pluginJumpFunc();
 #else /* MAKE_STATIC_PLUGIN */
 
@@ -311,14 +311,14 @@ void startPlugins(void) {
 
 /* ******************* */
 
-void handlePluginHostCreationDeletion(HostTraffic *el, u_short deviceId, u_char hostCreation) {
+void handlePluginHostCreationDeletion(HostTraffic *el, u_short deviceId, u_char host_creation) {
   FlowFilterList *flows = myGlobals.flowsList;
 
   while(flows != NULL) {
     if(flows->pluginStatus.pluginMemoryPtr != NULL) {
       if(flows->pluginStatus.activePlugin 
 	 && (flows->pluginStatus.pluginPtr->crtDltFunct != NULL))
-	flows->pluginStatus.pluginPtr->crtDltFunct(el, deviceId, hostCreation);
+	flows->pluginStatus.pluginPtr->crtDltFunct(el, deviceId, host_creation);
     }
     
     flows = flows->next;
