@@ -1,5 +1,5 @@
 /*
- *  Copyright(C) 2008-09 Luca Deri <deri@ntop.org>
+ *  Copyright(C) 2008-10 Luca Deri <deri@ntop.org>
  *
  *  		       http://www.ntop.org/
  *
@@ -55,7 +55,7 @@ static void handlecPacketHTTPrequest(char* url);
 static void printcPacketConfiguration(int deviceId);
 static int createcPacketDevice(int cpacketDeviceId);
 static int mapcPacketDeviceToNtopDevice(int deviceId);
-static void printcPacketStatistics();
+static void printcPacketStatistics(void);
 static void printcPacketCounterStats(int deviceId, int page_header, int print_table);
 
 /* ****************************** */
@@ -315,7 +315,7 @@ static void* cpacketMainLoop(void* _deviceId) {
   if(!(myGlobals.device[deviceId].cpacketGlobals->cpacketInSocket > 0)) return(NULL);
 
   traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: CPACKET: thread starting [p%d]",
-             pthread_self(), getpid());
+             (long unsigned int)pthread_self(), getpid());
 
 #ifdef MAKE_WITH_CPACKETSIGTRAP
   signal(SIGSEGV, cpacketcleanup);
@@ -351,7 +351,7 @@ static void* cpacketMainLoop(void* _deviceId) {
 
   ntopSleepUntilStateRUN();
   traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: CPACKET: (port %d) thread running [p%d]",
-             pthread_self(), myGlobals.device[deviceId].cpacketGlobals->cpacketInPort, getpid());
+             (long unsigned int)pthread_self(), myGlobals.device[deviceId].cpacketGlobals->cpacketInPort, getpid());
 
   for(;myGlobals.ntopRunState <= FLAG_NTOPSTATE_RUN;) {
     int maxSock = myGlobals.device[deviceId].cpacketGlobals->cpacketInSocket;
@@ -413,7 +413,7 @@ static void* cpacketMainLoop(void* _deviceId) {
   myGlobals.device[deviceId].activeDevice = 0;
 
   traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: CPACKET: thread terminated [p%d][cpacketDeviceId=%d]",
-	     pthread_self(), getpid(), myGlobals.device[deviceId].cpacketGlobals->cpacketDeviceId);
+	     (long unsigned int)pthread_self(), getpid(), myGlobals.device[deviceId].cpacketGlobals->cpacketDeviceId);
 
   return(NULL);
 }
@@ -440,8 +440,7 @@ static char* cpValue(int deviceId, char *name, int appendDeviceId) {
 /* ****************************** */
 
 static void initcPacketDevice(int deviceId) {
-  int a, b, c, d, a1, b1, c1, d1, rc;
-  char value[1024], workList[1024];
+  char value[1024];
 
   if(!pluginActive) return;
 
@@ -585,7 +584,7 @@ static void printcPacketDeviceConfiguration(void) {
 
 /* ****************************** */
 
-static void printcPacketStatistics() {
+static void printcPacketStatistics(void) {
   int i, printedStatistics = 0;
   char buf[1024], formatBuf[64];
 
@@ -625,7 +624,6 @@ static void printcPacketStatistics() {
 
 static void printcPacketCounterStats(int deviceId, int page_header, int print_table) {
   char buf[1024], *title = "cTap Counters";
-  int i, printedStatistics=0;
 
   if((deviceId < 0) || (deviceId >myGlobals.numDevices)) return;
   
@@ -697,7 +695,7 @@ static void printcPacketCounterStats(int deviceId, int page_header, int print_ta
 /* ****************************** */
 
 static void printcPacketConfiguration(int deviceId) {
-  char buf[512], buf1[32], buf2[32];
+  char buf[512];
 
 #define UDPSLASHSCPT "UDP"
 
