@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-09 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 2002-10 Luca Deri <deri@ntop.org>
  *
  *  		       http://www.ntop.org/
  *
@@ -186,7 +186,8 @@ static char *rrdNames[] = {
 /* RRD_DEBUG controls debug messages in rrdPlugin.c.  See the definition in globals-defines.h,
  * where you really should set it, so the util.c calls get enabled.
  */
-#define RRD_DEBUG 3
+//#define RRD_DEBUG 3
+#undef RRD_DEBUG
 
 /*
  * Optional: Set a default font for the rrd generated graphs
@@ -374,3 +375,6 @@ static int rrdcycleBufferInit,
            rrdcycleBufferCount;
 static float rrdcmaxLength;
 #endif
+
+#define RRD_DEFAULT_SPACER_LEN    20
+#define RRD_SHORT_SPACER_LEN      11
