@@ -1,8 +1,8 @@
-/**
+/*
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *                          http://www.ntop.org
  *
- * Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
+ * Copyright (C) 1998-2010 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -56,11 +56,32 @@ static char __see__ []    =
    * ('P' option) and so these are processed separately.
    */
 #ifdef WIN32
-static char*  short_options = "46a:bce:f:ghi:jl:m:nop:qr:st:w:x:zAB:C:D:F:MN:O:P:Q:S:U:VX:W:";
+static char*  short_options = "46a:bce:f:ghi:jl:m:n:op:qr:st:w:x:zAB:C:D:F:M" 
+#ifdef ENABLE_FC
+  "N:"
+#endif
+#if defined(DARWIN) && (!defined(TIGER))
+  "v"
+#endif
+  "O:P:Q:S:U:VX:W:";
 #elif defined(MAKE_WITH_SYSLOG)
-static char*  short_options = "46a:bcde:f:ghi:jl:m:nop:qr:st:u:w:x:zAB:C:D:F:IKLMN:O:P:Q:S:U:VX:W:";
+static char*  short_options = "46a:bcde:f:ghi:jl:m:n:op:qr:st:u:w:x:zAB:C:D:F:IKLM" 
+#ifdef ENABLE_FC
+  "N:"
+#endif
+#if defined(DARWIN) && (!defined(TIGER))
+  "v"
+#endif
+  "O:P:Q:S:U:VX:W:";
 #else
-static char*  short_options = "46a:bcde:f:ghi:jl:m:nop:qr:st:u:w:x:zAB:C:D:F:IKMN:O:P:Q:S:U:VX:W:";
+static char*  short_options = "46a:bcde:f:ghi:jl:m:n:op:qr:st:u:w:x:zAB:C:D:F:IKM"
+#ifdef ENABLE_FC
+  "N:"
+#endif
+#if defined(DARWIN) && (!defined(TIGER))
+  "v"
+#endif
+  "O:P:Q:S:U:VX:W:";
 #endif
 
 static struct option const long_options[] = {
@@ -82,7 +103,7 @@ static struct option const long_options[] = {
   { "create-other-packets",             no_argument,       NULL, 'j' },
   { "pcap-log",                         required_argument, NULL, 'l' },
   { "local-subnets",                    required_argument, NULL, 'm' },
-  { "numeric-ip-addresses",             no_argument,       NULL, 'n' },
+  { "numeric-ip-addresses",             required_argument, NULL, 'n' },
   { "no-mac",                           no_argument,       NULL, 'o' },
 
   { "protocols",                        required_argument, NULL, 'p' },
@@ -95,6 +116,10 @@ static struct option const long_options[] = {
   { "user",                             required_argument, NULL, 'u' },
 #endif
 
+#if defined(DARWIN) && (!defined(TIGER))
+  { "osx-daemon",                       no_argument,       NULL, 'v' },
+#endif
+
   { "http-server",                      required_argument, NULL, 'w' },
   { "disable-sessions",                 no_argument,       NULL, 'z' },
   { "filter-expression",                required_argument, NULL, 'B' },
@@ -109,7 +134,9 @@ static struct option const long_options[] = {
 
   { "no-interface-merge",               no_argument,       NULL, 'M' },
 
+#ifdef ENABLE_FC
   { "wwn-map",                          required_argument, NULL, 'N' },
+#endif
 
   { "output-packet-path",               required_argument, NULL, 'O' },
   { "db-file-path",                     required_argument, NULL, 'P' },
@@ -131,13 +158,7 @@ static struct option const long_options[] = {
   { "use-syslog",                       optional_argument, NULL, 131 },
 #endif
 
-#ifdef MAKE_WITH_SSLWATCHDOG_RUNTIME
-  { "ssl-watchdog",                     no_argument,       NULL, 133 },
-#endif
-
-#ifdef MAKE_WITH_SCHED_YIELD
-  { "disable-schedyield",               optional_argument, NULL, 134 },
-#endif
+  /* 132/133/134 are AVAILABLE */
 
   { "set-admin-password",               optional_argument, NULL, 135 },
 
@@ -152,15 +173,18 @@ static struct option const long_options[] = {
   { "disable-instantsessionpurge",      no_argument,       NULL, 144 },
   { "disable-mutexextrainfo",           no_argument,       NULL, 145 },
 
+#ifdef ENABLE_FC
   { "fc-only",                          no_argument,       NULL, 147 },
   { "no-fc",                            no_argument,       0,    148 },
   { "no-invalid-lun",                   no_argument,       0,    149 },
+#endif
 
   { "skip-version-check",               required_argument, NULL, 150 },
-  { "pcap-file-list",                   required_argument, NULL, 151 },
-  { "known-subnets",                    required_argument, NULL, 152 },
-  { "live",                             no_argument,       NULL, 153 },
-  { "enable-efficiency",                no_argument,       NULL, 154 },
+  { "known-subnets",                    required_argument, NULL, 151 },
+  { "live",                             no_argument,       NULL, 152 },
+#ifdef ENABLE_EFFICIENCY
+  { "enable-efficiency",                no_argument,       NULL, 153 },
+#endif
 
   {NULL, 0, NULL, 0}
 };
@@ -180,6 +204,7 @@ void loadPrefs(int argc, char* argv[]) {
 
   traceEvent(CONST_TRACE_NOISY, "NOTE: Processing parameters (pass1)");
   opt_index = 0, optind = 0;
+    
   while ((opt = getopt_long(argc, argv, short_options, long_options, &opt_index)) != EOF) {
     switch (opt) {
     case 'h': /* help */
@@ -237,11 +262,10 @@ void loadPrefs(int argc, char* argv[]) {
   traceEvent(CONST_TRACE_NOISY, "NOTE: Reading preferences file entries");
   key = gdbm_firstkey(myGlobals.prefsFile);
   while (key.dptr) {
-
     /* Handle key w/o trailing \0 so valgrind is happy */
     zeroPadMallocString(key.dsize, key.dptr);
 
-    if (fetchPrefsValue(key.dptr, buf, sizeof(buf)) == 0) {
+    if(fetchPrefsValue(key.dptr, buf, sizeof(buf)) == 0) {
       processNtopPref(key.dptr, buf, FALSE, &myGlobals.runningPref);
     }
 
@@ -263,11 +287,10 @@ void loadPrefs(int argc, char* argv[]) {
 /* ***************************************************** */
 
 static void readPcapFileList(char * filenames) {
-  const int block = 32;
-  char *line;
-  int i=0,j=0, idx;
+  char *line, ebuf[CONST_SIZE_PCAP_ERR_BUF];
+  int idx;
   struct fileList *fl, *prev;
-  
+
   line = strtok(filenames, ",");
 
   while(line != NULL) {
@@ -276,7 +299,7 @@ static void readPcapFileList(char * filenames) {
       fl = (struct fileList*)malloc(sizeof(struct fileList));
 
       if(!fl) {
-	traceEvent(CONST_TRACE_ERROR, "Not enough memory parsing --pcap-file-list argument");
+	traceEvent(CONST_TRACE_ERROR, "Not enough memory parsing -f argument");
 	return;
       }
 
@@ -286,24 +309,32 @@ static void readPcapFileList(char * filenames) {
 
       if(!fl->fileName) {
 	free(fl);
-	traceEvent(CONST_TRACE_ERROR, "Not enough memory parsing --pcap-file-list argument");
+	traceEvent(CONST_TRACE_ERROR, "Not enough memory parsing -f argument");
 	return;
       }
 
-      fl->next = NULL;      
-      
-      if(myGlobals.pcap_file_list != NULL) {
-	prev = myGlobals.pcap_file_list;
-	while(prev != NULL) {
-	  if(prev->next)
-	    prev = prev->next;
-	  else
-	    break;
-	}
+      fl->pcapPtr = pcap_open_offline(fl->fileName, ebuf);
+
+      if(fl->pcapPtr == NULL) {
+	traceEvent(CONST_TRACE_ERROR, "Skipping pcap file %s: '%s'", fl->fileName, ebuf);
+	free(fl->fileName);
+	free(fl);
+      } else {	
+	fl->next = NULL;      
 	
-	prev->next = fl;
-      } else
-	myGlobals.pcap_file_list = fl;
+	if(myGlobals.pcap_file_list != NULL) {
+	  prev = myGlobals.pcap_file_list;
+	  while(prev != NULL) {
+	    if(prev->next)
+	      prev = prev->next;
+	    else
+	      break;
+	  }
+	
+	  prev->next = fl;
+	} else
+	  myGlobals.pcap_file_list = fl;
+      }
     }
 
     line = strtok(NULL, ",");
@@ -367,6 +398,7 @@ int parseOptions(int argc, char* argv[]) {
 #ifndef WIN32
     case 'd':
       myGlobals.runningPref.daemonMode = 1;
+      /* traceEvent(CONST_TRACE_ERROR, "++++ DEMON MODE=%d\n", myGlobals.runningPref.daemonMode); */
       break;
 #endif
 
@@ -408,7 +440,11 @@ int parseOptions(int argc, char* argv[]) {
       break;
 
     case 'n':
-      myGlobals.runningPref.numericFlag = 1;
+      myGlobals.runningPref.numericFlag = atoi(optarg);
+      if(myGlobals.runningPref.numericFlag > dnsResolutionForAll) {
+	traceEvent(CONST_TRACE_WARNING, "Invalid value for -n: setting it to 0");
+	myGlobals.runningPref.numericFlag = noDnsResolution;
+      }
       break;
 
     case 'o': /* Do not trust MAC addresses */
@@ -464,6 +500,12 @@ int parseOptions(int argc, char* argv[]) {
       break;
 #endif /* WIN32 */
 
+#if defined(DARWIN) && (!defined(TIGER))
+    case 'v':
+      myGlobals.runningPref.daemonMode = 1;
+      break;
+#endif
+
     case 'w':
       stringSanityCheck(optarg, "-w | --http-server");
       if(!isdigit(optarg[0])) {
@@ -532,10 +574,12 @@ int parseOptions(int argc, char* argv[]) {
       traceEvent(CONST_TRACE_ALWAYSDISPLAY, "NOTE: Interface merge disabled due to command line switch");
       break;
 
+#ifdef ENABLE_FC
     case 'N':
       pathSanityCheck(optarg, "-N | --wwn-map");
       myGlobals.runningPref.fcNSCacheFile = strdup (optarg);
       break;
+#endif
 
     case 'O': /* pcap log path - Ola Lundqvist <opal@debian.org> */
       pathSanityCheck(optarg, "-O | --output-packet-path");
@@ -616,18 +660,18 @@ int parseOptions(int argc, char* argv[]) {
        *     -L sets myGlobals.useSyslog to the default facility (DEFAULT_SYSLOG_FACILITY in ntop.h)
        *     --use-syslog requires a facility parameter (see /usr/include/sys/syslog.h)
        */
-      if (optarg) {
+      if(optarg) {
 	int i;
 
 	stringSanityCheck(optarg, "--use-syslog");
 
 	for (i=0; myFacilityNames[i].c_name != NULL; i++) {
-	  if (strcmp(optarg, myFacilityNames[i].c_name) == 0) {
+	  if(strcmp(optarg, myFacilityNames[i].c_name) == 0) {
 	    break;
 	  }
 	}
 
-	if (myFacilityNames[i].c_name == NULL) {
+	if(myFacilityNames[i].c_name == NULL) {
 	  printf("WARNING: --use-syslog=unknown log facility('%s'), using default value\n",
 		 optarg);
 	  myGlobals.runningPref.useSyslog = DEFAULT_SYSLOG_FACILITY;
@@ -641,22 +685,9 @@ int parseOptions(int argc, char* argv[]) {
       break;
 #endif
 
-#ifdef MAKE_WITH_SSLWATCHDOG_RUNTIME
-    case 133:
-      /* Burton M. Strauss III - Jun 2002 */
-      myGlobals.runningPref.useSSLwatchdog = 1;
-      break;
-#endif
-
-#ifdef MAKE_WITH_SCHED_YIELD
-    case 134: /* disable-schedyield */
-      myGlobals.runningPref.disableSchedYield = TRUE;
-      break;
-#endif
-
     case 135:
       /* Dennis Schoen (dennis@cns.dnsalias.org) allow --set-admin-password=<password> */
-      if (optarg) {
+      if(optarg) {
         stringSanityCheck(optarg, "--set-admin-password");
 	adminPw = strdup(optarg);
       } else {
@@ -746,22 +777,19 @@ int parseOptions(int argc, char* argv[]) {
       break;
 
     case 151:
-      stringSanityCheck(optarg, "--pcap-file-list");
-      readPcapFileList(optarg);
-      break;
-
-    case 152:
       stringSanityCheck(optarg, "--known-subnets");
       myGlobals.runningPref.knownSubnets = strdup(optarg);
       break;
 
-    case 153:
+    case 152:
       myGlobals.runningPref.liveMode = 1;
       break;
 
-    case 154:
+#ifdef ENABLE_EFFICIENCY
+    case 153:
       myGlobals.runningPref.calculateEfficiency = 1;
       break;
+#endif
 
     default:
       printf("FATAL ERROR: unknown ntop option, '%c'\n", opt);
@@ -782,12 +810,30 @@ int parseOptions(int argc, char* argv[]) {
 
   /* *********************** */
 
+  if(myGlobals.runningPref.daemonMode) {
+    /* 
+       In order to avoid ntop fail to start when running in daemon mode
+       we check if the admin password has been set and if not we set 
+       the default password. This way we can avoid ntop startup fail 
+    */
+    setAdminPw = 1; /* NOTE: the password is not overwritten if already present */
+    if(adminPw == NULL) adminPw = "admin";
+  }
+
   if(setAdminPw) {
     setAdminPassword(adminPw);
-    termGdbm();
-    exit(0);
+
+    if(!myGlobals.runningPref.daemonMode) {
+      termGdbm();
+      exit(0);
+    }
   }
 
+#if defined(DARWIN) && (!defined(TIGER))
+  /* Trick for OSX: search check_osx_daemonization */
+  myGlobals.runningPref.daemonMode = 0;
+#endif
+
 #ifndef WIN32
   /* Handle any unrecognized options, such as a nested @filename */
   if(optind < argc) {
@@ -849,8 +895,6 @@ int parseOptions(int argc, char* argv[]) {
   }
 #endif
 
-  
-
   return(userSpecified);
 }
 
@@ -946,6 +990,8 @@ static void storeGdbmValue(GDBM_FILE gdbmfile, char *key, char *value) {
 
 void storePrefsValue(char *key, char *value) {
   storeGdbmValue(myGlobals.prefsFile, key, value);
+  checkCommunities(); /* Check if communities are defined */
+  readSessionPurgeParams(); /* Re-read if necessary */
 }
 
 /* ******************************** */
@@ -991,6 +1037,7 @@ static void delGdbmValue(GDBM_FILE gdbmfile, char *key) {
 
 void delPrefsValue(char *key) {
   delGdbmValue(myGlobals.prefsFile, key);
+  checkCommunities(); /* Check if communities are defined */
 }
 
 /* ******************************** */
@@ -1003,23 +1050,23 @@ void delPwValue(char *key) {
 
 void processStrPref(char *key, char *value, char **globalVar, bool savePref)
 {
-  if (key == NULL) return;
+  if(key == NULL) return;
 
-  if (strcmp(value, "") == 0) {
+  if(strcmp(value, "") == 0) {
     /* If a value is specified as NULL but the current value is not, delete
      * the pref. This is assumed to be the way the user will change such a
      * pref.
      */
-    if (*globalVar != NULL) {
+    if(*globalVar != NULL) {
       free (*globalVar);
       *globalVar = NULL;
     }
 	
 	 *globalVar = strdup(value);
-     if (savePref) delPrefsValue (key);
+     if(savePref) delPrefsValue (key);
   }
   else {
-    if (savePref) {
+    if(savePref) {
 
       if((strcmp(key, NTOP_PREF_DEVICES) == 0)
 	 && (*globalVar && (*globalVar[0] != '\0'))) {
@@ -1028,15 +1075,15 @@ void processStrPref(char *key, char *value, char **globalVar, bool savePref)
 
 	safe_snprintf(__FILE__, __LINE__, tmpValue, sizeof(tmpValue),
 		      "%s,%s", *globalVar, value);
-	storePrefsValue (key, tmpValue);
+	storePrefsValue(key, tmpValue);
 	free(*globalVar);
 	*globalVar = strdup (tmpValue);
 	return;
       } else
-	storePrefsValue (key, value);
+	storePrefsValue(key, value);
     }
 
-    if (*globalVar)
+    if(*globalVar)
       free (*globalVar);
 
     if((value == NULL) || (value[0] == '\0'))
@@ -1057,9 +1104,8 @@ void processIntPref(char *key, char *value, int *globalVar, bool savePref)
   *globalVar = atoi(value);
 
   if(savePref) {
-    safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf),
-		  "%d", *globalVar);
-    storePrefsValue (key, buf);
+    safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%d", *globalVar);
+    storePrefsValue(key, buf);
   }
 }
 
@@ -1069,12 +1115,12 @@ void processBoolPref(char *key, bool value, bool *globalVar, bool savePref)
 {
   char buf[512];
 
-  if (key == NULL) return;
+  if(key == NULL) return;
 
-  if (savePref) {
+  if(savePref) {
     safe_snprintf (__FILE__, __LINE__, buf, sizeof(buf),
 		   "%d", value);
-    storePrefsValue (key, buf);
+    storePrefsValue(key, buf);
   }
 
   *globalVar = value;
@@ -1094,7 +1140,9 @@ static bool value2bool(char* value) {
 bool processNtopPref(char *key, char *value, bool savePref, UserPref *pref) {
   bool startCap = FALSE;
   char buf[16], *tmpStr = NULL;
+#ifdef ENABLE_FC
   int tmpInt;
+#endif
 
   if(value == NULL) value = ""; /* Safer */
 
@@ -1126,7 +1174,7 @@ bool processNtopPref(char *key, char *value, bool savePref, UserPref *pref) {
 	/* DS: Search for : to find xxx.xxx.xxx.xxx:port */
 	/* This code is to be able to bind to a particular interface */
 	if(savePref) {
-	  storePrefsValue (key, value);
+	  storePrefsValue(key, value);
 	}
 	*pref->webAddr = '\0';
 	pref->webPort = atoi(pref->webAddr+1);
@@ -1156,9 +1204,9 @@ bool processNtopPref(char *key, char *value, bool savePref, UserPref *pref) {
       if((pref->sslAddr = strchr(tmpStr,':'))) {
 	/* DS: Search for : to find xxx.xxx.xxx.xxx:port */
 	/* This code is to be able to bind to a particular interface */
-	if(savePref) {
-	  storePrefsValue (key, value);
-	}
+	if(savePref)
+	  storePrefsValue(key, value);
+	
 	*pref->sslAddr = '\0';
 	pref->sslPort = atoi(pref->sslAddr+1);
 	pref->sslAddr = value;
@@ -1217,6 +1265,7 @@ bool processNtopPref(char *key, char *value, bool savePref, UserPref *pref) {
     }
     processIntPref(NTOP_PREF_MAXLINES, value, &pref->maxNumLines,
 		    savePref);
+#ifdef ENABLE_FC
   } else if(strcmp(key, NTOP_PREF_PRINT_FCORIP) == 0) {
     tmpInt = atoi (value);
     if(tmpInt == NTOP_PREF_VALUE_PRINT_IPONLY) {
@@ -1231,6 +1280,7 @@ bool processNtopPref(char *key, char *value, bool savePref, UserPref *pref) {
   } else if(strcmp(key, NTOP_PREF_NO_INVLUN) == 0) {
     processBoolPref(NTOP_PREF_NO_INVLUN, value2bool(value),
 		     &pref->noInvalidLunDisplay, savePref);
+#endif
   } else if(strcmp(key, NTOP_PREF_W3C) == 0) {
     processBoolPref(NTOP_PREF_W3C, value2bool(value), &pref->w3c, savePref);
   } else if(strcmp(key, NTOP_PREF_IPV4V6) == 0) {
@@ -1243,7 +1293,7 @@ bool processNtopPref(char *key, char *value, bool savePref, UserPref *pref) {
       free (tmpStr);      /* alloc'd in processStrPref() */
     }
   } else if(strcmp(key, NTOP_PREF_NUMERIC_IP) == 0) {
-    processBoolPref(NTOP_PREF_NUMERIC_IP, value2bool(value), &pref->numericFlag,
+    processIntPref(NTOP_PREF_NUMERIC_IP, value, (int*)&pref->numericFlag,
 		     savePref);
   } else if(strcmp(key, NTOP_PREF_PROTOSPECS) == 0) {
     processStrPref(NTOP_PREF_PROTOSPECS, value, &pref->protoSpecs,
@@ -1252,9 +1302,11 @@ bool processNtopPref(char *key, char *value, bool savePref, UserPref *pref) {
     processStrPref(NTOP_PREF_P3PCP, value, &pref->P3Pcp, savePref);
   } else if(strcmp(key, NTOP_PREF_P3PURI) == 0) {
     processStrPref(NTOP_PREF_P3PURI, value, &pref->P3Puri, savePref);
+#ifdef ENABLE_FC
   } else if(strcmp(key, NTOP_PREF_WWN_MAP) == 0) {
     processStrPref(NTOP_PREF_WWN_MAP, value, &pref->fcNSCacheFile,
 		    savePref);
+#endif
   } else if(strcmp(key, NTOP_PREF_MAXHASH) == 0) {
     if(value == NULL) {
       safe_snprintf (__FILE__, __LINE__, buf, sizeof(buf), "%d",
@@ -1313,12 +1365,6 @@ bool processNtopPref(char *key, char *value, bool savePref, UserPref *pref) {
     processStrPref(NTOP_PREF_PCAP_LOGBASE, value,
 		    &pref->pcapLogBasePath, savePref);
   }
-#ifdef MAKE_WITH_SSLWATCHDOG_RUNTIME
-  else if(strcmp(key, NTOP_PREF_USE_SSLWATCH) == 0) {
-    processBoolPref(NTOP_PREF_USE_SSLWATCH, value2bool(value),
-		     &pref->useSSLwatchdog, savePref);
-  }
-#endif
   else if(strcmp(key, NTOP_PREF_DBG_MODE) == 0) {
     processBoolPref(NTOP_PREF_DBG_MODE, value2bool(value), &pref->debugMode,
 		     savePref);
@@ -1358,12 +1404,6 @@ bool processNtopPref(char *key, char *value, bool savePref, UserPref *pref) {
     processBoolPref(NTOP_PREF_NO_MUTEX_EXTRA, value2bool(value),
 		     &pref->disableMutexExtraInfo, savePref);
   }
-#ifdef MAKE_WITH_SCHED_YIELD
-  else if(strcmp(key, NTOP_PREF_NO_SCHEDYLD) == 0) {
-    processBoolPref(NTOP_PREF_NO_SCHEDYLD, value2bool(value),
-		     &pref->disableSchedYield, savePref);
-  }
-#endif
   else if(strncmp (key, "ntop.", strlen ("ntop.")) == 0) {
     traceEvent(CONST_TRACE_WARNING, "Unknown preference: %s, value = %s\n",
 		key, (value == NULL) ? "(null)" : value);
@@ -1418,20 +1458,15 @@ void initUserPrefs(UserPref *pref) {
 #else
   pref->pcapLogBasePath = strdup(CFG_DBFILE_DIR);
 #endif
+#ifdef ENABLE_FC
   pref->fcNSCacheFile   = DEFAULT_NTOP_FCNS_FILE;
+#endif
   /* note that by default ntop will merge network interfaces */
   pref->mapperURL = DEFAULT_NTOP_MAPPER_URL;
 // #ifdef HAVE_OPENSSL
 //   pref->sslAddr = DEFAULT_NTOP_WEB_ADDR;
 //   pref->sslPort = DEFAULT_NTOP_WEB_PORT+1;
 // #endif
-#ifdef MAKE_WITH_SSLWATCHDOG_RUNTIME
-   pref->useSSLwatchdog = 0;
-#endif
-
-#ifdef MAKE_WITH_SCHED_YIELD
-   pref->disableSchedYield = DEFAULT_NTOP_SCHED_YIELD;
-#endif
 
    pref->w3c    = DEFAULT_NTOP_W3C;
    pref->P3Pcp  = DEFAULT_NTOP_P3PCP;
