@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2010 Luca Deri <deri@ntop.org>
  *
  *  			    http://www.ntop.org/
  *
@@ -53,9 +53,8 @@ void handleBootp(HostTraffic *srcHost,
   BootProtocol bootProto;
   u_int len;
   int rc;
-  char savechar; /* Courtesy of
-		    Axel Thimm <Axel.Thimm+ntop@physik.fu-berlin.de>
-		 */
+  /* Courtesy of Axel Thimm <Axel.Thimm+ntop@physik.fu-berlin.de> */
+  char savechar;
 
   if((!myGlobals.runningPref.enablePacketDecoding)
      || (packetData == NULL) /* packet too short ? */
@@ -66,7 +65,7 @@ void handleBootp(HostTraffic *srcHost,
 
   switch(sport) {
   case 67: /* BOOTP/DHCP: server -> client*/
-    FD_SET(FLAG_HOST_TYPE_SVC_DHCP_SERVER, &srcHost->flags);
+    setHostFlag(FLAG_HOST_TYPE_SVC_DHCP_SERVER, srcHost);
 
 #ifdef DHCP_DEBUG
     traceEvent(CONST_TRACE_INFO, "%s:%d->%s:%d",
@@ -144,7 +143,7 @@ void handleBootp(HostTraffic *srcHost,
 	      allocHostTrafficCounterMemory(srcHost, protocolInfo, sizeof(ProtocolInfo));
 	      allocHostTrafficCounterMemory(srcHost, protocolInfo->dhcpStats, sizeof(DHCPStats));
 
-	      FD_SET(FLAG_HOST_TYPE_SVC_DHCP_CLIENT, &realDstHost->flags);
+	      setHostFlag(FLAG_HOST_TYPE_SVC_DHCP_CLIENT, realDstHost);
 	      realDstHost->protocolInfo->dhcpStats->assignTime = myGlobals.actTime;
 	      realDstHost->protocolInfo->dhcpStats->dhcpServerIpAddress.s_addr = srcHost->hostIp4Address.s_addr;
 	      realDstHost->protocolInfo->dhcpStats->dhcpServerIpAddress.s_addr = srcHost->hostIp4Address.s_addr;
@@ -161,18 +160,18 @@ void handleBootp(HostTraffic *srcHost,
 		strncpy(realDstHost->hostNumIpAddress,
 			_intoa(realDstHost->hostIp4Address, buf, sizeof(buf)),
 			sizeof(realDstHost->hostNumIpAddress));
-		if(myGlobals.runningPref.numericFlag == 0) {
-			realDstHost->hostIpAddress.hostFamily = AF_INET;
-			ipaddr2str(realDstHost->hostIpAddress, 1);
+		if(myGlobals.runningPref.numericFlag != noDnsResolution) {
+		  realDstHost->hostIpAddress.hostFamily = AF_INET;
+		  ipaddr2str(realDstHost, realDstHost->hostIpAddress, realDstHost->vlanId, actualDeviceId);
 		}
                 if (realDstHost->dnsDomainValue != NULL) free(realDstHost->dnsDomainValue);
 		realDstHost->dnsDomainValue = NULL;
                 if (realDstHost->dnsTLDValue != NULL) free(realDstHost->dnsTLDValue);
 		realDstHost->dnsTLDValue = NULL;
 		if(isBroadcastAddress(&realDstHost->hostIpAddress, NULL, NULL))
-		  FD_SET(FLAG_BROADCAST_HOST, &realDstHost->flags);
+		  setHostFlag(FLAG_BROADCAST_HOST, realDstHost);
 		else
-		  FD_CLR(FLAG_BROADCAST_HOST, &realDstHost->flags);
+		  clearHostFlag(FLAG_BROADCAST_HOST, realDstHost);
 	      }
 
 	      while(idx < 64 /* Length of the BOOTP vendor-specific area */) {
@@ -204,7 +203,7 @@ void handleBootp(HostTraffic *srcHost,
 		  trafficHost = findHostByNumIP(addr, srcHost->vlanId, actualDeviceId);
 		  if(trafficHost != NULL) {
 		    incrementUsageCounter(&realDstHost->contactedRouters, trafficHost, actualDeviceId);
-		    FD_SET(FLAG_GATEWAY_HOST, &trafficHost->flags);
+		    setHostFlag(FLAG_GATEWAY_HOST, trafficHost);
 		  }
 
 		  /* *************** */
@@ -306,7 +305,7 @@ void handleBootp(HostTraffic *srcHost,
 
 		  trafficHost = findHostByNumIP(addr, srcHost->vlanId, actualDeviceId);
 		  if(trafficHost != NULL) {
-		    FD_SET(FLAG_HOST_TYPE_SVC_WINS, &trafficHost->flags);
+		    setHostFlag(FLAG_HOST_TYPE_SVC_WINS, trafficHost);
 		  }
 
 		  /* *************** */
@@ -445,8 +444,8 @@ void handleBootp(HostTraffic *srcHost,
 	For more info see http://www.dhcp.org/
       */
 
-      FD_SET(FLAG_HOST_TYPE_SVC_DHCP_CLIENT, &srcHost->flags);
-      FD_SET(FLAG_HOST_TYPE_SVC_DHCP_SERVER, &dstHost->flags);
+      setHostFlag(FLAG_HOST_TYPE_SVC_DHCP_CLIENT, srcHost);
+      setHostFlag(FLAG_HOST_TYPE_SVC_DHCP_SERVER, dstHost);
 
       if(packetDataLength >= sizeof(BootProtocol))
 	len = sizeof(BootProtocol);
@@ -604,8 +603,6 @@ u_int16_t processDNSPacket(HostTraffic *srcHost, u_short sport,
 
   memset(tmpBuf, 0, sizeof(tmpBuf)); /* quiet Valgrind */
 
-  if(myGlobals.dnsCacheFile == NULL) return(-1); /* ntop is quitting... */
-
   if((!myGlobals.runningPref.enablePacketDecoding)
      ||(packetData == NULL) /* packet too short ? */)
     return(transactionId);
@@ -670,6 +667,9 @@ u_int16_t processDNSPacket(HostTraffic *srcHost, u_short sport,
       data_data.dptr = (void*)&addrStore;
       data_data.dsize = sizeof(addrStore);
 
+      // FIX
+      //updateHostNameInfo(elem->addr, storedAddress.symAddress, storedAddress.symAddressType);  	  
+
 #ifdef DNS_SNIFF_DEBUG
       traceEvent(CONST_TRACE_INFO, "DNS_SNIFF_DEBUG: Sniffed DNS response: %s(%d) = %s(t=%d)",
                  key_data.dptr, key_data.dsize,
@@ -677,8 +677,6 @@ u_int16_t processDNSPacket(HostTraffic *srcHost, u_short sport,
                  ((StoredAddress *)data_data.dptr)->recordCreationTime);
 #endif
 
-      if(myGlobals.dnsCacheFile == NULL) return(-1); /* ntop is quitting... */
-      gdbm_store(myGlobals.dnsCacheFile, key_data, data_data, GDBM_REPLACE);
       myGlobals.dnsSniffStoredInCache++;
     }
   }
@@ -855,7 +853,7 @@ void handleNetbios(HostTraffic *srcHost,
 			free(srcHost->nonIPTraffic->nbDescr);
 
 		      if(tmpBuffer[17] == 0x0F)
-			FD_SET(FLAG_HOST_TYPE_MASTER_BROWSER, &srcHost->flags);
+			setHostFlag(FLAG_HOST_TYPE_MASTER_BROWSER, srcHost);
 
 		      srcHost->nonIPTraffic->nbDescr = strdup(&tmpBuffer[49]);
 #ifdef DEBUG
@@ -914,9 +912,7 @@ void handleNetbios(HostTraffic *srcHost,
 	    char buffer[64];
 
 	    safe_snprintf(__FILE__, __LINE__, buffer, sizeof(buffer), ":%s", &data[45]);
-	    accessAddrResMutex("makeHostLink");
 	    srcHost->fingerprint = strdup(buffer);
-	    releaseAddrResMutex();
 	  }
 	} else /* dport == 139 */ {
 	  /* Request */
@@ -946,9 +942,7 @@ void handleNetbios(HostTraffic *srcHost,
 	    char buffer[64];
 
 	    safe_snprintf(__FILE__, __LINE__, buffer, sizeof(buffer), ":%s", &data[i]);
-	    accessAddrResMutex("makeHostLink");
 	    srcHost->fingerprint = strdup(buffer);
-	    releaseAddrResMutex();
 	  }
 	}
       }
