@@ -2,7 +2,7 @@
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *                          http://www.ntop.org
  *
- *           Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
+ *           Copyright (C) 1998-2010 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -19,6 +19,20 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ * In addition, as a special exception, the copyright holders give
+ * permission to link the code of portions of this program with the
+ * OpenSSL library under certain conditions as described in each
+ * individual source file, and distribute linked combinations
+ * including the two.
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * file(s) with this exception, you may extend this exception to your
+ * version of the file(s), but you are not obligated to do so.  If you
+ * do not wish to do so, delete this exception statement from your
+ * version.  If you delete this exception statement from all source
+ * files in the program, then also delete it here.
+ *
  */
 
 
@@ -148,9 +162,13 @@ int init_ssl(void) {
 
 
   SSL_load_error_strings();
-  SSLeay_add_ssl_algorithms();
+  SSL_library_init();
+  OpenSSL_add_ssl_algorithms();
+#if (OPENSSL_VERSION_NUMBER < 0x00905100l)
+  needs_openssl_095_or_later();
+#endif
   meth = SSLv23_server_method();
-  myGlobals.ctx = SSL_CTX_new (meth);
+  myGlobals.ctx = SSL_CTX_new(meth);
   if (!myGlobals.ctx) {
     ntop_ssl_error_report("ssl_init-server_method");
     return(2);
