@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2010 Luca Deri <deri@ntop.org>
  *
  *		 	    http://www.ntop.org/
  *
@@ -28,8 +28,6 @@ void termIPServices(void) {
   int i;
   ProtocolsList *proto = myGlobals.ipProtosList;
 
-  evdns_shutdown(1);
-  
   for(i=0; i<myGlobals.numActServices; i++) {
     if(myGlobals.udpSvc[i] != NULL) {
       free(myGlobals.udpSvc[i]->name);
@@ -92,7 +90,6 @@ void termIPSessions(void) {
 /* ************************************************ */
 
 void termGdbm() {
-  if(myGlobals.dnsCacheFile)     { gdbm_close(myGlobals.dnsCacheFile); myGlobals.dnsCacheFile = NULL;         }
   if(myGlobals.pwFile)           { gdbm_close(myGlobals.pwFile); myGlobals.pwFile = NULL;                     }
   if(myGlobals.prefsFile)        { gdbm_close(myGlobals.prefsFile); myGlobals.prefsFile = NULL;               }
 
