@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2009 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2010 Luca Deri <deri@ntop.org>
  *
  *  			    http://www.ntop.org/
  *
@@ -20,6 +20,8 @@
 
 #include "ntop.h"
 
+static u_int8_t are_communities_defined = 0;
+
 void updateFcTrafficMatrix(HostTraffic *srcHost, HostTraffic *dstHost,
                            TrafficCounter length, int actualDeviceId);
 
@@ -549,8 +551,10 @@ int isMatrixHost(HostTraffic *host, int actualDeviceId) {
 
 unsigned int matrixHostHash(HostTraffic *host, int actualDeviceId, int rehash) {
   unsigned long hash = 0;
+#ifdef ENABLE_FC
   char tmpBuf[80], *str;
   int c;
+#endif
 
   if(myGlobals.device[actualDeviceId].numHosts  == 0) return(0);
 
@@ -562,6 +566,7 @@ unsigned int matrixHostHash(HostTraffic *host, int actualDeviceId, int rehash) {
       hash = *(u_int32_t *)&host->hostIp6Address.s6_addr[0];
 #endif
   }
+#ifdef ENABLE_FC
   else {
     if (host->fcCounters->vsanId) {
       hash ^= host->fcCounters->vsanId;
@@ -591,7 +596,8 @@ unsigned int matrixHostHash(HostTraffic *host, int actualDeviceId, int rehash) {
       hash += c;
     }
   }
-    
+#endif
+ 
   return((unsigned int)(hash) % myGlobals.device[actualDeviceId].numHosts);
 }
 
@@ -629,9 +635,10 @@ void updateTrafficMatrix(HostTraffic *srcHost,
 
 /* ************************ */
 
+#ifdef ENABLE_FC
 void updateFcTrafficMatrix(HostTraffic *srcHost,
-			 HostTraffic *dstHost,
-			 TrafficCounter length, 
+			   HostTraffic *dstHost,
+			   TrafficCounter length, 
 			   int actualDeviceId) {
   unsigned int a, b, id;
 
@@ -687,6 +694,7 @@ void updateFcTrafficMatrix(HostTraffic *srcHost,
   incrementTrafficCounter(&myGlobals.device[actualDeviceId].fcTrafficMatrix[id]->bytesRcvd, length.value);
   incrementTrafficCounter(&myGlobals.device[actualDeviceId].fcTrafficMatrix[id]->pktsRcvd, 1);
 }
+#endif
 
 /* ************************ */
 
@@ -730,40 +738,67 @@ int isInitialFtpData(char* packetData) {
 
 /* ********************************** */
 
-char* findHostCommunity(u_int32_t host_ip, char *buf, u_short buf_len) {
+void checkCommunities() {
   datum key, nextkey;
   int len = strlen(COMMUNITY_PREFIX);
 
-
   key = gdbm_firstkey(myGlobals.prefsFile);
+
   while (key.dptr) {
-    char val[256], localAddresses[2048], *communityName;
-    NetworkStats localNetworks[MAX_NUM_NETWORKS]; /* [0]=network, [1]=mask, [2]=broadcast, [3]=mask_v6 */
-    u_short numLocalNetworks = 0, i;
-    
+    char val[256];
+
     if((fetchPrefsValue(key.dptr, val, sizeof(val)) == 0)
        && (!strncmp(key.dptr, COMMUNITY_PREFIX, len))) {
-      localAddresses[0] = '\0';
-      communityName = (char*)&key.dptr[len];
-
-      handleAddressLists(val, localNetworks, &numLocalNetworks,
-			 localAddresses, sizeof(localAddresses),
-			 CONST_HANDLEADDRESSLISTS_COMMUNITY);
-
-      // traceEvent(CONST_TRACE_WARNING, "--> Community %s has %d entries", communityName, numLocalNetworks);
-      for(i=0; i<numLocalNetworks; i++) {
-	if((host_ip & localNetworks[i].address[1]) == localNetworks[i].address[0]) {
-	  //traceEvent(CONST_TRACE_WARNING, "--> Found community %s [%d]", communityName, numLocalNetworks);
-	  snprintf(buf, buf_len, "%s", communityName);
-	  return(buf);
-	}
-      }
+      free(key.dptr);
+      are_communities_defined = 1;
+      return;
     }
 
     nextkey = gdbm_nextkey(myGlobals.prefsFile, key);
     free (key.dptr);
     key = nextkey;
   }
+  
+  are_communities_defined = 0;
+}
+
+/* ********************************** */
+
+char* findHostCommunity(u_int32_t host_ip, char *buf, u_short buf_len) {
+  if(are_communities_defined) {
+    datum key, nextkey;
+    int len = strlen(COMMUNITY_PREFIX);
+
+    key = gdbm_firstkey(myGlobals.prefsFile);
+    while (key.dptr) {
+      char val[256], localAddresses[2048], *communityName;
+      NetworkStats localNetworks[MAX_NUM_NETWORKS]; /* [0]=network, [1]=mask, [2]=broadcast, [3]=mask_v6 */
+      u_short numLocalNetworks = 0, i;
+    
+      if((fetchPrefsValue(key.dptr, val, sizeof(val)) == 0)
+	 && (!strncmp(key.dptr, COMMUNITY_PREFIX, len))) {
+	localAddresses[0] = '\0';
+	communityName = (char*)&key.dptr[len];
+
+	handleAddressLists(val, localNetworks, &numLocalNetworks,
+			   localAddresses, sizeof(localAddresses),
+			   CONST_HANDLEADDRESSLISTS_COMMUNITIES);
+
+	// traceEvent(CONST_TRACE_WARNING, "--> Community %s has %d entries", communityName, numLocalNetworks);
+	for(i=0; i<numLocalNetworks; i++) {
+	  if((host_ip & localNetworks[i].address[1]) == localNetworks[i].address[0]) {
+	    //traceEvent(CONST_TRACE_WARNING, "--> Found community %s [%d]", communityName, numLocalNetworks);
+	    snprintf(buf, buf_len, "%s", communityName);
+	    return(buf);
+	  }
+	}
+      }
+
+      nextkey = gdbm_nextkey(myGlobals.prefsFile, key);
+      free (key.dptr);
+      key = nextkey;
+    }
+  }
 
   return(NULL);
 }
